@@ -1,18 +1,36 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015 Christoph Reiter
+# Copyright 2015-2016 Christoph Reiter
 #
-# This software and accompanying documentation, if any, may be freely
-# used, distributed, and/or modified, in any form and for any purpose,
-# as long as this notice is preserved. There is no warranty, either
-# express or implied, for this software.
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 from __future__ import absolute_import
 
 import os
 import sys
-import urllib
 
-from distutils.core import Command
+try:
+    from urllib import pathname2url
+except:
+    from urllib.request import pathname2url
+
+from .util import Command
 
 
 class coverage_cmd(Command):
@@ -66,6 +84,6 @@ class coverage_cmd(Command):
 
         dest = os.path.abspath(self.options["directory"])
         index = os.path.join(dest, "index.html")
-        index_url = urllib.pathname2url(index)
+        index_url = pathname2url(index)
 
         print("Coverage summary: file://%s" % index_url)
