@@ -1,10 +1,24 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014 Christoph Reiter
+# Copyright 2015-2017 Christoph Reiter
 #
-# This software and accompanying documentation, if any, may be freely
-# used, distributed, and/or modified, in any form and for any purpose,
-# as long as this notice is preserved. There is no warranty, either
-# express or implied, for this software.
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 """So we don't have to touch intltool directly
 (and maybe can get rid of it one day)
@@ -19,8 +33,46 @@ class GettextError(Exception):
     pass
 
 
-# pgettext isn't included by default for Python
-XGETTEXT_ARGS = "--keyword=C_:1c,2 --keyword=npgettext:1c,2,3"
+def _get_xgettext_args():
+    # pgettext isn't included by default for Python for example
+    EXTRA_KEYWORDS = {
+        "_": "",
+        "N_": "",
+        "C_": "1c,2",
+        "NC_": "1c,2",
+        "Q_": "",
+        "pgettext": "1c,2",
+        "npgettext": "1c,2,3",
+        "numeric_phrase": "1,2",
+        "dgettext": "2",
+        "ngettext": "1,2",
+        "dngettext": "2,3",
+    }
+
+    # The lone -k disables default xgettext keywords
+    args = ["-k"]
+
+    # There are still some keywords defined by intltool-update which we can't
+    # change, but they shouldn't conflict with anything.
+
+    for name, spec in EXTRA_KEYWORDS.items():
+        if spec:
+            args.append("--keyword=%s:%s" % (name, spec))
+        else:
+            args.append("--keyword=%s" % name)
+
+    return args
+
+XGETTEXT_ARGS = " ".join(_get_xgettext_args())
+
+
+def intltool(*args):
+    command = args[0]
+    args = args[1:]
+    if os.name == "nt":
+        return ["perl", "/usr/bin/intltool-%s" % command] + list(args)
+    else:
+        return ["intltool-%s" % command] + list(args)
 
 
 def update_pot(po_dir, package):
@@ -35,9 +87,9 @@ def update_pot(po_dir, package):
     try:
         os.environ["XGETTEXT_ARGS"] = XGETTEXT_ARGS
         with open(os.devnull, 'wb') as devnull:
-            subprocess.check_call(["intltool-update", "--pot",
-                                   "--gettext-package", package],
-                                   stderr=devnull, stdout=devnull)
+            subprocess.check_call(
+                intltool("update", "--pot", "--gettext-package", package),
+                stderr=devnull, stdout=devnull)
     except subprocess.CalledProcessError as e:
         raise GettextError(e)
     finally:
@@ -59,9 +111,8 @@ def update_po(po_dir, package, lang_code, output_file=None):
     os.chdir(po_dir)
     try:
         os.environ["XGETTEXT_ARGS"] = XGETTEXT_ARGS
-        args = ["intltool-update", "--dist",
-                "--gettext-package", package,
-                lang_code]
+        args = intltool(
+            "update", "--dist", "--gettext-package", package, lang_code)
         if output_file is not None:
             args.extend(["--output-file", output_file])
         with open(os.devnull, 'wb') as devnull:
@@ -127,8 +178,7 @@ def get_missing(po_dir, package):
     try:
         with open(os.devnull, 'wb') as devnull:
             subprocess.check_call(
-                ["intltool-update", "--maintain",
-                 "--gettext-package", package],
+                intltool("update", "--maintain", "--gettext-package", package),
                 stderr=devnull, stdout=devnull)
     except subprocess.CalledProcessError as e:
         raise GettextError(e)
@@ -164,7 +214,7 @@ def check_version():
     Tries to include a helpful error message..
     """
 
-    if find_executable("intltool-update") is None:
+    if os.name != "nt" and find_executable("intltool-update") is None:
         raise GettextError("intltool-update missing")
 
     if find_executable("xgettext") is None:
