@@ -1,10 +1,25 @@
 # -*- coding: utf-8 -*-
 # Copyright 2007 Joe Wreschnig
+#           2009-2010,2012-2016 Christoph Reiter
 #
-# This software and accompanying documentation, if any, may be freely
-# used, distributed, and/or modified, in any form and for any purpose,
-# as long as this notice is preserved. There is no warranty, either
-# express or implied, for this software.
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 """translation support
 
@@ -16,12 +31,11 @@ import os
 import glob
 from subprocess import Popen, PIPE
 from tempfile import mkstemp
-
 from distutils.dep_util import newer
 from distutils.spawn import find_executable
-from distutils.core import Command
 from distutils.errors import DistutilsOptionError
 
+from .util import Command
 from . import gettextutil
 
 
@@ -161,13 +175,13 @@ def strip_pot_date(path):
     done = False
     lines = []
     for line in open(path, "rb"):
-        if not done and line.startswith('"POT-Creation-Date:'):
+        if not done and line.startswith(b'"POT-Creation-Date:'):
             done = True
             continue
         lines.append(line)
 
     with open(path, "wb") as h:
-        h.write("".join(lines))
+        h.write(b"".join(lines))
 
 
 class build_mo(Command):
