@@ -1,15 +1,31 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007 Joe Wreschnig
+# Copyright 2007-2008 Joe Wreschnig
+#           2009,2012-2016 Christoph Reiter
 #
-# This software and accompanying documentation, if any, may be freely
-# used, distributed, and/or modified, in any form and for any purpose,
-# as long as this notice is preserved. There is no warranty, either
-# express or implied, for this software.
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 import os
-
 from distutils.dep_util import newer
-from distutils.core import Command
+
+from .util import Command
+from .gettextutil import intltool
 
 
 class build_shortcuts(Command):
@@ -43,9 +59,9 @@ class build_shortcuts(Command):
                 fullpath = os.path.join(basepath, os.path.basename(shortcut))
                 self.__check_po()
                 if newer(shortcut + ".in", fullpath):
-                    self.spawn(["intltool-merge",
-                                "-d", self.po_directory,
-                                shortcut + ".in", fullpath])
+                    self.spawn(
+                        intltool("merge", "-d", self.po_directory,
+                                 shortcut + ".in", fullpath))
             else:
                 self.copy_file(shortcut, os.path.join(basepath, shortcut))
 
