@@ -1,10 +1,24 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014 Christoph Reiter
+# Copyright 2014-2016 Christoph Reiter
 #
-# This software and accompanying documentation, if any, may be freely
-# used, distributed, and/or modified, in any form and for any purpose,
-# as long as this notice is preserved. There is no warranty, either
-# express or implied, for this software.
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 from __future__ import absolute_import
 
@@ -12,10 +26,9 @@ import os
 import sys
 import subprocess
 import tarfile
-
-from distutils.core import Command
 from distutils import dir_util
-from distutils.command.sdist import sdist
+
+from .util import Command, get_dist_class
 
 
 class test_cmd(Command):
@@ -28,7 +41,6 @@ class test_cmd(Command):
         ("exitfirst", "x", "stop after first failing test"),
         ("no-network", "n", "skip tests requiring a network connection"),
     ]
-    use_colors = sys.stderr.isatty() and os.name != "nt"
 
     def initialize_options(self):
         self.to_run = []
@@ -47,39 +59,19 @@ class test_cmd(Command):
         self.exitfirst = bool(self.exitfirst)
         self.no_network = bool(self.no_network)
 
-    @classmethod
-    def _red(cls, text):
-        from quodlibet.util.dprint import Colorise
-        return Colorise.red(text) if cls.use_colors else text
-
     def run(self):
-        mods = sys.modules.keys()
-        if "gi" in mods:
-            raise SystemExit("E: setup.py shouldn't depend on gi")
-
         import tests
 
-        main = False
-        if not self.suite or self.all:
-            main = True
-
-        subdirs = []
+        suite = self.suite
         if self.all:
-            test_path = tests.__path__[0]
-            for entry in os.listdir(test_path):
-                if os.path.isdir(os.path.join(test_path, entry)):
-                    subdirs.append(entry)
-        elif self.suite:
-            subdirs.append(self.suite)
+            suite = None
 
-        failures, errors, all_ = tests.unit(
-            self.to_run, main=main, subdirs=subdirs,
-            strict=self.strict, stop_first=self.exitfirst,
-            no_network=self.no_network)
-        if failures or errors:
-            raise SystemExit(self._red("%d test failure(s) and "
-                                       "%d test error(s) for %d tests."
-                             % (failures, errors, all_)))
+        status = tests.unit(run=self.to_run, suite=suite,
+                            strict=self.strict, exitfirst=self.exitfirst,
+                            network=(not self.no_network or self.all),
+                            quality=self.all)
+        if status != 0:
+            raise SystemExit(status)
 
 
 class quality_cmd(Command):
@@ -93,10 +85,14 @@ class quality_cmd(Command):
         pass
 
     def run(self):
-        cmd = self.reinitialize_command("test")
-        cmd.suite = "quality"
-        cmd.ensure_finalized()
-        cmd.run()
+        import tests
+
+        status = tests.unit(suite="quality", quality=True)
+        if status != 0:
+            raise SystemExit(status)
+
+
+sdist = get_dist_class("sdist")
 
 
 class distcheck_cmd(sdist):
@@ -105,6 +101,8 @@ class distcheck_cmd(sdist):
     def _check_manifest(self):
         assert self.get_archive_files()
 
+        skip = os.path.join("quodlibet", "optpackages", "")
+
         # make sure MANIFEST.in includes all tracked files
         if subprocess.call(["git", "status"],
                            stdout=subprocess.PIPE,
@@ -120,6 +118,8 @@ class distcheck_cmd(sdist):
             assert process.returncode == 0
 
             tracked_files = out.splitlines()
+            tracked_files = [
+                p for p in tracked_files if not p.startswith(skip)]
 
             diff = set(tracked_files) - set(included_files)
             assert not diff, (
@@ -149,7 +149,7 @@ class distcheck_cmd(sdist):
         self.spawn([sys.executable, "setup.py", "build"])
         self.spawn([sys.executable, "setup.py", "build_sphinx"])
         self.spawn([sys.executable, "setup.py", "install",
-                    "--prefix", "../prefix", "--record", "../log.txt"])
+                    "--root", "../prefix", "--record", "../log.txt"])
         os.chdir(old_pwd)
 
     def run(self):
