@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # Copyright 2012,2013 Christoph Reiter
 #
@@ -6,15 +6,10 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-"""A simple command line tagger"""
-
 import sys
 
-import quodlibet
 from quodlibet.operon import main
-from quodlibet import util
 
 
 if __name__ == "__main__":
-    quodlibet.init_cli()
-    sys.exit(main(util.argv))
+    sys.exit(main())
