@@ -13,12 +13,14 @@
 import os
 
 from gi.repository import Gtk, Gdk
+from senf import fsn2uri
 
 from quodlibet import config
 from quodlibet import formats
 from quodlibet import qltk
-
+from quodlibet import _
 from quodlibet.browsers import Browser
+from quodlibet.compat import listfilter
 from quodlibet.library import SongFileLibrary
 from quodlibet.qltk.filesel import MainDirectoryTree
 from quodlibet.qltk.songsmenu import SongsMenu
@@ -27,7 +29,7 @@ from quodlibet.qltk import Icons
 from quodlibet.util import copool
 from quodlibet.util.library import get_scan_dirs
 from quodlibet.util.dprint import print_d
-from quodlibet.util.path import normalize_path, uri_from_path
+from quodlibet.util.path import normalize_path
 from quodlibet.util import connect_obj
 
 
@@ -105,7 +107,7 @@ class FileSystem(Browser, Gtk.HBox):
         for songs in self.__find_songs(view.get_selection()):
             pass
         if tid == self.TARGET_QL:
-            cant_add = filter(lambda s: not s.can_add, songs)
+            cant_add = listfilter(lambda s: not s.can_add, songs)
             if cant_add:
                 qltk.ErrorMessage(
                     qltk.get_top_parent(self), _("Unable to copy songs"),
@@ -119,7 +121,7 @@ class FileSystem(Browser, Gtk.HBox):
             qltk.selection_set_songs(sel, songs)
         else:
             # External target (app) is delivered a list of URIS of songs
-            uris = list({uri_from_path(dir) for dir in dirs})
+            uris = list({fsn2uri(dir) for dir in dirs})
             print_d("Directories to drop: %s" % dirs)
             sel.set_uris(uris)
 
