@@ -1,27 +1,29 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014 Nick Boultbee
+# Copyright 2014-2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
 import os
+
 from gi.repository import Gtk
+from senf import uri2fsn, fsnative, fsn2text, path2fsn, bytes2fsn
+
 import quodlibet
+from quodlibet import _
 from quodlibet import formats, qltk
-from quodlibet.qltk.wlw import WaitLoadWindow
-from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk import Icons
+from quodlibet.qltk.getstring import GetStringDialog
+from quodlibet.qltk.wlw import WaitLoadWindow
 from quodlibet.util import escape
-from quodlibet.util.collection import Playlist
-from quodlibet.util.path import mkdir, fsdecode, is_fsnative
+from quodlibet.util.collection import FileBackedPlaylist
+from quodlibet.util.path import mkdir, uri_is_valid
 
 
 # Directory for playlist files
-from quodlibet.util.uri import URI
-
 PLAYLISTS = os.path.join(quodlibet.get_user_dir(), "playlists")
-assert is_fsnative(PLAYLISTS)
+assert isinstance(PLAYLISTS, fsnative)
 if not os.path.isdir(PLAYLISTS):
     mkdir(PLAYLISTS)
 
@@ -51,55 +53,56 @@ class GetPlaylistName(GetStringDialog):
 
 
 def parse_m3u(filename, library=None):
-    plname = fsdecode(os.path.basename(
-        os.path.splitext(filename)[0]))
+    plname = fsn2text(path2fsn(os.path.basename(
+        os.path.splitext(filename)[0])))
 
     filenames = []
 
     with open(filename, "rb") as h:
         for line in h:
             line = line.strip()
-            if line.startswith("#"):
+            if line.startswith(b"#"):
                 continue
             else:
-                filenames.append(line)
+                try:
+                    filenames.append(bytes2fsn(line, "utf-8"))
+                except ValueError:
+                    continue
     return __parse_playlist(plname, filename, filenames, library)
 
 
 def parse_pls(filename, name="", library=None):
-    plname = fsdecode(os.path.basename(
-        os.path.splitext(filename)[0]))
+    plname = fsn2text(path2fsn(os.path.basename(
+        os.path.splitext(filename)[0])))
 
     filenames = []
-    h = file(filename)
-    for line in h:
-        line = line.strip()
-        if not line.lower().startswith("file"):
-            continue
-        else:
-            try:
-                line = line[line.index("=") + 1:].strip()
-            except ValueError:
-                pass
+    with open(filename, "rb") as h:
+        for line in h:
+            line = line.strip()
+            if not line.lower().startswith(b"file"):
+                continue
             else:
-                filenames.append(line)
-    h.close()
+                try:
+                    line = line[line.index(b"=") + 1:].strip()
+                except ValueError:
+                    pass
+                else:
+                    try:
+                        filenames.append(bytes2fsn(line, "utf-8"))
+                    except ValueError:
+                        continue
     return __parse_playlist(plname, filename, filenames, library)
 
 
 def __parse_playlist(name, plfilename, files, library):
-    playlist = Playlist.new(PLAYLISTS, name, library=library)
+    playlist = FileBackedPlaylist.new(PLAYLISTS, name, library=library)
     songs = []
     win = WaitLoadWindow(
         None, len(files),
         _("Importing playlist.\n\n%(current)d/%(total)d songs added."))
     win.show()
     for i, filename in enumerate(files):
-        try:
-            uri = URI(filename)
-        except ValueError:
-            if os.name == "nt":
-                filename = filename.decode("utf-8", "replace")
+        if not uri_is_valid(filename):
             # Plain filename.
             filename = os.path.realpath(os.path.join(
                 os.path.dirname(plfilename), filename))
@@ -108,17 +111,19 @@ def __parse_playlist(name, plfilename, files, library):
             else:
                 songs.append(formats.MusicFile(filename))
         else:
-            if uri.scheme == "file":
+            try:
+                filename = uri2fsn(filename)
+            except ValueError:
+                # Who knows! Hand it off to GStreamer.
+                songs.append(formats.remote.RemoteFile(filename))
+            else:
                 # URI-encoded local filename.
                 filename = os.path.realpath(os.path.join(
-                    os.path.dirname(plfilename), uri.filename))
+                    os.path.dirname(plfilename), filename))
                 if library and filename in library:
                     songs.append(library[filename])
                 else:
                     songs.append(formats.MusicFile(filename))
-            else:
-                # Who knows! Hand it off to GStreamer.
-                songs.append(formats.remote.RemoteFile(uri))
         if win.step():
             break
     win.destroy()
