@@ -6,7 +6,10 @@
 # published by the Free Software Foundation
 
 import os
-import sys
+
+from senf import fsn2text, uri2fsn
+
+from quodlibet import C_, _
 from quodlibet.util.dprint import print_, print_e
 from quodlibet.remote import Remote, RemoteError
 
@@ -60,12 +63,12 @@ def control(command, arg=None, ignore_error=False):
         exit_(str(e), notify_startup=True)
     else:
         if response is not None:
-            print_(response, end="")
+            print_(response, end="", flush=True)
         exit_(notify_startup=True)
 
 
 def process_arguments(argv):
-    from quodlibet.util.path import uri_is_valid, uri_to_path
+    from quodlibet.util.path import uri_is_valid
     from quodlibet import util
     from quodlibet import const
 
@@ -73,9 +76,9 @@ def process_arguments(argv):
     controls = ["next", "previous", "play", "pause", "play-pause", "stop",
                 "hide-window", "show-window", "toggle-window",
                 "focus", "quit", "unfilter", "refresh", "force-previous"]
-    controls_opt = ["seek", "order", "repeat", "query", "volume", "filter",
-                    "set-rating", "set-browser", "open-browser", "random",
-                    "song-list", "queue", "stop-after"]
+    controls_opt = ["seek", "repeat", "query", "volume", "filter",
+                    "set-rating", "set-browser", "open-browser", "shuffle",
+                    "song-list", "queue", "stop-after", "random"]
 
     options = util.OptionParser(
         "Quod Libet", const.VERSION,
@@ -84,6 +87,7 @@ def process_arguments(argv):
 
     options.add("print-playing", help=_("Print the playing song and exit"))
     options.add("start-playing", help=_("Begin playing immediately"))
+    options.add("start-hidden", help=_("Don't show any windows on start"))
 
     for opt, help in [
         ("next", _("Jump to next song")),
@@ -115,8 +119,7 @@ def process_arguments(argv):
 
     for opt, help, arg in [
         ("seek", _("Seek within the playing song"), _("[+|-][HH:]MM:SS")),
-        ("order", _("Set or toggle the playback order"),
-            "[order]|toggle"),
+        ("shuffle", _("Set or toggle shuffle mode"), "[0|1|t"),
         ("repeat", _("Turn repeat off, on, or toggle it"), "0|1|t"),
         ("volume", _("Set the volume"), "(+|-|)0..100"),
         ("query", _("Search your audio library"), _("query")),
@@ -172,8 +175,7 @@ def process_arguments(argv):
             return True
 
     validators = {
-        "order": ["0", "1", "t", "toggle", "inorder", "shuffle",
-                  "weighted", "onesong"].__contains__,
+        "shuffle": ["0", "1", "t", "on", "off", "toggle"].__contains__,
         "repeat": ["0", "1", "t", "on", "off", "toggle"].__contains__,
         "volume": is_vol,
         "seek": is_time,
@@ -199,7 +201,7 @@ def process_arguments(argv):
         elif command in controls_opt:
             if command in validators and not validators[command](arg):
                 print_e(_("Invalid argument for '%s'.") % command)
-                print_e(_("Try %s --help.") % sys.argv[0])
+                print_e(_("Try %s --help.") % fsn2text(argv[0]))
                 exit_(True, notify_startup=True)
             else:
                 queue(command, arg)
@@ -217,7 +219,7 @@ def process_arguments(argv):
             queue("volume -")
         elif command == "enqueue" or command == "unqueue":
             try:
-                filename = uri_to_path(arg)
+                filename = uri2fsn(arg)
             except ValueError:
                 filename = arg
             queue(command, filename)
@@ -229,7 +231,7 @@ def process_arguments(argv):
                 queue("uri-received", arg)
             else:
                 try:
-                    filename = uri_to_path(arg)
+                    filename = uri2fsn(arg)
                 except ValueError:
                     filename = arg
                 filename = os.path.abspath(util.path.expanduser(arg))
@@ -245,6 +247,8 @@ def process_arguments(argv):
             queue(command)
         elif command == "start-playing":
             actions.append(command)
+        elif command == "start-hidden":
+            actions.append(command)
         elif command == "no-plugins":
             actions.append(command)
         elif command == "run":
