@@ -9,6 +9,20 @@ import sys
 import os
 
 
+# MSYS2 defines MSYSTEM which changes os.sep/os.path.sep for the mingw
+# Python build. Unset here and restart.. (does not work for py.test etc.)
+# XXX: do this here since it gets executed by all scripts
+if os.name == "nt" and "MSYSTEM" in os.environ:
+    import subprocess
+    del os.environ["MSYSTEM"]
+    argv = []
+    for arg in [sys.executable] + sys.argv:
+        if os.path.exists(arg):
+            arg = arg.replace("/", "\\")
+        argv.append(arg)
+    sys.exit(subprocess.call(argv))
+
+
 class Version(tuple):
     """Represent the version of a dependency as a tuple"""
 
@@ -42,16 +56,16 @@ class MinVersions(object):
     MUTAGEN = Version("Mutagen", 1, 32,
         message="Use the Quod Libet unstable PPAs/repos to get a newer "
                 "mutagen version.")
-    GTK = Version("GTK+", 3, 10)
-    PYGOBJECT = Version("PyGObject", 3, 12)
-    GSTREAMER = Version("GStreamer", 1, 0)
+    GTK = Version("GTK+", 3, 14)
+    PYGOBJECT = Version("PyGObject", 3, 14)
+    GSTREAMER = Version("GStreamer", 1, 4)
 
 
-VERSION_TUPLE = Version("", 3, 7, 1)
+VERSION_TUPLE = Version("", 3, 9, 1)
 VERSION = str(VERSION_TUPLE)
 
 # entry point for the user guide / wiki
-BRANCH_NAME = "quodlibet-3.7"
+BRANCH_NAME = "quodlibet-3.9"
 DOCS_BASE_URL = "https://quodlibet.readthedocs.org/en/%s"
 DOCS_LATEST = DOCS_BASE_URL % "latest"
 DOCS_BASE_URL %= BRANCH_NAME if BRANCH_NAME != "master" else "latest"
@@ -65,17 +79,9 @@ EMAIL = os.environ.get("EMAIL", "quodlibet@lists.sacredchao.net")
 # Displayed as registered / help email address
 SUPPORT_EMAIL = "quod-libet-development@googlegroups.com"
 
-MAIN_AUTHORS = u"""\
-Joe Wreschnig
-Michael Urman
-Iñigo Serna
-Steven Robertson
-Christoph Reiter
-Nick Boultbee""".split("\n")
-
 # about dialog, --version etc.
 WEBSITE = "https://quodlibet.readthedocs.org/"
-COPYRIGHT = u"""Copyright © 2004-2016 %s...""" % u", ".join(MAIN_AUTHORS)
+COPYRIGHT = u"Copyright 2004-2017"
 
 AUTHORS = sorted(u"""\
 Alexandre Passos
@@ -96,9 +102,11 @@ Bruno Bergot
 Carlo Teubner
 Christine Spang
 Christoph Reiter
+Corentin Néau
 David Kågedal
 David Schneider
 Decklin Foster
+Didier Villevalois
 Eduardo Gonzalez
 Eric Casteleijn
 Erich Schubert
@@ -106,6 +114,7 @@ Eric Le Lay
 Federico Pelloni
 Felix Krull
 Florian Demmer
+Fredrik Strupe
 Guillaume Chazarain
 Hans Scholze
 Iñigo Serna
@@ -152,14 +161,23 @@ Tomasz Miasko
 Tomasz Torcz
 Tshepang Lekhonkhobe
 Türerkan İnce
+Uriel Zajaczkovski
 Vasiliy Faronov
 Victoria Hayes
 Zack Weinberg
+Vimalan Reddy
+Jason Heard
+David Pérez Carmona
+Jakub Wilk
+IBBoard@github
+CreamyCookie@github
 """.strip().split("\n"))
 
 TRANSLATORS = sorted(u"""
+Åka Sikrom (nb)
 Alexandre Passos (pt)
 Andreas Bertheussen (nb)
+Olivier Humbert (fr)
 Anton Shestakov (ru)
 Bastian Kleineidam (de)
 Bastien Gorissen (fr)
@@ -210,6 +228,10 @@ Yasushi Iwata (ja)
 Николай Прокошенко (ru)
 Ростислав "zbrox" Райков (bg)
 Сергей Федосеев (ru)
+scootergrisen@github (da)
+Marek Suchánek (cs)
+Till Berger (de)
+Jean-Michel Pouré (fr)
 """.strip().splitlines())
 
 ARTISTS = sorted(u"""\
