@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014 Nick Boultbee
+# Copyright 2014, 2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,10 +8,11 @@
 import socket
 from telnetlib import Telnet
 import time
-import urllib
 
+from quodlibet import _
 from quodlibet import app
 from quodlibet.util.dprint import print_w, print_d, print_
+from quodlibet.compat import quote, unquote
 
 
 class SqueezeboxException(Exception):
@@ -62,8 +63,8 @@ class SqueezeboxServer(object):
                 if self._debug:
                     print_d("Trying %s..." % self.config)
                 self.telnet = Telnet(hostname, port, self._TIMEOUT)
-            except socket.error:
-                print_d("Couldn't talk to %s" % (self.config,))
+            except socket.error as e:
+                print_d("Couldn't talk to %s (%s)" % (self.config, e))
             else:
                 result = self.__request("login %s %s" % (user, password))
                 if result != (6 * '*'):
@@ -93,10 +94,10 @@ class SqueezeboxServer(object):
         if self._debug:
             print_(">>>> \"%s\"" % line)
         try:
-            self.telnet.write(line + "\n")
+            self.telnet.write((line + "\n").encode('utf-8'))
             if not want_reply:
                 return None
-            raw_response = self.telnet.read_until("\n").strip()
+            raw_response = self.telnet.read_until(b"\n").decode('utf-8')
         except socket.error as e:
             print_w("Couldn't communicate with squeezebox (%s)" % e)
             self.failures += 1
@@ -104,11 +105,11 @@ class SqueezeboxServer(object):
                 print_w("Too many Squeezebox failures. Disconnecting")
                 self.is_connected = False
             return None
-        response = raw_response if raw else urllib.unquote(raw_response)
+        response = (raw_response if raw else unquote(raw_response)).strip()
         if self._debug:
             print_("<<<< \"%s\"" % (response,))
-        return response[len(line) - 1:] if line.endswith("?")\
-            else response[len(line) + 1:]
+        return (response[len(line) - 1:] if line.endswith("?")
+                else response[len(line) + 1:])
 
     def get_players(self):
         """ Returns (and caches) a list of the Squeezebox players available"""
@@ -117,12 +118,12 @@ class SqueezeboxServer(object):
         pairs = self.__request("players 0 99", True).split(" ")
 
         def demunge(string):
-            s = urllib.unquote(string)
+            s = unquote(string)
             cpos = s.index(":")
-            return (s[0:cpos], s[cpos + 1:])
+            return s[0:cpos], s[cpos + 1:]
 
         # Do a meaningful URL-unescaping and tuplification for all values
-        pairs = map(demunge, pairs)
+        pairs = [demunge(p) for p in pairs]
 
         # First element is always count
         count = int(pairs.pop(0)[1])
@@ -168,26 +169,26 @@ class SqueezeboxServer(object):
 
     def playlist_play(self, path):
         """Play song immediately"""
-        self.player_request("playlist play %s" % (urllib.quote(path)))
+        self.player_request("playlist play %s" % (quote(path)))
 
     def playlist_add(self, path):
-        self.player_request("playlist add %s" % (urllib.quote(path)), False)
+        self.player_request("playlist add %s" % (quote(path)), False)
 
     def playlist_save(self, name):
-        self.player_request("playlist save %s" % (urllib.quote(name)), False)
+        self.player_request("playlist save %s" % (quote(name)), False)
 
     def playlist_clear(self):
         self.player_request("playlist clear", False)
 
     def playlist_resume(self, name, resume, wipe=False):
         cmd = ("playlist resume %s noplay:%d wipePlaylist:%d"
-               % (urllib.quote(name), int(not resume), int(wipe)))
+               % (quote(name), int(not resume), int(wipe)))
         self.player_request(cmd, want_reply=False)
 
     def change_song(self, path):
         """Queue up a song"""
         self.player_request("playlist clear")
-        self.player_request("playlist insert %s" % (urllib.quote(path)))
+        self.player_request("playlist insert %s" % (quote(path)))
 
     def seek_to(self, ms):
         """Seeks the current song to `ms` milliseconds from start"""
