@@ -7,11 +7,10 @@
 
 # Encoding magic. Show off the submenu stuff.
 
-import locale
-
 from gi.repository import Gtk
 
-from quodlibet.util import get_fs_encoding
+from quodlibet import _
+from quodlibet.qltk import Icons
 from quodlibet.plugins.editing import EditTagsPlugin
 
 ENCODINGS = """\
@@ -21,18 +20,13 @@ iso2022_kr iso8859_2 iso8859_3 iso8859_4 iso8859_5 iso8859_6 iso8859_7
 iso8859_8 iso8859_9 iso8859_10 iso8859_13 iso8859_14 iso8859_15 johab
 koi8_r koi8_u ptcp154 shift_jis utf_16_be utf_16_le""".split()
 
-FSCODING = get_fs_encoding()
-if FSCODING not in ENCODINGS + ["utf-8", "latin1"]:
-    ENCODINGS.append(FSCODING)
-if locale.getpreferredencoding() not in ENCODINGS + ["utf-8", "latin1"]:
-    ENCODINGS.append(FSCODING)
-
 
 class Iconv(EditTagsPlugin):
     PLUGIN_ID = "Convert Encodings"
     PLUGIN_NAME = _("Convert Encodings")
     PLUGIN_DESC = _("Fixes misinterpreted tag value encodings in the "
                     "tag editor.")
+    PLUGIN_ICON = Icons.EDIT_FIND_REPLACE
 
     def __init__(self, tag, value):
         super(Iconv, self).__init__(
