@@ -1,64 +1,68 @@
 # -*- coding: utf-8 -*-
 # Copyright 2010 Steven Robertson
 #           2012 Christoph Reiter
+#           2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-# TODO: Include presets, saving and loading.
+# TODO: Include saving and loading.
 
-from gi.repository import Gtk
+from gi.repository import Gtk, Gdk
 
+from quodlibet import _
 from quodlibet import app
 from quodlibet import config
 from quodlibet.qltk import Button, Icons
 from quodlibet.plugins.events import EventPlugin
+from quodlibet.compat import iteritems
 
-# Presets taken from pulseaudio equalizer
+
+# Presets (roughly) taken from Pulseaudio equalizer
 PRESET_BANDS = [50, 100, 156, 220, 311, 440, 622, 880, 1250, 1750, 2500,
                 3500, 5000, 10000, 20000]
 PRESETS = {
     "flat": (_("Flat"), [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]),
-    "live": (_("Live"), [-9.0, -5.5, 0.0, 1.5, 2.1, 3.4, 3.4, 3.4, 3.4,
-                         3.4, 3.4, 3.4, 2.8, 1.6, 1.8]),
+    "live": (_("Live"), [-9.0, -5.5, 0.0, 1.5, 2.0, 3.5, 3.5, 3.5, 3.5,
+                         3.5, 3.5, 3.5, 3.0, 1.5, 2.0]),
     "full_bass_treble": (_("Full Bass & Treble"),
-                         [4.8, 4.8, 3.5, 2.5, 0.0, -7.0, -14.0, -10.0, -10.0,
-                          -8.0, 1.0, 1.0, 5.2, 7.7, 9.5]),
-    "club": (_("Club"), [-0.2, -0.2, -0.2, -0.2, 3.5, 3.5, 3.5, 3.5, 3.5,
+                         [5.0, 5.0, 3.5, 2.5, 0.0, -7.0, -14.0, -10.0, -10.0,
+                          -8.0, 1.0, 1.0, 5.0, 7.5, 9.5]),
+    "club": (_("Club"), [0.0, 0.0, 0.0, 0.0, 3.5, 3.5, 3.5, 3.5, 3.5,
                          3.5, 3.5, 2.5, 2.5, 0.0, 0.0]),
     "large_hall": (_("Large Hall"), [7.0, 7.0, 7.0, 3.5, 3.0, 3.0, 3.0, 1.5,
                                      0.0, -2.0, -3.5, -6.0, -9.0, -1.0, 0.0]),
-    "party": (_("Party"), [4.8, 4.8, 4.8, 3.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
-                           0.0, 0.0, 0.0, 2.5, 4.8]),
-    "rock": (_("Rock"), [5.3, 2.6, 2.6, -8.5, -10.5, -11.2, -16.0, -14.7,
-                         -6.6, -5.7, -3.0, 3.0, 6.7, 7.3, 7.3]),
-    "soft": (_("Soft"), [3.2, 2.8, 0.8, 0.9, 0.0, -2.4, -4.8, 1.5, 0.0, 1.1,
-                         3.0, 3.0, 5.8, 7.8, 7.8]),
+    "party": (_("Party"), [5.0, 5.0, 5.0, 3.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
+                           0.0, 0.0, 0.0, 2.5, 5.0]),
+    "rock": (_("Rock"), [5.5, 2.5, 2.5, -8.5, -10.5, -11.0, -16.0, -14.5,
+                         -6.5, -5.5, -3.0, 3.0, 6.5, 7.0, 7.0]),
+    "soft": (_("Soft"), [3.0, 3.0, 1.0, 1.0, 0.0, -2.5, -5.0, 1.5, 0.0, 1.0,
+                         3.0, 3.0, 6.0, 8.0, 8.0]),
     "full_bass": (_("Full Bass"),
                   [-16.0, -16.0, 6.5, 6.5, 6.0, 5.5, 4.5, 1.0, 1.0, 1.0, -8.0,
-                   -10.0, -16.0, -16.0, -20.4]),
+                   -10.0, -16.0, -16.0, -20.5]),
     "classical": (_("Classical"),
-                  [-0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2, -0.2,
-                   -0.2, -0.2, -0.2, -21.0, -21.0, -27.0]),
+                  [0, 0, 0, 0, 0, 0, 0, 0, 0,
+                   0, 0, 0, -21.0, -21.0, -27.0]),
     "reggae": (_("Reggae"), [0.0, 0.0, 0.0, 0.0, 0.0, -4.5, -10.0, -6.0, 0.5,
                              1.0, 2.0, 4.0, 4.0, 0.0, 0.0]),
     "headphones": (_("Headphones"),
-                   [3.0, 3.0, 7.3, 7.0, 3.0, -1.0, -6.6, -6.3, -4.5, -4.0,
-                    1.1, 1.2, 5.8, 7.9, 8.8]),
-    "soft_rock": (_("Soft Rock"), [2.7, 2.7, 2.7, 1.5, 1.5, 1.4, 0.0, -3.6,
-                                   -8.0, -7.2, -9.8, -8.9, -6.6, 1.4, 5.8]),
+                   [3.0, 3.0, 7.0, 7.0, 3.0, -1.0, -6.5, -6.0, -4.5, -4.0,
+                    1.0, 1.0, 6.0, 8.0, 9.0]),
+    "soft_rock": (_("Soft Rock"), [3.0, 3.0, 3.0, 1.5, 1.5, 1.5, 0.0, -3.5,
+                                   -8.0, -7.0, -10, -9.0, -6.5, 1.5, 6.0]),
     "full_treble": (_("Full Treble"),
-                    [4.8, -18.6, -18.6, -18.6, -18.6, -10.0, -8.0, -6.5, 1.5,
-                     1.5, 1.5, 8.5, 10.6, 10.6, 10.6]),
-    "dance": (_("Dance"), [6.1, 4.3, 4.3, 1.7, 1.7, 1.7, -0.1, -0.1, -0.1,
-                           0.8, -10.7, -14.2, -15.1, -7.2, 0.0]),
-    "pop": (_("Pop"), [-3.4, 1.7, 2.0, 3.0, 5.0, 5.6, 6.5, 5.2, 3.2, 1.5, 0.0,
-                       -2.5, -4.8, -4.8, -3.2]),
-    "techno": (_("Techno"), [5.0, 4.0, 3.9, 3.3, 0.0, -4.5, -10.0, -8.9, -8.1,
-                             -5.5, -1.5, 3.0, 6.0, 6.1, 5.8]),
-    "ska": (_("Ska"), [-4.5, -8.1, -8.9, -8.5, -8.0, -6.0, 0.0, 1.5, 2.5, 2.7,
-                       3.2, 3.3, 5.8, 6.4, 6.4]),
+                    [5.0, -18.5, -18.5, -18.5, -18.5, -10.0, -8.0, -6.5, 1.5,
+                     1.5, 1.5, 8.5, 10.5, 10.5, 10.5]),
+    "dance": (_("Dance"), [6.0, 4.0, 4.0, 1.5, 1.5, 1.5, 0.0, 0.0, 0.0,
+                           1.0, -10.5, -14.0, -15.0, -7.0, 0.0]),
+    "pop": (_("Pop"), [-3.5, 1.0, 2.0, 3.0, 5.0, 5.5, 6.5, 5.0, 3.0, 1.5, 0.0,
+                       -2.5, -5.0, -5.0, -3.0]),
+    "techno": (_("Techno"), [5.0, 4.0, 4.0, 3.0, 0.0, -4.5, -10.0, -9.0, -8.0,
+                             -5.5, -1.5, 3.0, 6.0, 6.0, 6.0]),
+    "ska": (_("Ska"), [-4.5, -8.0, -9.0, -8.5, -8.0, -6.0, 0.0, 1.5, 2.5, 2.5,
+                       3.0, 3.0, 6.0, 6.0, 6.0]),
     "laptop": (_("Laptop"), [-1, -1, -1, -1, -5, -10, -18, -15, -10, -5, -5,
                              -5, -5, 0, 0]),
 }
@@ -82,7 +86,8 @@ def interp_bands(src_band, target_band, src_gain):
 
 def get_config():
     try:
-        return map(float, config.get('plugins', 'equalizer_levels').split(','))
+        eq_levels_str = config.get('plugins', 'equalizer_levels')
+        return [float(s) for s in eq_levels_str.split(',')]
     except (config.Error, ValueError):
         return []
 
@@ -90,7 +95,9 @@ def get_config():
 class Equalizer(EventPlugin):
     PLUGIN_ID = "Equalizer"
     PLUGIN_NAME = _("Equalizer")
-    PLUGIN_DESC = _("Controls the tone of your music with an equalizer.")
+    PLUGIN_DESC = _("Controls the tone of your music with an equalizer.\n"
+                    "Click or use keys to customise levels "
+                    "(right-click resets the band).")
     PLUGIN_ICON = Icons.AUDIO_CARD
 
     @property
@@ -138,9 +145,11 @@ class Equalizer(EventPlugin):
         table.set_col_spacings(6)
 
         def set_band(adj, idx):
-            levels[idx] = adj.get_value()
+            rounded = int(adj.get_value() * 2) / 2.0
+            adj.set_value(rounded)
+            levels[idx] = rounded
             config.set('plugins', 'equalizer_levels',
-                       ','.join(map(str, levels)))
+                       ','.join(str(lv) for lv in levels))
             self.apply()
 
         adjustments = []
@@ -154,10 +163,11 @@ class Equalizer(EventPlugin):
             lbl = Gtk.Label(label=band.split()[1])
             lbl.set_alignment(1, 0.5)
             table.attach(lbl, 1, 2, i, i + 1, xoptions=Gtk.AttachOptions.FILL)
-            adj = Gtk.Adjustment(levels[i], -24., 12., 0.1)
+            adj = Gtk.Adjustment(levels[i], -24., 12., 0.5, 3, 0)
             adj.connect('value-changed', set_band, i)
             adjustments.append(adj)
             hs = Gtk.HScale(adjustment=adj)
+            hs.connect('button-press-event', self.__rightclick)
             hs.set_draw_value(True)
             hs.set_value_pos(Gtk.PositionType.RIGHT)
             hs.connect('format-value', lambda s, v: _('%.1f dB') % v)
@@ -167,7 +177,7 @@ class Equalizer(EventPlugin):
         def clicked_cb(button):
             [adj.set_value(0) for adj in adjustments]
 
-        sorted_presets = sorted(PRESETS.iteritems())
+        sorted_presets = sorted(iteritems(PRESETS))
 
         def combo_changed(combo):
             # custom, skip
@@ -192,3 +202,7 @@ class Equalizer(EventPlugin):
         bbox.pack_start(clear, True, True, 0)
         vb.pack_start(bbox, True, True, 0)
         return vb
+
+    def __rightclick(self, hs, event):
+        if event.button == Gdk.BUTTON_SECONDARY:
+            hs.set_value(0)
