@@ -10,11 +10,12 @@ import tempfile
 
 import dbus
 import dbus.service
+from senf import fsn2uri
 
 from quodlibet import app
 from quodlibet.util.dbusutils import DBusIntrospectable, DBusProperty
 from quodlibet.util.dbusutils import dbus_unicode_validate as unival
-from quodlibet.util.path import uri_from_path
+from quodlibet.compat import iteritems, listmap
 
 from .util import MPRISObject
 
@@ -108,7 +109,7 @@ value="false"/>
         self.__repeat_id = player_options.connect(
             "notify::repeat", self.__repeat_changed)
         self.__random_id = player_options.connect(
-            "notify::random", self.__random_changed)
+            "notify::shuffle", self.__shuffle_changed)
         self.__single_id = player_options.connect(
             "notify::single", self.__single_changed)
 
@@ -135,9 +136,8 @@ value="false"/>
     def __repeat_changed(self, *args):
         self.emit_properties_changed(self.PLAYER_IFACE, ["LoopStatus"])
 
-    def __random_changed(self, *args):
-        self.emit_properties_changed(self.PLAYER_IFACE,
-                                     ["Shuffle", "LoopStatus"])
+    def __shuffle_changed(self, *args):
+        self.emit_properties_changed(self.PLAYER_IFACE, ["Shuffle"])
 
     def __single_changed(self, *args):
         self.emit_properties_changed(self.PLAYER_IFACE, ["LoopStatus"])
@@ -145,8 +145,8 @@ value="false"/>
     def __seeked(self, player, song, ms):
         self.Seeked(ms * 1000)
 
-    def __library_changed(self, library, song):
-        if song and song is not app.player.info:
+    def __library_changed(self, library, songs):
+        if not songs or app.player.info not in songs:
             return
         self.emit_properties_changed(self.PLAYER_IFACE, ["Metadata"])
 
@@ -245,7 +245,7 @@ value="false"/>
             is_temp = name.startswith(tempfile.gettempdir())
             # This doesn't work for embedded images.. the file gets unlinked
             # after loosing the file handle
-            metadata["mpris:artUrl"] = uri_from_path(name)
+            metadata["mpris:artUrl"] = fsn2uri(name)
 
         if not is_temp:
             self.__cover = None
@@ -254,14 +254,14 @@ value="false"/>
         list_val = {"artist": "artist", "albumArtist": "albumartist",
             "comment": "comment", "composer": "composer", "genre": "genre",
             "lyricist": "lyricist"}
-        for xesam, tag in list_val.iteritems():
+        for xesam, tag in iteritems(list_val):
             vals = song.list(tag)
             if vals:
-                metadata["xesam:" + xesam] = map(unival, vals)
+                metadata["xesam:" + xesam] = listmap(unival, vals)
 
         # All single values
         sing_val = {"album": "album", "title": "title", "asText": "~lyrics"}
-        for xesam, tag in sing_val.iteritems():
+        for xesam, tag in iteritems(sing_val):
             vals = song.comma(tag)
             if vals:
                 metadata["xesam:" + xesam] = unival(vals)
@@ -273,7 +273,7 @@ value="false"/>
         num_val = {"audioBPM": "bpm", "discNumber": "disc",
                    "trackNumber": "track", "useCount": "playcount"}
 
-        for xesam, tag in num_val.iteritems():
+        for xesam, tag in iteritems(num_val):
             val = song("~#" + tag, None)
             if val is not None:
                 metadata["xesam:" + xesam] = int(val)
@@ -317,7 +317,7 @@ value="false"/>
             elif name == "Rate":
                 pass
             elif name == "Shuffle":
-                player_options.random = value
+                player_options.shuffle = value
             elif name == "Volume":
                 player.volume = value
 
@@ -362,7 +362,7 @@ value="false"/>
             elif name == "Rate":
                 return 1.0
             elif name == "Shuffle":
-                return player_options.random
+                return player_options.shuffle
             elif name == "Metadata":
                 return self.__get_metadata()
             elif name == "Volume":
