@@ -2,7 +2,7 @@
 #
 #    Duplicates songs plugin.
 #
-#    Copyright (C) 2012, 2011, 2016 Nick Boultbee
+#    Copyright (C) 2011-2017 Nick Boultbee
 #
 #    Finds "duplicates" of songs selected by searching the library for
 #    others with the same user-configurable "key", presenting a browser-like
@@ -13,13 +13,13 @@
 #    published by the Free Software Foundation.
 #
 
-import string
 import unicodedata
 
+import sys
 from gi.repository import Gtk, Pango
 
 from quodlibet import app
-from quodlibet import print_d, util, qltk
+from quodlibet import print_d, util, qltk, _
 from quodlibet.plugins import PluginConfigMixin
 from quodlibet.plugins.songshelpers import any_song, is_finite
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
@@ -29,7 +29,9 @@ from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.views import RCMHintedTreeView
 from quodlibet.qltk import Icons, Button
-from quodlibet.util import connect_obj, connect_destroy
+from quodlibet.util import connect_obj, connect_destroy, cached_func
+from quodlibet.util.i18n import numeric_phrase
+from quodlibet.compat import text_type, xrange, unichr
 
 
 class DuplicateSongsView(RCMHintedTreeView):
@@ -191,7 +193,6 @@ class DuplicatesTreeModel(Gtk.TreeStore):
     def add_to_existing_group(self, key, song):
         """Tries to add a song to an existing group. Returns None if not able
         """
-        #print_d("Trying to add %s to group \"%s\"" % (song("~filename"), key))
         for parent in self:
             if key == parent[0]:
                 print_d("Found group", self)
@@ -286,8 +287,9 @@ class DuplicateDialog(Gtk.Window):
             return songlist.popup_menu(menu, 0, Gtk.get_current_event_time())
 
     def __init__(self, model):
-        songs_text = ngettext("%d duplicate group", "%d duplicate groups",
-                len(model)) % len(model)
+        songs_text = numeric_phrase("%d duplicate group",
+                                    "%d duplicate groups",
+                                    len(model))
         super(DuplicateDialog, self).__init__()
         self.set_destroy_with_parent(True)
         self.set_title("Quod Libet - %s (%s)" % (Duplicates.PLUGIN_NAME,
@@ -356,6 +358,15 @@ class DuplicateDialog(Gtk.Window):
         self.show_all()
 
 
+@cached_func
+def _remove_punctuation_trans():
+    """Lookup all Unicode punctuation, and remove it"""
+
+    return dict.fromkeys(
+        i for i in xrange(sys.maxunicode)
+        if unicodedata.category(unichr(i)).startswith('P'))
+
+
 class Duplicates(SongsMenuPlugin, PluginConfigMixin):
     PLUGIN_ID = 'Duplicates'
     PLUGIN_NAME = _('Duplicates Browser')
@@ -377,9 +388,6 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
     key_expression = None
     __cfg_cache = {}
 
-    # Faster than a speeding bullet
-    __trans = string.maketrans("", "")
-
     @classmethod
     def get_key_expression(cls):
         if not cls.key_expression:
@@ -431,8 +439,8 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
 
     @staticmethod
     def remove_accents(s):
-        return filter(lambda c: not unicodedata.combining(c),
-                      unicodedata.normalize('NFKD', unicode(s)))
+        return "".join(c for c in unicodedata.normalize('NFKD', text_type(s))
+                       if not unicodedata.combining(c))
 
     @classmethod
     def get_key(cls, song):
@@ -442,7 +450,7 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
         if cls.config_get_bool(cls._CFG_CASE_INSENSITIVE):
             key = key.lower()
         if cls.config_get_bool(cls._CFG_REMOVE_PUNCTUATION):
-            key = str(key).translate(cls.__trans, string.punctuation)
+            key = (key.translate(_remove_punctuation_trans()))
         if cls.config_get_bool(cls._CFG_REMOVE_WHITESPACE):
             key = "_".join(key.split())
         return key
@@ -453,11 +461,12 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
 
         # Index all songs by our custom key
         # TODO: make this cache-friendly
-        print_d("Calculating duplicates...")
+        print_d("Calculating duplicates for %d song(s)..." % len(songs))
         groups = {}
         for song in songs:
             key = self.get_key(song)
             if key and key in groups:
+                print_d("Found duplicate based on '%s'" % key)
                 groups[key].add(song._song)
             elif key:
                 groups[key] = {song._song}
