@@ -8,17 +8,14 @@
 import json
 import collections
 import threading
-import urllib
-import urllib2
-import socket
-import Queue
-import StringIO
 import gzip
 from xml.dom.minidom import parseString
 
 from gi.repository import GLib
 
 from quodlibet.util import print_w
+from quodlibet.compat import iteritems, urlencode, queue, cBytesIO
+from quodlibet.util.urllib import urlopen, Request
 from .util import get_api_key, GateKeeper
 
 
@@ -56,14 +53,14 @@ class AcoustidSubmissionThread(threading.Thread):
 
         self.__done += len(urldata)
 
-        basedata = urllib.urlencode({
+        basedata = urlencode({
             "format": "xml",
             "client": APP_KEY,
             "user": get_api_key(),
         })
 
-        urldata = "&".join([basedata] + map(urllib.urlencode, urldata))
-        obj = StringIO.StringIO()
+        urldata = "&".join([basedata] + map(urlencode, urldata))
+        obj = cBytesIO()
         gzip.GzipFile(fileobj=obj, mode="wb").write(urldata)
         urldata = obj.getvalue()
 
@@ -71,12 +68,12 @@ class AcoustidSubmissionThread(threading.Thread):
             "Content-Encoding": "gzip",
             "Content-type": "application/x-www-form-urlencoded"
         }
-        req = urllib2.Request(self.URL, urldata, headers)
+        req = Request(self.URL, urldata, headers)
 
         error = None
         try:
-            response = urllib2.urlopen(req, timeout=self.TIMEOUT)
-        except (urllib2.URLError, socket.timeout) as e:
+            response = urlopen(req, timeout=self.TIMEOUT)
+        except EnvironmentError as e:
             error = "urllib error: " + str(e)
         else:
             xml = response.read()
@@ -116,7 +113,7 @@ class AcoustidSubmissionThread(threading.Thread):
             }
 
             tuples = []
-            for key, value in track.iteritems():
+            for key, value in iteritems(track):
                 # this also dismisses 0.. which should be ok here.
                 if not value:
                     continue
@@ -265,7 +262,7 @@ class AcoustidLookupThread(threading.Thread):
     def __init__(self, progress_cb):
         super(AcoustidLookupThread, self).__init__()
         self.__progress_cb = progress_cb
-        self.__queue = Queue.Queue()
+        self.__queue = queue.Queue()
         self.__stopped = False
         self.start()
 
@@ -284,7 +281,7 @@ class AcoustidLookupThread(threading.Thread):
 
     def __process(self, results):
         req_data = []
-        req_data.append(urllib.urlencode({
+        req_data.append(urlencode({
             "format": "json",
             "client": APP_KEY,
             "batch": "1",
@@ -292,7 +289,7 @@ class AcoustidLookupThread(threading.Thread):
 
         for i, result in enumerate(results):
             postfix = ".%d" % i
-            req_data.append(urllib.urlencode({
+            req_data.append(urlencode({
                 "duration" + postfix: str(int(round(result.length))),
                 "fingerprint" + postfix: result.chromaprint,
             }))
@@ -300,7 +297,7 @@ class AcoustidLookupThread(threading.Thread):
         req_data.append("meta=releases+recordings+tracks+sources")
 
         urldata = "&".join(req_data)
-        obj = StringIO.StringIO()
+        obj = cBytesIO()
         gzip.GzipFile(fileobj=obj, mode="wb").write(urldata)
         urldata = obj.getvalue()
 
@@ -308,13 +305,13 @@ class AcoustidLookupThread(threading.Thread):
             "Content-Encoding": "gzip",
             "Content-type": "application/x-www-form-urlencoded"
         }
-        req = urllib2.Request(self.URL, urldata, headers)
+        req = Request(self.URL, urldata, headers)
 
         releases = {}
         error = ""
         try:
-            response = urllib2.urlopen(req, timeout=self.TIMEOUT)
-        except (urllib2.URLError, socket.timeout) as e:
+            response = urlopen(req, timeout=self.TIMEOUT)
+        except EnvironmentError as e:
             error = "urllib error: " + str(e)
         else:
             try:
@@ -343,7 +340,7 @@ class AcoustidLookupThread(threading.Thread):
                 timeout = 0.5 / len(results)
                 try:
                     results.append(self.__queue.get(timeout=timeout))
-                except Queue.Empty:
+                except queue.Empty:
                     break
 
             if self.__stopped:
