@@ -1,10 +1,13 @@
 # -*- coding: utf-8 -*-
 # Copyright 2008 Joe Wreschnig
+#           2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from quodlibet import _
+from quodlibet.plugins.songshelpers import any_song, is_finite, is_writable
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.qltk import Icons
 
@@ -34,6 +37,8 @@ class MakeSortTags(SongsMenuPlugin):
     PLUGIN_DESC = _("Converts album and artist names to sort names, poorly.")
     PLUGIN_ICON = Icons.EDIT
 
+    plugin_handles = any_song(is_writable, is_finite)
+
     def plugin_song(self, song):
         for tag in ["album"]:
             values = filter(None, map(album_to_sort, song.list(tag)))
