@@ -11,7 +11,8 @@ from ._audio import PEOPLE, AudioFile, DUMMY_SONG, decode_value, \
 from ._image import EmbeddedImage, APICType
 from ._misc import AudioFileError, init, MusicFile, types, loaders, filter, \
     mimes
+from ._serialize import load_audio_files, dump_audio_files, SerializationError
 
 AudioFile, AudioFileError, EmbeddedImage, DUMMY_SONG, PEOPLE, decode_value,
 APICType, FILESYSTEM_TAGS, TIME_TAGS, init, MusicFile, types, loaders, filter,
-mimes
+mimes, load_audio_files, dump_audio_files, SerializationError
