@@ -6,10 +6,13 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from quodlibet import const
+import importlib
+
 from quodlibet import util
+from quodlibet.compat import swap_to_string, text_type
 
 
+@swap_to_string
 class PlayerError(Exception):
     """Error raised by player loading/initialization and emitted by the
     error signal during playback.
@@ -22,19 +25,31 @@ class PlayerError(Exception):
         self.short_desc = short_desc
         self.long_desc = long_desc
 
-    def __unicode__(self):
+    def __str__(self):
         return self.short_desc + (
             u"\n" + self.long_desc if self.long_desc else u"")
 
-    def __str__(self):
-        return unicode(self).encode('utf-8')
+    def __bytes__(self):
+        return text_type(self).encode('utf-8')
 
     def __repr__(self):
         return "%s(%r, %r)" % (
             type(self).__name__, repr(self.short_desc), repr(self.long_desc))
 
 
-def init(backend_name):
+def init_player(backend_name, librarian):
+    """Loads the specified backend and initializes it.
+
+    Returns a BasePlayer implementation instance.
+
+    Raises PlayerError in case of an error.
+    """
+
+    backend = init_backend(backend_name)
+    return backend.init(librarian)
+
+
+def init_backend(backend_name):
     """Imports the player backend module for the given name.
     Raises PlayerError if the import fails.
 
@@ -45,14 +60,9 @@ def init(backend_name):
     modulename = "quodlibet.player." + backend_name
 
     try:
-        backend = __import__(modulename, {}, {}, "quodlibet.player")
-    except ImportError:
-        if const.DEBUG:
-            util.print_exc()
-
-        raise PlayerError(
-            _("Invalid audio backend"),
-            _("The audio backend '%(backend-name)s' could not be loaded.") % {
-                "backend-name": backend_name})
+        backend = importlib.import_module(modulename)
+    except Exception as e:
+        util.print_exc()
+        util.reraise(PlayerError, str(e))
     else:
         return backend
