@@ -10,7 +10,7 @@ import gi
 try:
     gi.require_version("Gst", "1.0")
     gi.require_version("GstPbutils", "1.0")
-except ValueError, e:
+except ValueError as e:
     raise ImportError(e)
 
 from gi.repository import Gst, GLib, GstPbutils
@@ -19,12 +19,15 @@ from quodlibet import const
 from quodlibet import config
 from quodlibet import util
 from quodlibet import app
+from quodlibet import _
 
 from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
-    MainRunnerAbortedError, MainRunnerTimeoutError
+    MainRunnerAbortedError, MainRunnerTimeoutError, print_w, print_d, \
+    print_e, print_
 from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.qltk.notif import Task
+from quodlibet.compat import iteritems
 
 from .util import (parse_gstreamer_taglist, TagListWrapper, iter_to_list,
     GStreamerSink, link_many, bin_debug)
@@ -34,6 +37,9 @@ from .prefs import GstPlayerPreferences
 STATE_CHANGE_TIMEOUT = Gst.SECOND * 4
 
 
+const.MinVersions.GSTREAMER.check(Gst.version())
+
+
 class BufferingWrapper(object):
     """A wrapper for a Gst.Element (usually GstPlayBin) which pauses the
     elmement in case buffering is needed, but hides the fact that it does.
@@ -405,7 +411,8 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                         "mpg123audiodec": -2
                     }.get(f.get_name(), i)
                     return (i, f)
-                return zip(*sorted(map(set_prio, enumerate(factories))))[1]
+                return list(
+                    zip(*sorted(map(set_prio, enumerate(factories)))))[1]
 
             for e in iter_to_list(self.bin.iterate_recurse):
                 try:
@@ -485,13 +492,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             gerror, debug_info = message.parse_error()
             message = u""
             if gerror:
-                message = gerror.message.decode("utf-8").rstrip(".")
+                message = util.gdecode(gerror.message).rstrip(".")
             details = None
             if debug_info:
                 # strip the first line, not user friendly
                 debug_info = "\n".join(debug_info.splitlines()[1:])
                 # can contain paths, so not sure if utf-8 in all cases
-                details = debug_info.decode("utf-8", errors="replace")
+                details = util.gdecode(debug_info)
             self._error(PlayerError(message, details))
         elif message.type == Gst.MessageType.STATE_CHANGED:
             # pulsesink doesn't notify a volume change on startup
@@ -552,7 +559,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         format_desc = get_description(message)
         title = _(u"No GStreamer element found to handle media format")
         error_details = _(u"Media format: %(format-description)s") % {
-            "format-description": format_desc.decode("utf-8")}
+            "format-description": util.gdecode(format_desc)}
 
         def install_done_cb(plugins_return, *args):
             print_d("Gstreamer plugin install return: %r" % plugins_return)
@@ -588,9 +595,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self._error(PlayerError(title, error_details))
 
     def __about_to_finish_sync(self):
-        """Returns a tuple (ok, next_song). ok is True if the next song
-        should be set.
-        """
+        """Returns the next song uri to play or None"""
 
         print_d("About to finish (sync)")
 
@@ -601,30 +606,32 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         # https://bugzilla.gnome.org/show_bug.cgi?id=695474
         if self.song.multisong:
             print_d("multisong: ignore about to finish")
-            return (False, None)
+            return
 
         if config.getboolean("player", "gst_disable_gapless"):
             print_d("Gapless disabled")
-            return (False, None)
+            return
 
         # this can trigger twice, see issue 987
         if self._in_gapless_transition:
-            return (False, None)
+            return
         self._in_gapless_transition = True
 
         print_d("Select next song in mainloop..")
         self._source.next_ended()
         print_d("..done.")
 
-        return (True, self._source.current)
+        song = self._source.current
+        if song is not None:
+            return song("~uri")
 
     def __about_to_finish(self, playbin):
         print_d("About to finish (async)")
 
         try:
-            ok, song = self._runner.call(self.__about_to_finish_sync,
-                                         priority=GLib.PRIORITY_HIGH,
-                                         timeout=0.5)
+            uri = self._runner.call(self.__about_to_finish_sync,
+                                    priority=GLib.PRIORITY_HIGH,
+                                    timeout=0.5)
         except MainRunnerTimeoutError as e:
             # Due to some locks being held during this signal we can get
             # into a deadlock when a seek or state change event happens
@@ -633,16 +640,15 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             # in a non-gapless transition.
             print_d("About to finish (async): %s" % e)
             return
-        except MainRunnerAbortedError:
+        except MainRunnerAbortedError as e:
             print_d("About to finish (async): %s" % e)
             return
         except MainRunnerError:
             util.print_exc()
             return
 
-        if ok:
+        if uri is not None:
             print_d("About to finish (async): setting uri")
-            uri = song("~uri") if song is not None else None
             playbin.set_property('uri', uri)
         print_d("About to finish (async): done")
 
@@ -771,7 +777,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         self.error = True
         self.paused = True
 
-        print_w(unicode(player_error))
+        print_w(player_error)
         self.emit('error', self.song, player_error)
         self._active_error = False
 
@@ -876,12 +882,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         tags = TagListWrapper(tags, merge=True)
         tags = parse_gstreamer_taglist(tags)
 
-        for key, value in sanitize_tags(tags, stream=False).iteritems():
+        for key, value in iteritems(sanitize_tags(tags, stream=False)):
             if self.song.get(key) != value:
                 changed = True
                 self.song[key] = value
 
-        for key, value in sanitize_tags(tags, stream=True).iteritems():
+        for key, value in iteritems(sanitize_tags(tags, stream=True)):
             if new_info.get(key) != value:
                 info_changed = True
                 new_info[key] = value
