@@ -10,8 +10,6 @@ from quodlibet.player import PlayerError
 
 
 class NullPlayer(BasePlayer):
-    __gproperties__ = BasePlayer._gproperties_
-    __gsignals__ = BasePlayer._gsignals_
     version_info = "Null Player Backend"
     name = "Null"
 
@@ -19,6 +17,8 @@ class NullPlayer(BasePlayer):
         super(NullPlayer, self).__init__()
         self._paused = True
         self._source = None
+        self._volume = 1.0
+        self._mute = False
         self._position = 0
 
     def _destroy(self):
@@ -40,9 +40,23 @@ class NullPlayer(BasePlayer):
         self._paused = paused
         self.emit((paused and 'paused') or 'unpaused')
 
+    def do_get_property(self, property):
+        if property.name == 'volume':
+            return self._volume
+        elif property.name == 'seekable':
+            if self.song is None:
+                return False
+            return True
+        elif property.name == 'mute':
+            return self._mute
+        else:
+            raise AttributeError
+
     def do_set_property(self, property, v):
         if property.name == 'volume':
             self._volume = v
+        elif property.name == 'mute':
+            self._mute = v
         else:
             raise AttributeError
 
@@ -77,9 +91,11 @@ class NullPlayer(BasePlayer):
             self.emit("unpaused")
 
         self._position = 0
+        # seekable might change if we change to None, so notify just in case
+        self.notify("seekable")
 
     def can_play_uri(self, uri):
-        return False
+        return True
 
 
 def init(librarian):
