@@ -5,12 +5,14 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from quodlibet import _
 from quodlibet import config
 from quodlibet import util
 from quodlibet.util.modulescanner import ModuleScanner
 from quodlibet.util.dprint import print_d
 from quodlibet.util.config import ConfigProxy
 from quodlibet.qltk.ccb import ConfigCheckButton
+from quodlibet.compat import itervalues, iteritems, listkeys, string_types
 
 
 def init(folders=None, disable_plugins=False):
@@ -148,7 +150,7 @@ class Plugin(object):
     @property
     def tags(self):
         tags = getattr(self.cls, "PLUGIN_TAGS", [])
-        if isinstance(tags, basestring):
+        if isinstance(tags, string_types):
             tags = [tags]
         return tags
 
@@ -275,14 +277,14 @@ class PluginManager(object):
 
     @property
     def _modules(self):
-        return self.__scanner.modules.itervalues()
+        return itervalues(self.__scanner.modules)
 
     @property
     def _plugins(self):
         """All registered plugins"""
 
         plugins = []
-        for module in self.__modules.itervalues():
+        for module in itervalues(self.__modules):
             for plugin in module.plugins:
                 plugins.append(plugin)
         return plugins
@@ -366,7 +368,7 @@ class PluginManager(object):
         """module name: list of error message text lines"""
 
         errors = {}
-        for name, error in self.__scanner.failures.iteritems():
+        for name, error in iteritems(self.__scanner.failures):
             exception = error.exception
             if isinstance(exception, PluginImportException):
                 if not exception.should_show():
@@ -380,7 +382,7 @@ class PluginManager(object):
     def quit(self):
         """Disable plugins and tell all handlers to clean up"""
 
-        for name in self.__modules.keys():
+        for name in listkeys(self.__modules):
             self.__remove_module(name)
 
     def __remove_module(self, name):
@@ -419,6 +421,13 @@ class PluginManager(object):
 PM = PluginManager
 
 
+def plugin_enabled(plugin):
+    """Returns true if the plugin is enabled (or "always" enabled)"""
+    pm = PluginManager.instance
+    enabled = pm.enabled(plugin) or not plugin.can_enable
+    return enabled
+
+
 def get_config_option(plugin_cls, option):
     try:
         prefix = plugin_cls.CONFIG_SECTION
@@ -432,7 +441,7 @@ class PluginConfig(ConfigProxy):
     """A proxy for a Config object that can be used by plugins.
 
     Provides some methods of the Config class but doesn't need a
-    sections and prefixes the config option name.
+    section and prefixes the config option name.
     """
 
     def __init__(self, prefix, _config=None, _defaults=True):
@@ -500,3 +509,69 @@ class PluginConfigMixin(object):
             cls.config_set(name, default)
         return ConfigCheckButton(label, PM.CONFIG_SECTION,
                                  option, populate=True)
+
+
+class ConfProp(object):
+
+    def __init__(self, conf, name, default):
+        self._conf = conf
+        self._name = name
+
+        self._conf.defaults.set(name, default)
+
+    def __get__(self, *args, **kwargs):
+        return self._conf.get(self._name)
+
+    def __set__(self, obj, value):
+        self._conf.set(self._name, value)
+
+
+class BoolConfProp(ConfProp):
+
+    def __get__(self, *args, **kwargs):
+        return self._conf.getboolean(self._name)
+
+
+class IntConfProp(ConfProp):
+
+    def __get__(self, *args, **kwargs):
+        return self._conf.getint(self._name)
+
+
+class FloatConfProp(ConfProp):
+
+    def __get__(self, *args, **kwargs):
+        return self._conf.getfloat(self._name)
+
+
+def str_to_color_tuple(s):
+    """Raises ValueError"""
+
+    lst = [float(p) for p in s.split()]
+    while len(lst) < 4:
+        lst.append(0.0)
+    return tuple(lst)
+
+
+def color_tuple_to_str(t):
+    return " ".join(map(str, t))
+
+
+class ColorConfProp(ConfProp):
+
+    def __init__(self, conf, name, default):
+        self._conf = conf
+        self._name = name
+
+        self._conf.defaults.set(name, color_tuple_to_str(default))
+
+    def __get__(self, *args, **kwargs):
+        s = self._conf.get(self._name)
+
+        try:
+            return str_to_color_tuple(s)
+        except ValueError:
+            return str_to_color_tuple(self._conf.defaults.get(self._name))
+
+    def __set__(self, obj, value):
+        self._conf.set(self._name, color_tuple_to_str(value))
