@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2005 Michael Urman, Joe Wreschnig
-#           2014 Nick Boultbee
+#           2014, 2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,12 +8,15 @@
 
 from gi.repository import GObject
 
+from quodlibet.util.dprint import print_e
+
 from quodlibet import util
 from quodlibet.plugins import PluginHandler
 
 from quodlibet.util.songwrapper import SongWrapper, ListWrapper
 from quodlibet.util.songwrapper import check_wrapper_changed
 from quodlibet.util import connect_obj
+from quodlibet.compat import itervalues
 
 
 class EventPlugin(object):
@@ -22,18 +25,36 @@ class EventPlugin(object):
     Event plugins, unlike other plugins, are instantiated on startup and
     the same instance is used even when the plugin is enabled or disabled.
 
-    Callables:
-        obj.plugin_on_song_started(song)
-        obj.plugin_on_song_ended(song, stopped)
-        obj.plugin_on_added([song1, song2, ...])
-        obj.plugin_on_changed([song1, song2, ...])
-        obj.plugin_on_removed([song1, song2, ...])
-        obj.plugin_on_paused()
-        obj.plugin_on_unpaused()
-        obj.plugin_on_seek(song, msec)
-        obj.plugin_on_error(song, error)
+    Methods `plugin_on_*` can be overridden to provide behaviour hooks
     """
 
+    def plugin_on_song_started(self, song):
+        pass
+
+    def plugin_on_song_ended(self, song, stopped):
+        pass
+
+    def plugin_on_added(self, songs):
+        pass
+
+    def plugin_on_changed(self, songs):
+        pass
+
+    def plugin_on_removed(self, songs):
+        pass
+
+    def plugin_on_paused(self):
+        pass
+
+    def plugin_on_unpaused(self):
+        pass
+
+    def plugin_on_seek(self, song, msec):
+        pass
+
+    def plugin_on_error(self, song, error):
+        pass
+
     PLUGIN_INSTANCE = True
 
     def enabled(self):
@@ -96,13 +117,19 @@ class EventPluginHandler(PluginHandler):
                 args[0] = SongWrapper(args[0])
             elif isinstance(args[0], (set, list)):
                 args[0] = ListWrapper(args[0])
-        for plugin in self.__plugins.itervalues():
+        for plugin in itervalues(self.__plugins):
             method_name = 'plugin_on_' + event.replace('-', '_')
             handler = getattr(plugin, method_name, None)
-            if handler is not None:
+
+            def overridden(obj, name):
+                return name in type(obj).__dict__
+
+            if overridden(plugin, method_name):
                 try:
                     handler(*args)
                 except Exception:
+                    print_e("Error during %s on %s" %
+                            (method_name, type(plugin)))
                     util.print_exc()
 
         if event not in ["removed", "changed"] and args:
