@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2006 Federico Pelloni <federico.pelloni@gmail.com>
 #           2013 Christoph Reiter
 #
@@ -10,9 +11,10 @@ import dbus.service
 from dbus import DBusException
 
 from quodlibet.util import dbusutils
-from quodlibet.parse import Query
+from quodlibet.query import Query
 from quodlibet.qltk.songlist import SongList
-from quodlibet.util.path import fsdecode
+from quodlibet.formats import decode_value
+from quodlibet.compat import itervalues
 
 
 class DBusHandler(dbus.service.Object):
@@ -35,10 +37,7 @@ class DBusHandler(dbus.service.Object):
     def __dict(self, song):
         dict = {}
         for key, value in (song or {}).items():
-            if not isinstance(value, basestring):
-                value = unicode(value)
-            elif isinstance(value, str):
-                value = fsdecode(value)
+            value = decode_value(key, value)
             dict[key] = dbusutils.dbus_unicode_validate(value)
         if song:
             dict["~uri"] = song("~uri")
@@ -117,6 +116,6 @@ class DBusHandler(dbus.service.Object):
             except Query.error:
                 pass
             else:
-                return [self.__dict(s) for s in self.library.itervalues()
+                return [self.__dict(s) for s in itervalues(self.library)
                         if results(s)]
         return None
