@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna, Nick Boultbee
+# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna
+#           2011-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,8 +10,8 @@ import sys
 
 from gi.repository import Gtk, Pango, Gdk
 
+from quodlibet import C_, _
 from quodlibet import qltk
-
 from quodlibet import config
 from quodlibet import util
 
@@ -30,9 +31,12 @@ from quodlibet.qltk._editutils import WriteFailedError
 from quodlibet.qltk import Icons
 from quodlibet.plugins import PluginManager
 from quodlibet.util import connect_obj, gdecode
+from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 from quodlibet.util.string.splitters import (split_value, split_title,
     split_people, split_album)
+from quodlibet.compat import iteritems, string_types, text_type, listkeys, \
+    listmap, itervalues
 
 
 class Comment(object):
@@ -56,20 +60,20 @@ class Comment(object):
 
     def _paren(self):
         if self.shared:
-            return ngettext('missing from %d song',
-                            'missing from %d songs',
-                            self.missing) % self.missing
+            return numeric_phrase('missing from %d song',
+                                  'missing from %d songs',
+                                  self.missing)
         elif self.complete:
-            return ngettext('different across %d song',
-                            'different across %d songs',
-                            self.total) % self.total
+            return numeric_phrase('different across %d song',
+                                  'different across %d songs',
+                                  self.total)
         else:
-            d = ngettext('different across %d song',
-                          'different across %d songs',
-                          self.have) % self.have
-            m = ngettext('missing from %d song',
-                          'missing from %d songs',
-                          self.missing) % self.missing
+            d = numeric_phrase('different across %d song',
+                               'different across %d songs',
+                               self.have)
+            m = numeric_phrase('missing from %d song',
+                               'missing from %d songs',
+                               self.missing)
             return ", ".join([d, m])
 
     def is_special(self):
@@ -122,7 +126,7 @@ class AudioFileGroup(dict):
             if real_keys_only:
                 iter_func = song.iterrealitems
             else:
-                iter_func = song.iteritems
+                iter_func = song.items
 
             for comment, val in iter_func():
                 keys[comment] = keys.get(comment, 0) + 1
@@ -147,10 +151,10 @@ class AudioFileGroup(dict):
         self._can_change = can_change
 
         # collect comment representations
-        for tag, count in keys.iteritems():
+        for tag, count in iteritems(keys):
             first_value = first[tag]
-            if not isinstance(first_value, basestring):
-                first_value = unicode(first_value)
+            if not isinstance(first_value, string_types):
+                first_value = text_type(first_value)
             shared = all[tag]
             complete = count == total
             if shared and complete:
@@ -189,13 +193,11 @@ class SplitValues(Gtk.ImageMenuItem):
             label=_("Split into _Multiple Values"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(len(split_value(value, spls)) > 1)
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         return [(tag, v) for v in split_value(value, spls)]
 
 
@@ -226,13 +228,11 @@ class SplitTitle(Gtk.ImageMenuItem):
             label=_("Split _Version out of Title"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(bool(split_title(value, spls)[1]))
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         title, versions = split_title(value, spls)
         return [(tag, title)] + [("version", v) for v in versions]
 
@@ -245,13 +245,11 @@ class SplitPerson(Gtk.ImageMenuItem):
         super(SplitPerson, self).__init__(label=self.title, use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(bool(split_people(value, spls)[1]))
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         artist, others = split_people(value, spls)
         return [(tag, artist)] + [(self.needs[0], o) for o in others]
 
@@ -623,7 +621,8 @@ class EditTags(Gtk.VBox):
                 else:
                     b.connect('activate', self.__menu_activate, view)
 
-                    if (not min(map(self.__songinfo.can_change, b.needs) + [1])
+                    if (not min(listmap(self.__songinfo.can_change, b.needs) +
+                                [1])
                             or comment.is_special()):
                         b.set_sensitive(False)
 
@@ -654,7 +653,7 @@ class EditTags(Gtk.VBox):
         remove.set_sensitive(bool(rows))
 
     def __add_new_tag(self, model, tag, value):
-        assert isinstance(value, unicode)
+        assert isinstance(value, text_type)
         iters = [i for (i, v) in model.iterrows() if v.tag == tag]
         if iters and not self.__songinfo.can_multiple_values(tag):
             title = _("Unable to add tag")
@@ -683,9 +682,9 @@ class EditTags(Gtk.VBox):
                 break
             tag = add.get_tag()
             value = add.get_value()
-            assert isinstance(value, unicode)
+            assert isinstance(value, text_type)
             value = massagers.validate(tag, value)
-            assert isinstance(value, unicode)
+            assert isinstance(value, text_type)
             if not self.__songinfo.can_change(tag):
                 title = _("Invalid tag")
                 msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
@@ -717,7 +716,7 @@ class EditTags(Gtk.VBox):
         added = {}
         renamed = {}
 
-        for entry in model.itervalues():
+        for entry in itervalues(model):
             if entry.edited and not (entry.deleted or entry.renamed):
                 if entry.origvalue is not None:
                     l = updated.setdefault(entry.tag, [])
@@ -750,7 +749,7 @@ class EditTags(Gtk.VBox):
                     break
 
             changed = False
-            for key, values in updated.iteritems():
+            for key, values in iteritems(updated):
                 for (new_value, old_value) in values:
                     if song.can_change(key):
                         if old_value is None:
@@ -759,13 +758,13 @@ class EditTags(Gtk.VBox):
                             song.change(key, old_value.text, new_value.text)
                         changed = True
 
-            for key, values in added.iteritems():
+            for key, values in iteritems(added):
                 for value in values:
                     if song.can_change(key):
                         song.add(key, value.text)
                         changed = True
 
-            for key, values in deleted.iteritems():
+            for key, values in iteritems(deleted):
                 for value in values:
                     if not value.shared:
                         # In case it isn't shared we don't know the actual
@@ -779,7 +778,7 @@ class EditTags(Gtk.VBox):
                         changed = True
 
             save_rename = []
-            for new_tag, values in renamed.iteritems():
+            for new_tag, values in iteritems(renamed):
                 for old_tag, new_value, old_value in values:
                     if (song.can_change(new_tag) and old_tag in song):
                         if not new_value.is_special():
@@ -818,7 +817,7 @@ class EditTags(Gtk.VBox):
             b.set_sensitive(not all_done)
 
     def __edit_tag(self, renderer, path, new_value, model):
-        new_value = new_value.decode("utf-8")
+        new_value = gdecode(new_value)
         new_value = ', '.join(new_value.splitlines())
         path = Gtk.TreePath.new_from_string(path)
         entry = model[path][0]
@@ -927,7 +926,7 @@ class EditTags(Gtk.VBox):
             self.__songinfo = AudioFileGroup(songs)
         songinfo = self.__songinfo
 
-        keys = songinfo.keys()
+        keys = listkeys(songinfo)
         default_tags = get_default_tags()
         keys = set(keys + default_tags)
 
