@@ -6,15 +6,17 @@
 # published by the Free Software Foundation
 
 import os
-import urlparse
+from quodlibet.compat import urlsplit
 import errno
 
 from gi.repository import Gtk, GObject, Gdk, Gio, Pango
+from senf import uri2fsn, fsnative, fsn2text, bytes2fsn
 
 from quodlibet import formats
 from quodlibet import qltk
-from quodlibet import windows
+from quodlibet import _
 
+from quodlibet.util import windows
 from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.views import AllTreeView, RCMHintedTreeView, \
     MultiDragTreeView
@@ -22,10 +24,9 @@ from quodlibet.qltk.views import TreeViewColumn
 from quodlibet.qltk.x import ScrolledWindow, Paned
 from quodlibet.qltk.models import ObjectStore, ObjectTreeStore
 from quodlibet.qltk import Icons
-
-from quodlibet.util.path import fsdecode, listdir, is_fsnative, \
-    glib2fsnative, fsnative, xdg_get_user_dirs, get_home_dir
-from quodlibet.util.uri import URI
+from quodlibet.compat import xrange
+from quodlibet.util.path import listdir, \
+    glib2fsn, xdg_get_user_dirs, get_home_dir
 from quodlibet.util import connect_obj
 
 
@@ -75,11 +76,23 @@ def _get_win_favorites():
     # if not already present
     links = windows.get_links_dir()
     if links is not None:
-        for entry in os.listdir(links):
+        try:
+            link_entries = os.listdir(links)
+        except OSError:
+            link_entries = []
+
+        for entry in link_entries:
             if entry.endswith(".lnk"):
-                target = windows.get_link_target(os.path.join(links, entry))
-                if target is not None:
-                    folders.append(target)
+                try:
+                    target = windows.get_link_target(
+                        os.path.join(links, entry))
+                except WindowsError:
+                    pass
+                else:
+                    if target:
+                        # RecentPlaces.lnk resolves
+                        # to an empty string for example
+                        folders.append(target)
 
     # remove duplicated entries
     filtered = []
@@ -129,15 +142,39 @@ def get_drives():
         for mount in Gio.VolumeMonitor.get().get_mounts():
             path = mount.get_root().get_path()
             if path is not None:
-                paths.append(glib2fsnative(path))
+                paths.append(glib2fsn(path))
         paths.append("/")
         return paths
 
 
+def parse_gtk_bookmarks(data):
+    """
+    Args:
+        data (bytes)
+    Retruns:
+        List[fsnative]
+    Raises:
+        ValueError
+    """
+
+    assert isinstance(data, bytes)
+
+    paths = []
+    for line in data.splitlines():
+        parts = line.split()
+        if not parts:
+            continue
+        folder_url = parts[0]
+        paths.append(bytes2fsn(urlsplit(folder_url)[2], "utf-8"))
+    return paths
+
+
 def get_gtk_bookmarks():
     """A list of paths from the GTK+ bookmarks.
-
     The paths don't have to exist.
+
+    Returns:
+        List[fsnative]
     """
 
     if os.name == "nt":
@@ -147,13 +184,8 @@ def get_gtk_bookmarks():
     folders = []
     try:
         with open(path, "rb") as f:
-            for line in f.readlines():
-                parts = line.split()
-                if not parts:
-                    continue
-                folder_url = parts[0]
-                folders.append(urlparse.urlsplit(folder_url)[2])
-    except EnvironmentError:
+            folders = parse_gtk_bookmarks(f.read())
+    except (EnvironmentError, ValueError):
         pass
 
     return folders
@@ -173,9 +205,9 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         super(DirectoryTree, self).__init__(model=model)
 
         if initial is not None:
-            assert is_fsnative(initial)
+            assert isinstance(initial, fsnative)
 
-        column = TreeViewColumn(_("Folders"))
+        column = TreeViewColumn(title=_("Folders"))
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         render = Gtk.CellRendererPixbuf()
         render.set_property('icon-name', Icons.FOLDER)
@@ -189,7 +221,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         def cell_data(column, cell, model, iter_, userdata):
             value = model.get_value(iter_)
             if value is not None:
-                text = fsdecode(os.path.basename(value) or value)
+                text = fsn2text(os.path.basename(value) or value)
                 cell.set_property('text', text)
 
         column.set_cell_data_func(render, cell_data)
@@ -204,7 +236,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         for path in folders:
             niter = model.append(None, [path])
             if path is not None:
-                assert is_fsnative(path)
+                assert isinstance(path, fsnative)
                 model.append(niter, ["dummy"])
 
         self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
@@ -227,7 +259,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         m = qltk.MenuItem(_("_Refresh"), Icons.VIEW_REFRESH)
         m.connect('activate', self.__refresh)
         menu.append(m)
-        m = qltk.MenuItem(_("_Select All Subfolders"), Icons.FOLDER)
+        m = qltk.MenuItem(_("_Select all Sub-Folders"), Icons.FOLDER)
         m.connect('activate', self.__expand)
         menu.append(m)
         menu.show_all()
@@ -249,7 +281,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         return [model[p][0] for p in paths]
 
     def go_to(self, path_to_go):
-        assert is_fsnative(path_to_go)
+        assert isinstance(path_to_go, fsnative)
 
         # FIXME: what about non-normalized paths?
         model = self.get_model()
@@ -301,7 +333,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
             uris = data.get_uris()
             if uris:
                 try:
-                    filename = URI(uris[0]).filename
+                    filename = uri2fsn(uris[0])
                 except ValueError:
                     pass
                 else:
@@ -320,7 +352,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         delete = menu.get_children()[1]
         try:
             delete.set_sensitive(len(os.listdir(directory)) == 0)
-        except OSError, err:
+        except OSError as err:
             if err.errno == errno.ENOENT:
                 model.remove(model.get_iter(path))
             return False
@@ -344,12 +376,12 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         if not dir_:
             return
 
-        dir_ = glib2fsnative(dir_)
+        dir_ = glib2fsn(dir_)
         fullpath = os.path.realpath(os.path.join(directory, dir_))
 
         try:
             os.makedirs(fullpath)
-        except EnvironmentError, err:
+        except EnvironmentError as err:
             error = "<b>%s</b>: %s" % (err.filename, err.strerror)
             qltk.ErrorMessage(
                 None, _("Unable to create folder"), error).run()
@@ -366,7 +398,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         directory = model[paths[0]][0]
         try:
             os.rmdir(directory)
-        except EnvironmentError, err:
+        except EnvironmentError as err:
             error = "<b>%s</b>: %s" % (err.filename, err.strerror)
             qltk.ErrorMessage(
                 None, _("Unable to delete folder"), error).run()
@@ -475,7 +507,7 @@ class FileSelector(Paned):
         self.__filter = filter
 
         if initial is not None:
-            assert is_fsnative(initial)
+            assert isinstance(initial, fsnative)
 
         if initial and os.path.isfile(initial):
             initial = os.path.dirname(initial)
@@ -484,7 +516,7 @@ class FileSelector(Paned):
         model = ObjectStore()
         filelist = AllTreeView(model=model)
 
-        column = TreeViewColumn(_("Songs"))
+        column = TreeViewColumn(title=_("Songs"))
         column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         render = Gtk.CellRendererPixbuf()
         render.props.xpad = 3
@@ -506,7 +538,7 @@ class FileSelector(Paned):
 
         def cell_data(column, cell, model, iter_, userdata):
             value = model.get_value(iter_)
-            cell.set_property('text', fsdecode(os.path.basename(value)))
+            cell.set_property('text', fsn2text(os.path.basename(value)))
 
         column.set_cell_data_func(render, cell_data)
 
