@@ -9,9 +9,11 @@ import re
 import os
 
 from gi.repository import Gtk
+from senf import fsn2text
 
 import quodlibet
 
+from quodlibet import _
 from quodlibet import config
 from quodlibet import qltk
 from quodlibet import util
@@ -26,10 +28,10 @@ from quodlibet.qltk.views import TreeViewColumn
 from quodlibet.qltk.cbes import ComboBoxEntrySave
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.qltk import Icons
-from quodlibet.util.path import fsdecode
 from quodlibet.util.tagsfrompath import TagsFromPattern
 from quodlibet.util.string.splitters import split_value
 from quodlibet.util import connect_obj, gdecode
+from quodlibet.compat import itervalues
 
 
 TBP = os.path.join(quodlibet.get_user_dir(), "lists", "tagpatterns")
@@ -68,7 +70,7 @@ class SplitTag(FilterCheckButton):
     _order = 1.2
 
     def filter(self, tag, value):
-        spls = config.get("editing", "split_on").decode('utf-8', 'replace')
+        spls = config.gettext("editing", "split_on")
         spls = spls.split()
         return "\n".join(split_value(value, spls))
 
@@ -92,7 +94,7 @@ class ListEntry(object):
 
     @property
     def name(self):
-        return fsdecode(self.song("~basename"))
+        return fsn2text(self.song("~basename"))
 
 
 class TagsFromPath(Gtk.VBox):
@@ -279,7 +281,7 @@ class TagsFromPath(Gtk.VBox):
         was_changed = set()
 
         all_done = False
-        for entry in ((model and model.itervalues()) or []):
+        for entry in ((model and itervalues(model)) or []):
             song = entry.song
             changed = False
             if not song.valid():
