@@ -7,22 +7,33 @@
 # published by the Free Software Foundation
 
 import re
-import sys
 
+from senf import fsn2bytes, bytes2fsn, fsnative, expanduser
+
+from quodlibet import _
 from quodlibet import app
 from quodlibet import config
 from quodlibet.qltk.notif import Task
 from quodlibet.util.dprint import print_d
-from quodlibet.util import copool
+from quodlibet.util import copool, is_windows
 
-from quodlibet.parse import Query
+from quodlibet.query import Query
 from quodlibet.qltk.songlist import SongList
 from quodlibet.util.string import split_escape, join_escape
-from quodlibet.util.path import bytes2fsnative, fsnative2bytes, fsnative
 
 
 def background_filter():
-    bg = config.get("browsers", "background").decode('utf-8')
+    """Returns a filter function for AudioFile or None if nothing should be
+    filtered.
+
+    The filter is meant to be used globally to hide songs from the main
+    library.
+
+    Returns:
+        function or None
+    """
+
+    bg = config.gettext("browsers", "background")
     if not bg:
         return
     try:
@@ -31,38 +42,74 @@ def background_filter():
         pass
 
 
-def split_scan_dirs(s):
-    """Split the value of the "scan" setting, accounting for drive letters on
-    win32."""
-    if sys.platform == "win32":
-        return filter(None, re.findall(r"[a-zA-Z]:[\\/][^:]*", s))
+def split_scan_dirs(joined_paths):
+    """Returns a list of paths
+
+    Args:
+        joined_paths (fsnative)
+    Return:
+        list
+    """
+
+    assert isinstance(joined_paths, fsnative)
+
+    if is_windows():
+        # we used to separate this config with ":", so this is tricky
+        return list(
+            filter(None, re.findall(r"[a-zA-Z]:[\\/][^:]*", joined_paths)))
     else:
-        # See Issue 1413 - allow escaped colons
-        return filter(None, split_escape(s, ":"))
+        return list(filter(None, split_escape(joined_paths, ":")))
 
 
 def get_scan_dirs():
-    dirs = split_scan_dirs(config.get("settings", "scan"))
-    return [bytes2fsnative(d) for d in dirs if d]
+    """Returns a list of paths which should be scanned
+
+    Returns:
+        list
+    """
+
+    joined_paths = bytes2fsn(config.getbytes("settings", "scan"), "utf-8")
+    return [expanduser(p) for p in split_scan_dirs(joined_paths)]
 
 
 def set_scan_dirs(dirs):
-    if sys.platform == "win32":
+    """Saves a list of fs paths which should be scanned
+
+    Args:
+        list
+    """
+
+    assert all(isinstance(d, fsnative) for d in dirs)
+
+    if is_windows():
         joined = fsnative(u":").join(dirs)
     else:
         joined = join_escape(dirs, fsnative(u":"))
-    config.set("settings", "scan", fsnative2bytes(joined))
+    config.setbytes("settings", "scan", fsn2bytes(joined, "utf-8"))
+
+
+def get_exclude_dirs():
+    """Returns a list of paths which should be ignored during scanning
+
+    Returns:
+        list
+    """
+
+    paths = split_scan_dirs(
+        bytes2fsn(config.getbytes("library", "exclude"), "utf-8"))
+    return [expanduser(p) for p in paths]
 
 
 def scan_library(library, force):
     """Start the global library re-scan
 
-    If `force` is True, reload all existing valid items.
+    Args:
+        library (Library)
+        force (bool): if True, reload all existing valid items
     """
 
     paths = get_scan_dirs()
-    exclude = split_scan_dirs(config.get("library", "exclude"))
-    exclude = [bytes2fsnative(e) for e in exclude]
+    exclude = get_exclude_dirs()
     copool.add(library.rebuild, paths, force, exclude,
                cofuncid="library", funcid="library")
 
