@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2011 Christoph Reiter
+#           2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,14 +10,19 @@ import os
 import stat
 import sys
 import errno
-import urllib
 import time
+import shutil
 
 from os.path import join, islink, abspath, dirname
 from os.path import isdir, basename, exists, splitext
 
 from quodlibet import config
 from quodlibet.util.path import find_mount_point, xdg_get_data_home
+from quodlibet.compat import pathname2url
+
+_TRASH_TMPL = """[Trash Info]
+Path={path}
+DeletionDate={date}"""
 
 
 class TrashError(EnvironmentError):
@@ -27,32 +33,61 @@ def is_sticky(path):
     return bool(os.stat(path).st_mode & stat.S_ISVTX)
 
 
-def get_fd_trash_dir(path):
-    """Returns the right trash directory for the given path."""
+def _get_fd_trash_dirs(path):
+    """Returns verified trash folders for the given path.
+
+    Returns (rootdir, filesdir, infodir), or raises a TrashError if a
+    valid trash folder structure could not be found. The returned trash
+    folders are not guaranteed to be on the same volume as the original
+    path: a fallback may be returned instead.
+
+    Returned paths are absolute. This method may create partial or
+    complete trash directory structures as part of its search.
+    """
 
     path = abspath(path)
     mount = find_mount_point(path)
     xdg_data_home = xdg_get_data_home()
-    xdg_mount = find_mount_point(xdg_data_home)
-    if mount == xdg_mount:
-        trash_home = join(xdg_data_home, "Trash")
-        return trash_home
-    else:
+    xdg_home_mount = find_mount_point(xdg_data_home)
+    trash_home = join(xdg_data_home, "Trash")
+    # Build a list of trash roots to try.
+    trash_roots = []
+    if mount != xdg_home_mount:
         root = join(mount, ".Trash")
         uid = str(os.getuid())
         if isdir(root) and not islink(root) and is_sticky(root):
-            root = join(root, uid)
+            trash_roots.append(join(root, uid))
         else:
-            root = join(mount, ".Trash-" + uid)
-        return root
+            trash_roots.append(join(mount, ".Trash-" + uid))
+    trash_roots.append(trash_home)
+    trash_roots = [abspath(r) for r in trash_roots]
+    # Try and verify each potential trash path, and create its
+    # required structure if needed.
+    for trash_root in trash_roots:
+        if path.startswith(join(trash_root, "")) or path == trash_root:
+            # Can't move files to the trash from within the trash root.
+            # But a fallback root may be OK.
+            continue  # makes things easier
+        subdirs = [join(trash_root, s) for s in ("files", "info")]
+        subdirs_valid = True
+        for subdir in subdirs:
+            if not isdir(subdir):
+                try:
+                    os.makedirs(subdir, 0o700)
+                except OSError:
+                    subdirs_valid = False
+            if not os.access(subdir, os.W_OK):
+                subdirs_valid = False
+        if subdirs_valid:
+            return tuple([trash_root] + subdirs)
+    raise TrashError("No valid trash folder exists for %r" % (path,))
 
 
 def trash_free_desktop(path):
     """Partial implementation of
     http://www.freedesktop.org/wiki/Specifications/trash-spec
 
-    No copy fallback, either it can be moved on the same FS or it failes.
-    Also doesn't work for files in the trash directory.
+    This doesn't work for files in the trash directory.
     """
 
     path = abspath(path)
@@ -60,28 +95,16 @@ def trash_free_desktop(path):
     if not exists(path):
         raise TrashError("Path %s does not exist." % path)
 
-    trash_dir = abspath(get_fd_trash_dir(path))
-
-    # to make things easier
-    if path.startswith(join(trash_dir, "")) or path == trash_dir:
-        raise TrashError("Can't move files to the trash from within the"
-                         "trash directory.")
-
-    files = join(trash_dir, "files")
-    info = join(trash_dir, "info")
-
-    for d in (files, info):
-        if not isdir(d):
-            os.makedirs(d, 0700)
+    trash_dir, files, info = _get_fd_trash_dirs(path)
 
     info_ext = ".trashinfo"
     name = basename(path)
     flags = os.O_EXCL | os.O_CREAT | os.O_WRONLY
-    mode = 0644
+    mode = 0o644
     try:
         info_path = join(info, name + info_ext)
         info_fd = os.open(info_path, flags, mode)
-    except OSError, e:
+    except OSError as e:
         if e.errno != errno.EEXIST:
             raise
         i = 2
@@ -91,7 +114,7 @@ def trash_free_desktop(path):
             info_path = join(info, temp_name + info_ext)
             try:
                 info_fd = os.open(info_path, flags, mode)
-            except OSError, e:
+            except OSError as e:
                 if e.errno != errno.EEXIST:
                     raise
                 i += 1
@@ -107,17 +130,14 @@ def trash_free_desktop(path):
 
     del_date = time.strftime("%Y-%m-%dT%H:%M:%S", time.localtime())
 
-    data = "[Trash Info]\n"
-    data += "Path=%s\n" % urllib.pathname2url(norm_path)
-    data += "DeletionDate=%s\n" % del_date
-    os.write(info_fd, data)
+    data = _TRASH_TMPL.format(path=pathname2url(norm_path), date=del_date)
+    os.write(info_fd, data.encode())
     os.close(info_fd)
 
+    target_path = join(files, name)
     try:
-        # We only move to the same file system.. so this is ok.
-        os.rename(path, join(files, name))
+        shutil.move(path, target_path)
     except OSError:
-        # In case something failes, remove the info file and raise again
         os.unlink(info_path)
         raise
 
