@@ -3,9 +3,6 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import fnmatch
-import inspect
-from math import log
 import os
 import sys
 import unittest
@@ -13,10 +10,25 @@ import tempfile
 import shutil
 import atexit
 import subprocess
+import locale
 from quodlibet.compat import PY3
-from quodlibet.util.dprint import Colorise, print_
-from quodlibet.util.path import fsnative, is_fsnative, xdg_get_cache_home
-from quodlibet.util.misc import environ
+
+try:
+    import pytest
+except ImportError:
+    module = "python3-pytest" if PY3 else "python-pytest"
+    raise SystemExit("pytest missing: sudo apt-get install %s" % module)
+
+try:
+    import xvfbwrapper
+except ImportError:
+    xvfbwrapper = None
+
+import faulthandler
+from senf import fsnative, path2fsn, environ
+
+import quodlibet
+from quodlibet.util.path import xdg_get_cache_home
 from quodlibet import util
 
 from unittest import TestCase as OrigTestCase
@@ -37,22 +49,27 @@ class TestCase(OrigTestCase):
 skip = unittest.skip
 skipUnless = unittest.skipUnless
 skipIf = unittest.skipIf
-_NO_NETWORK = False
 
 
-def skipUnlessNetwork(*args, **kwargs):
-    return skipIf(_NO_NETWORK, "no network")(*args, **kwargs)
+def is_ci():
+    """Guesses if this is being run in (Travis, maybe other) CI.
+       See https://docs.travis-ci.com/user/environment-variables
+    """
+    return os.environ.get('CI', "").lower() == 'true'
 
-
-DATA_DIR = os.path.join(util.get_module_dir(), "data")
-assert is_fsnative(DATA_DIR)
+_DATA_DIR = os.path.join(util.get_module_dir(), "data")
+assert isinstance(_DATA_DIR, fsnative)
 _TEMP_DIR = None
 
 
+def get_data_path(filename):
+    return os.path.join(_DATA_DIR, path2fsn(filename))
+
+
 def _wrap_tempfile(func):
     def wrap(*args, **kwargs):
         if kwargs.get("dir") is None and _TEMP_DIR is not None:
-            assert is_fsnative(_TEMP_DIR)
+            assert isinstance(_TEMP_DIR, fsnative)
             kwargs["dir"] = _TEMP_DIR
         return func(*args, **kwargs)
     return wrap
@@ -63,13 +80,13 @@ NamedTemporaryFile = _wrap_tempfile(tempfile.NamedTemporaryFile)
 
 def mkdtemp(*args, **kwargs):
     path = _wrap_tempfile(tempfile.mkdtemp)(*args, **kwargs)
-    assert is_fsnative(path)
+    assert isinstance(path, fsnative)
     return path
 
 
 def mkstemp(*args, **kwargs):
     fd, filename = _wrap_tempfile(tempfile.mkstemp)(*args, **kwargs)
-    assert is_fsnative(filename)
+    assert isinstance(filename, fsnative)
     return (fd, filename)
 
 
@@ -106,70 +123,6 @@ def destroy_fake_app():
     app.cover_manager = None
 
 
-class Result(unittest.TestResult):
-    TOTAL_WIDTH = 80
-    TEST_RESULTS_WIDTH = 50
-    TEST_NAME_WIDTH = TOTAL_WIDTH - TEST_RESULTS_WIDTH - 3
-    MAJOR_SEPARATOR = '=' * TOTAL_WIDTH
-    MINOR_SEPARATOR = '-' * TOTAL_WIDTH
-
-    CHAR_SUCCESS, CHAR_ERROR, CHAR_FAILURE = '+', 'E', 'F'
-
-    def __init__(self, test_name, num_tests, out=sys.stdout, failfast=False):
-        super(Result, self).__init__()
-        self.out = out
-        self.failfast = failfast
-        if hasattr(out, "flush"):
-            out.flush()
-        pref = '%s (%d): ' % (Colorise.bold(test_name), num_tests)
-        line = pref + " " * (self.TEST_NAME_WIDTH - len(test_name)
-                             - 7 - int(num_tests and log(num_tests, 10) or 0))
-        print_(line, end="")
-
-    def addSuccess(self, test):
-        unittest.TestResult.addSuccess(self, test)
-        print_(Colorise.green(self.CHAR_SUCCESS), end="")
-
-    def addError(self, test, err):
-        unittest.TestResult.addError(self, test, err)
-        print_(Colorise.red(self.CHAR_ERROR), end="")
-
-    def addFailure(self, test, err):
-        unittest.TestResult.addFailure(self, test, err)
-        print_(Colorise.red(self.CHAR_FAILURE), end="")
-
-    def printErrors(self):
-        succ = self.testsRun - (len(self.errors) + len(self.failures))
-        v = Colorise.bold("%3d" % succ)
-        cv = Colorise.green(v) if succ == self.testsRun else Colorise.red(v)
-        count = self.TEST_RESULTS_WIDTH - self.testsRun
-        print_((" " * count) + cv)
-        self.printErrorList('ERROR', self.errors)
-        self.printErrorList('FAIL', self.failures)
-
-    def printErrorList(self, flavour, errors):
-        for test, err in errors:
-            print_(self.MAJOR_SEPARATOR)
-            print_(Colorise.red("%s: %s" % (flavour, str(test))))
-            print_(self.MINOR_SEPARATOR)
-            # tracebacks can contain encoded paths, not sure
-            # what the right fix is here, so use repr
-            for line in err.splitlines():
-                print_(repr(line)[1:-1])
-
-
-class Runner(object):
-
-    def run(self, test, failfast=False):
-        suite = unittest.makeSuite(test)
-        if suite.countTestCases() == 0:
-            return 0, 0, 0
-        result = Result(test.__name__, len(suite._tests), failfast=failfast)
-        suite(result)
-        result.printErrors()
-        return len(result.failures), len(result.errors), len(suite._tests)
-
-
 def dbus_launch_user():
     """Returns a dict with env vars, or an empty dict"""
 
@@ -200,6 +153,7 @@ def dbus_kill_user(info):
 
 
 _BUS_INFO = None
+_VDISPLAY = None
 
 
 def init_test_environ():
@@ -209,7 +163,7 @@ def init_test_environ():
     any resources created.
     """
 
-    global _TEMP_DIR, _BUS_INFO
+    global _TEMP_DIR, _BUS_INFO, _VDISPLAY, _faulthandler_fobj
 
     # create a user dir in /tmp and set env vars
     _TEMP_DIR = tempfile.mkdtemp(prefix=fsnative(u"QL-TEST-"))
@@ -235,26 +189,39 @@ def init_test_environ():
     # set to new default
     environ.pop("XDG_DATA_HOME", None)
 
+    if xvfbwrapper is not None:
+        _VDISPLAY = xvfbwrapper.Xvfb()
+        _VDISPLAY.start()
+
     _BUS_INFO = None
     if os.name != "nt" and "DBUS_SESSION_BUS_ADDRESS" in environ:
         _BUS_INFO = dbus_launch_user()
         environ.update(_BUS_INFO)
 
-    # Ideally nothing should touch the FS on import, but we do atm..
-    # Get rid of all modules so QUODLIBET_USERDIR gets used everywhere.
-    for key in list(sys.modules.keys()):
-        if key.startswith('quodlibet'):
-            del(sys.modules[key])
-
-    import quodlibet
     quodlibet.init(no_translations=True, no_excepthook=True)
     quodlibet.app.name = "QL Tests"
 
+    # to get around pytest silencing
+    _faulthandler_fobj = os.fdopen(os.dup(sys.__stderr__.fileno()), "w")
+    faulthandler.enable(_faulthandler_fobj)
+
+    # try to make things the same in case a different locale is active.
+    # LANG for gettext, setlocale for number formatting etc.
+    # Note: setlocale has to be called after Gtk.init()
+    try:
+        if os.name != "nt":
+            environ["LANG"] = locale.setlocale(locale.LC_ALL, "en_US.UTF-8")
+        else:
+            environ["LANG"] = "en_US.utf8"
+            locale.setlocale(locale.LC_ALL, "english")
+    except locale.Error:
+        pass
+
 
 def exit_test_environ():
     """Call after init_test_environ() and all tests are finished"""
 
-    global _TEMP_DIR, _BUS_INFO
+    global _TEMP_DIR, _BUS_INFO, _VDISPLAY
 
     try:
         shutil.rmtree(_TEMP_DIR)
@@ -263,6 +230,10 @@ def exit_test_environ():
 
     dbus_kill_user(_BUS_INFO)
 
+    if _VDISPLAY is not None:
+        _VDISPLAY.stop()
+        _VDISPLAY = None
+
 
 # we have to do this on import so the tests work with other test runners
 # like py.test which don't know about out setup code and just import
@@ -270,16 +241,9 @@ init_test_environ()
 atexit.register(exit_test_environ)
 
 
-def unit(run=[], filter_func=None, main=False, subdirs=None,
-               strict=False, stop_first=False, no_network=False):
-
-    global _NO_NETWORK
-
-    path = util.get_module_dir()
-    if subdirs is None:
-        subdirs = []
-
-    _NO_NETWORK = no_network
+def unit(run=[], suite=None, strict=False, exitfirst=False, network=True,
+         quality=False):
+    """Returns 0 if everything passed"""
 
     # make glib warnings fatal
     if strict:
@@ -289,46 +253,30 @@ def unit(run=[], filter_func=None, main=False, subdirs=None,
             GLib.LogLevelFlags.LEVEL_ERROR |
             GLib.LogLevelFlags.LEVEL_WARNING)
 
-    suites = []
+    args = []
 
-    def discover_tests(mod):
-        for k in vars(mod):
-            value = getattr(mod, k)
+    if run:
+        args.append("-k")
+        args.append(" or ".join(run))
 
-            if value is not TestCase and \
-                    inspect.isclass(value) and issubclass(value, TestCase):
-                suites.append(value)
+    skip_markers = []
 
-    if main:
-        for name in os.listdir(path):
-            if fnmatch.fnmatch(name, "test_*.py"):
-                mod = __import__(".".join([__name__, name[:-3]]), {}, {}, [])
-                discover_tests(getattr(mod, name[:-3]))
+    if not quality:
+        skip_markers.append("quality")
 
-    if main:
-        # include plugin tests by default
-        subdirs = (subdirs or []) + ["plugin"]
+    if not network:
+        skip_markers.append("network")
 
-    for subdir in subdirs:
-        sub_path = os.path.join(path, subdir)
-        for name in os.listdir(sub_path):
-            if fnmatch.fnmatch(name, "test_*.py"):
-                mod = __import__(
-                    ".".join([__name__, subdir, name[:-3]]), {}, {}, [])
-                discover_tests(getattr(getattr(mod, subdir), name[:-3]))
+    if skip_markers:
+        args.append("-m")
+        args.append(" and ".join(["not %s" % m for m in skip_markers]))
 
-    runner = Runner()
-    failures = errors = all_ = 0
-    use_suites = filter(filter_func, suites)
-    for test in sorted(use_suites, key=repr):
-        if (not run
-                or test.__name__ in run
-                or test.__module__[11:] in run):
-            df, de, num = runner.run(test, failfast=stop_first)
-            failures += df
-            errors += de
-            all_ += num
-            if stop_first and (df or de):
-                break
+    if exitfirst:
+        args.append("-x")
 
-    return failures, errors, all_
+    if suite is None:
+        args.append("tests")
+    else:
+        args.append(os.path.join("tests", suite))
+
+    return pytest.main(args=args)
