@@ -10,11 +10,14 @@ import os
 import contextlib
 import sys
 import shutil
+import locale
+import errno
 
 from gi.repository import Gtk, Gdk
+from senf import fsnative, environ
 
 from quodlibet.qltk import find_widgets, get_primary_accel_mod
-from quodlibet.util.path import fsnative, normalize_path
+from quodlibet.util.path import normalize_path
 from quodlibet.compat import StringIO
 
 
@@ -25,6 +28,27 @@ def dummy_path(path):
     return path
 
 
+@contextlib.contextmanager
+def locale_numeric_conv(
+        decimal_point=".", grouping=[3, 3, 0], thousands_sep=","):
+    """Temporarely change number formatting conventions.
+
+    By default this uses en_US conventions.
+    """
+
+    # XXX: locale internals
+    override = locale._override_localeconv
+    old = override.copy()
+    try:
+        override["decimal_point"] = decimal_point
+        override["grouping"] = grouping
+        override["thousands_sep"] = thousands_sep
+        yield
+    finally:
+        override.clear()
+        override.update(old)
+
+
 def _send_key_click_event(widget, **kwargs):
     """Returns True if the event was handled"""
 
@@ -195,22 +219,22 @@ def visible(widget, width=None, height=None):
 
 @contextlib.contextmanager
 def preserve_environ():
-    old = os.environ.copy()
+    old = environ.copy()
     yield
     # don't touch existing values as os.environ is broken for empty
     # keys on Windows: http://bugs.python.org/issue20658
-    for key, value in os.environ.items():
+    for key, value in list(environ.items()):
         if key not in old:
-            del os.environ[key]
+            del environ[key]
     for key, value in old.items():
-        if key not in os.environ or os.environ[key] != value:
-            os.environ[key] = value
+        if key not in environ or environ[key] != value:
+            environ[key] = value
 
 
 @contextlib.contextmanager
 def capture_output():
     """
-    with capture_output as (stdout, stderr):
+    with capture_output() as (stdout, stderr):
         some_action()
     print stdout.getvalue(), stderr.getvalue()
     """
@@ -234,7 +258,7 @@ def temp_filename(*args, **kwargs):
     """Creates an empty file and removes it when done.
 
         with temp_filename() as filename:
-            with open(filename) as h:
+            with open(filename, 'w') as h:
                 h.write("foo")
             do_stuff(filename)
     """
@@ -246,7 +270,11 @@ def temp_filename(*args, **kwargs):
 
     yield filename
 
-    os.remove(filename)
+    try:
+        os.remove(filename)
+    except OSError as e:
+        if e.errno != errno.ENOENT:
+            raise
 
 
 def get_temp_copy(path):
