@@ -5,7 +5,6 @@
 # it under the terms of version 2 of the GNU General Public License as
 # published by the Free Software Foundation.
 
-import os
 import time
 
 from gi.repository import Gtk
@@ -21,7 +20,7 @@ else:
 
 
 from tests.plugin import PluginTestCase
-from tests import skipUnless, DATA_DIR
+from tests import skipUnless, get_data_path
 from quodlibet import config
 from quodlibet.formats import MusicFile
 
@@ -42,7 +41,7 @@ class TFingerprint(PluginTestCase):
 
     def test_analyze_silence(self):
         pipeline = self.mod.analyze.FingerPrintPipeline()
-        song = MusicFile(os.path.join(DATA_DIR, "silence-44-s.ogg"))
+        song = MusicFile(get_data_path("silence-44-s.ogg"))
         done = []
 
         def callback(self, *args):
@@ -60,7 +59,7 @@ class TFingerprint(PluginTestCase):
 
     def test_analyze_pool(self):
         pool = self.mod.analyze.FingerPrintPool()
-        song = MusicFile(os.path.join(DATA_DIR, "silence-44-s.ogg"))
+        song = MusicFile(get_data_path("silence-44-s.ogg"))
 
         events = []
 
