@@ -8,6 +8,7 @@
 import os
 import socket
 
+from senf import fsnative
 from gi.repository import Gtk
 
 from quodlibet.formats import AudioFile
@@ -26,21 +27,21 @@ class TMPDServer(PluginTestCase):
     def test_parse_command(self):
         parse = self.mod.main.parse_command
 
-        self.assertEqual(parse("foo bar"), ("foo", ["bar"]))
-        self.assertEqual(parse("foo\tbar"), ("foo", ["bar"]))
-        self.assertEqual(parse("foo\t bar"), ("foo", ["bar"]))
-        self.assertEqual(parse("foo\t bar quux"), ("foo", ["bar", "quux"]))
+        self.assertEqual(parse(b"foo bar"), ("foo", ["bar"]))
+        self.assertEqual(parse(b"foo\tbar"), ("foo", ["bar"]))
+        self.assertEqual(parse(b"foo\t bar"), ("foo", ["bar"]))
+        self.assertEqual(parse(b"foo\t bar quux"), ("foo", ["bar", "quux"]))
         self.assertEqual(
-            parse("foo\t bar \"q 2\" x"), ("foo", ["bar", "q 2", "x"]))
-        self.assertEqual(parse("foo 'bar  quux'"), ("foo", ["'bar", "quux'"]))
+            parse(b"foo\t bar \"q 2\" x"), ("foo", ["bar", "q 2", "x"]))
+        self.assertEqual(parse(b"foo 'bar  quux'"), ("foo", ["'bar", "quux'"]))
         self.assertEqual(
-            parse("foo \xc3\xb6\xc3\xa4\xc3\xbc"), ("foo", [u"\xf6\xe4\xfc"]))
+            parse(b"foo \xc3\xb6\xc3\xa4\xc3\xbc"), ("foo", [u"\xf6\xe4\xfc"]))
 
     def test_format_tags(self):
         format_tags = self.mod.main.format_tags
 
         def getline(key, value):
-            song = AudioFile({"~filename": "/dev/null"})
+            song = AudioFile({"~filename": fsnative(u"/dev/null")})
             song.sanitize()
             song[key] = value
             lines = format_tags(song).splitlines()
@@ -94,19 +95,32 @@ class TMPDCommands(PluginTestCase):
         self.s.send(data)
         while Gtk.events_pending():
             Gtk.main_iteration_do(True)
-        if data.strip() != "idle":
+        if data.strip() != b"idle":
             return self.s.recv(99999)
 
     def tearDown(self):
         destroy_fake_app()
         config.quit()
 
+    def test_currentsong_length(self):
+        app.player.go_to(AudioFile({
+            "~filename": fsnative(),
+            "~#length": 12.25,
+        }))
+
+        response = self._cmd(b"currentsong\n")
+        assert b"Time: 12\n" in response
+
+    def test_tagtypes(self):
+        response = self._cmd(b"tagtypes\n")
+        assert b"Time\n" not in response
+
     def test_commands(self):
         skip = ["close", "idle", "noidle"]
         cmds = [c for c in self.conn.list_commands() if c not in skip]
         for cmd in cmds:
-            self._cmd(cmd + b"\n")
+            self._cmd(cmd.encode("ascii") + b"\n")
 
     def test_idle_close(self):
         for cmd in ["idle", "noidle", "close"]:
-            self._cmd(cmd + b"\n")
+            self._cmd(cmd.encode("ascii") + b"\n")
