@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,11 +22,10 @@ class PickleTestPlugin(EventPlugin):
     PLUGIN_NAME = "This is a test"
 
     def enabled(self):
-        # http://code.google.com/p/quodlibet/issues/detail?id=1093
+        # https://github.com/quodlibet/quodlibet/issues/1093
         fd, filename = mkstemp('.shelve')
         os.close(fd)
         os.unlink(filename)
         s = shelve.open(filename)
         s["foobar"] = PickleMe()
         s.close()
-        os.unlink(filename)
