@@ -6,14 +6,16 @@
 # published by the Free Software Foundation
 
 from gi.repository import Gtk
-from quodlibet.plugins.songsmenu import SongsMenuPlugin
+from senf import fsnative
 
+from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from tests.plugin import PluginTestCase
 from quodlibet import config
 from quodlibet.qltk.songsmenu import SongsMenuPluginHandler
 from quodlibet.formats import AudioFile
-from quodlibet.util.path import fsnative
 from quodlibet.library import SongLibrary, SongLibrarian
+from quodlibet.compat import iteritems
+
 
 SONGS = [
     AudioFile({
@@ -53,7 +55,7 @@ class TPluginsSongsMenu(PluginTestCase):
         config.quit()
 
     def test_init(self):
-        for id_, plugin in self.plugins.iteritems():
+        for id_, plugin in iteritems(self.plugins):
             if self.h.plugin_handle(plugin):
                 self.h.plugin_enable(plugin)
                 self.h.handle(id_, None, None, [])
@@ -64,14 +66,14 @@ class TPluginsSongsMenu(PluginTestCase):
                       "Still worth keeping whilst we don't have unit tests "
                       "for all plugins.")
         # Ignored...
-        for id_, plugin in self.plugins.iteritems():
+        for id_, plugin in iteritems(self.plugins):
             if self.h.plugin_handle(plugin):
                 self.h.plugin_enable(plugin, None)
                 self.h.handle(id_, self.lib, self.parent, SONGS)
                 self.h.plugin_disable(plugin)
 
     def test_handles_albums(self):
-        for id_, plugin in self.plugins.iteritems():
+        for id_, plugin in iteritems(self.plugins):
             if isinstance(plugin, SongsMenuPlugin):
                 ha = plugin.handles_albums
                 self.failIf(hasattr(plugin, "plugin_single_album") and not ha)
