@@ -7,6 +7,7 @@ from tests import TestCase
 from .helper import realized
 
 from gi.repository import Gtk
+from senf import fsnative
 
 from quodlibet import config
 
@@ -20,7 +21,6 @@ from quodlibet.browsers.paned.prefs import PreferencesButton
 from quodlibet.browsers.paned.pane import Pane
 from quodlibet.formats import AudioFile
 from quodlibet.util.collection import Collection
-from quodlibet.util.path import fsnative
 from quodlibet.library import SongLibrary, SongLibrarian
 
 
@@ -151,6 +151,25 @@ class TPanedBrowser(TestCase):
         self.bar.finalize(False)
         self.bar.set_all_panes()
 
+    def test_restore_pane_width(self):
+        config.set("browsers", "panes", "artist\talbum")
+        self.bar.set_all_panes()
+
+        paned = self.bar.multi_paned.get_paned()
+        paned.set_relative(0.8)
+        self.bar.set_all_panes()
+        self.failUnlessAlmostEqual(paned.get_relative(), 0.8)
+
+    def test_make_pane_widths_equal(self):
+        config.set("browsers", "panes", "artist\talbum\t~year\t~#track")
+        self.bar.set_all_panes()
+        self.bar.make_pane_widths_equal()
+        paneds = self.bar.multi_paned._get_paneds()
+
+        self.failUnlessAlmostEqual(paneds[0].get_relative(), 1.0 / 4.0)
+        self.failUnlessAlmostEqual(paneds[1].get_relative(), 1.0 / 3.0)
+        self.failUnlessAlmostEqual(paneds[2].get_relative(), 1.0 / 2.0)
+
     def test_wide_mode(self):
         self.bar.set_all_wide_mode(True)
         self.bar.set_all_wide_mode(False)
