@@ -3,20 +3,28 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from gi.repository import Gdk, Gtk
+from senf import fsnative, fsn2uri, fsn2bytes
+
+from quodlibet import app
+from quodlibet import qltk
 from quodlibet.browsers.playlists.prefs import DEFAULT_PATTERN_TEXT
 from quodlibet.browsers.playlists.util import PLAYLISTS, parse_m3u, parse_pls
+from quodlibet.qltk.songlist import DND_QL
 from quodlibet.util.collection import FileBackedPlaylist
-from tests import TestCase, DATA_DIR, mkstemp, mkdtemp, _TEMP_DIR
+from tests import TestCase, get_data_path, mkstemp, mkdtemp, _TEMP_DIR, \
+    init_fake_app, destroy_fake_app
+from tests.gtk_helpers import MockSelData
 from .helper import dummy_path
 
 import os
 import shutil
 
 from quodlibet.browsers.playlists import PlaylistsBrowser
-from quodlibet.library import SongLibrary
+from quodlibet.library import SongFileLibrary
 import quodlibet.config
 from quodlibet.formats import AudioFile
-from quodlibet.util.path import fsnative2glib, mkdir, uri_from_path
+from quodlibet.util.path import mkdir
 from quodlibet.library.librarians import SongLibrarian
 from quodlibet.library.libraries import FileLibrary
 from tests.test_browsers_search import SONGS, TSearchBar
@@ -46,7 +54,7 @@ class TParsePlaylistMixin(object):
         os.close(h)
         with open(name, "wb") as f:
             target = self.prefix
-            target += fsnative2glib(os.path.join(DATA_DIR, "silence-44-s.ogg"))
+            target += fsn2bytes(get_data_path("silence-44-s.ogg"), "utf-8")
             f.write(target)
         list = self.Parse(name)
         os.unlink(name)
@@ -57,10 +65,10 @@ class TParsePlaylistMixin(object):
     def test_parse_onesong_uri(self):
         h, name = mkstemp()
         os.close(h)
-        target = os.path.join(DATA_DIR, "silence-44-s.ogg")
-        target = uri_from_path(target)
+        target = get_data_path("silence-44-s.ogg")
+        target = fsn2uri(target).encode("ascii")
         target = self.prefix + target
-        with open(name, "w") as f:
+        with open(name, "wb") as f:
             f.write(target)
         list = self.Parse(name)
         os.unlink(name)
@@ -71,12 +79,12 @@ class TParsePlaylistMixin(object):
 
 class TParseM3U(TParsePlaylist, TParsePlaylistMixin):
     Parse = staticmethod(parse_m3u)
-    prefix = ""
+    prefix = b""
 
 
 class TParsePLS(TParsePlaylist, TParsePlaylistMixin):
     Parse = staticmethod(parse_pls)
-    prefix = "File1="
+    prefix = b"File1="
 
 
 class TPlaylistIntegration(TestCase):
@@ -174,7 +182,9 @@ class TPlaylistsBrowser(TSearchBar):
 
         mkdir(PLAYLISTS)
 
-        self.lib = quodlibet.browsers.playlists.library = SongLibrary()
+        init_fake_app()
+
+        self.lib = quodlibet.browsers.playlists.library = SongFileLibrary()
         self.lib.librarian = SongLibrarian()
         all_songs = SONGS + [self.ANOTHER_SONG]
         for af in all_songs:
@@ -201,6 +211,7 @@ class TPlaylistsBrowser(TSearchBar):
         self.lib.destroy()
         shutil.rmtree(PLAYLISTS)
         PlaylistsBrowser.deinit(self.lib)
+        destroy_fake_app()
 
     def test_saverestore(self):
         # Flush previous signals, etc. Hmm.
@@ -251,3 +262,36 @@ class TPlaylistsBrowser(TSearchBar):
         pattern_text = self.bar.display_pattern_text
         self.failUnlessEqual(pattern_text, DEFAULT_PATTERN_TEXT)
         self.failUnless("<~name>" in pattern_text)
+
+    def test_drag_data_get(self):
+        b = self.bar
+        song = AudioFile()
+        song["~filename"] = fsnative(u"foo")
+        sel = MockSelData()
+        qltk.selection_set_songs(sel, [song])
+        b._drag_data_get(None, None, sel, DND_QL, None)
+
+    def test_deletion(self):
+        def a_delete_event():
+            ev = Gdk.Event()
+            ev.type = Gdk.EventType.KEY_PRESS
+            ev.keyval, accel_mod = Gtk.accelerator_parse("Delete")
+            ev.state = Gtk.accelerator_get_default_mod_mask() & accel_mod
+            return ev
+
+        b = self.bar
+        self._fake_browser_pack(b)
+        event = a_delete_event()
+        # This is selected in setUp()
+        first_pl = b.playlists()[0]
+        app.window.songlist.set_songs(first_pl)
+        app.window.songlist.select_by_func(lambda x: True,
+                                           scroll=False, one=True)
+        original_length = len(first_pl)
+        ret = b.key_pressed(event)
+        self.failUnless(ret, msg="Didn't simulate a delete keypress")
+        self.failUnlessEqual(len(first_pl), original_length - 1)
+
+    @staticmethod
+    def _fake_browser_pack(b):
+        app.window.get_child().pack_start(b, True, True, 0)
