@@ -3,6 +3,8 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from senf import fsnative
+
 from quodlibet.formats import AudioFile
 from tests import TestCase, init_fake_app, destroy_fake_app
 from .helper import capture_output
@@ -11,6 +13,7 @@ from gi.repository import Gtk
 
 from quodlibet import config
 from quodlibet import app
+from quodlibet.compat import text_type
 
 from quodlibet.commands import registry
 
@@ -25,6 +28,7 @@ class TCommands(TestCase):
         config.quit()
 
     def __send(self, command):
+        command = fsnative(text_type(command))
         return registry.handle_line(app, command)
 
     def test_query(self):
@@ -64,7 +68,6 @@ class TCommands(TestCase):
         for window in Gtk.Window.list_toplevels():
             if isinstance(window, LibraryBrowser):
                 window.destroy()
-        self.__send("order shuffle")
         self.__send("properties")
         self.__send("queue 1")
         self.__send("quit")
