@@ -8,6 +8,7 @@ from quodlibet.config import RatingsPrefs
 from tests import TestCase, mkstemp
 
 from quodlibet import config
+from quodlibet.compat import PY2
 
 
 class Tconfig(TestCase):
@@ -17,7 +18,7 @@ class Tconfig(TestCase):
     def test_init_garbage_file(self):
         config.quit()
 
-        garbage = "\xf1=\xab\xac"
+        garbage = b"\xf1=\xab\xac"
 
         fd, filename = mkstemp()
         os.close(fd)
@@ -50,10 +51,14 @@ class TRatingsPrefs(TestCase):
         # A little pointless, and brittle, but still.
         self.failUnlessEqual(self.prefs.number, self.initial_number)
         self.failUnlessEqual(self.prefs.precision, 1.0 / self.initial_number)
-        self.failUnlessEqual(self.prefs.full_symbol, config.INITIAL[
-            "settings"]["rating_symbol_full"].decode("utf-8"))
-        self.failUnlessEqual(self.prefs.blank_symbol, config.INITIAL[
-            "settings"]["rating_symbol_blank"].decode("utf-8"))
+        symbol_full = config.INITIAL["settings"]["rating_symbol_full"]
+        if PY2:
+            symbol_full = symbol_full.decode("utf-8")
+        self.failUnlessEqual(self.prefs.full_symbol, symbol_full)
+        symbol_blank = config.INITIAL["settings"]["rating_symbol_blank"]
+        if PY2:
+            symbol_blank = symbol_blank.decode("utf-8")
+        self.failUnlessEqual(self.prefs.blank_symbol, symbol_blank)
 
     def test_caching(self):
         self.failUnlessEqual(self.prefs.number, self.initial_number)
