@@ -30,9 +30,6 @@ class Tconst(TestCase):
         self.failIf(p.returncode)
 
         # only check for stable/dev branches, no feature branches
-        if branch == "master" or branch.startswith("quodlibet"):
+        if branch == b"master" or branch.startswith(b"quodlibet"):
+            branch = branch.decode("utf-8")
             self.failUnlessEqual(branch, const.BRANCH_NAME)
-
-    def test_authors(self):
-        # Noting that <= is subset operator on sets...
-        self.assertLessEqual(set(const.MAIN_AUTHORS), set(const.AUTHORS))
