@@ -5,7 +5,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from tests import TestCase, DATA_DIR, mkstemp
+from tests import TestCase, get_data_path, mkstemp
 
 import os
 
@@ -83,7 +83,7 @@ class TEmbeddedImages(TestCase):
         self.assertFalse(image)
 
     def test_not_an_image(self):
-        path = os.path.join(DATA_DIR, 'test-2.wma')
+        path = get_data_path('test-2.wma')
         image = EmbeddedImage.from_path(path)
         self.assertFalse(image)
 
