@@ -3,9 +3,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from tests import TestCase, skipUnless, DATA_DIR
-
-import os
+from tests import TestCase, skipUnless, get_data_path
 
 from quodlibet.formats.mod import ModFile, extensions
 
@@ -13,7 +11,7 @@ from quodlibet.formats.mod import ModFile, extensions
 @skipUnless(extensions, "ModPlug missing")
 class TModFile(TestCase):
     def setUp(self):
-        self.song = ModFile(os.path.join(DATA_DIR, 'empty.xm'))
+        self.song = ModFile(get_data_path('empty.xm'))
 
     def test_length(self):
         self.failUnlessEqual(0, self.song("~#length", 0))
