@@ -5,18 +5,21 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import os
-
-from tests import TestCase, DATA_DIR
+from tests import TestCase, get_data_path
 from quodlibet.formats.mp3 import MP3File
 
 
 class TMP3File(TestCase):
 
     def setUp(self):
-        self.song = MP3File(os.path.join(DATA_DIR, 'silence-44-s.mp3'))
-        self.song2 = MP3File(os.path.join(DATA_DIR, 'test.mp2'))
-        self.song3 = MP3File(os.path.join(DATA_DIR, 'lame.mp3'))
+        self.song = MP3File(get_data_path('silence-44-s.mp3'))
+        self.song2 = MP3File(get_data_path('test.mp2'))
+        self.song3 = MP3File(get_data_path('lame.mp3'))
+
+    def test_channels(self):
+        assert self.song("~#channels") == 2
+        assert self.song2("~#channels") == 1
+        assert self.song3("~#channels") == 2
 
     def test_length(self):
         self.assertAlmostEqual(self.song("~#length"), 3.0, 1)
@@ -41,4 +44,5 @@ class TMP3File(TestCase):
     def test_encoding(self):
         self.assertEqual(self.song("~encoding"), "")
         self.assertEqual(self.song2("~encoding"), "")
-        self.assertEqual(self.song3("~encoding"), "LAME 3.99.1+\nVBR")
+        assert self.song3("~encoding") in [
+            "LAME 3.99.1+\nVBR", "LAME 3.99.1+\nVBR\n-V 2"]
