@@ -9,7 +9,7 @@ import os
 import mutagen
 
 from quodlibet.compat import cBytesIO
-from tests import TestCase, DATA_DIR
+from tests import TestCase, get_data_path
 from quodlibet.formats.mp4 import MP4File
 from quodlibet.formats._image import EmbeddedImage
 
@@ -21,7 +21,7 @@ from .helper import get_temp_copy
 class TMP4File(TestCase):
 
     def setUp(self):
-        self.f = get_temp_copy(os.path.join(DATA_DIR, 'test.m4a'))
+        self.f = get_temp_copy(get_data_path('test.m4a'))
         self.song = MP4File(self.f)
 
     def tearDown(self):
@@ -86,6 +86,9 @@ class TMP4File(TestCase):
     def test_bitrate(self):
         self.assertEqual(self.song("~#bitrate"), 2)
 
+    def test_channels(self):
+        assert self.song("~#channels") == 2
+
     def test_bpm_rounds(self):
         self.song["bpm"] = "98.76"
         self.song.write()
@@ -93,6 +96,15 @@ class TMP4File(TestCase):
         self.assertEqual(self.song("bpm"), "99")
         self.assertEqual(self.song("~#bpm"), 99)
 
+    def test_empty_disk_trkn(self):
+        for key in ["trkn", "disk"]:
+            tag = mutagen.mp4.MP4(self.f)
+            tag[key] = []
+            tag.save()
+            tag = mutagen.mp4.MP4(self.f)
+            assert tag[key] == []
+            self.song.reload()
+
     def test_write(self):
         self.song.write()
 
@@ -102,7 +114,7 @@ class TMP4File(TestCase):
         self.assertTrue("albumartist" in self.song.can_change())
 
     def test_invalid(self):
-        path = os.path.join(DATA_DIR, 'empty.xm')
+        path = get_data_path('empty.xm')
         self.assertTrue(os.path.exists(path))
         self.assertRaises(Exception, MP4File, path)
 
