@@ -7,26 +7,30 @@
 
 import os
 
-from tests import TestCase, DATA_DIR
+from tests import TestCase, get_data_path
 from quodlibet.formats.mpc import MPCFile
 
 
 class TMPCFile(TestCase):
 
     def setUp(self):
-        self.song = MPCFile(os.path.join(DATA_DIR, 'silence-44-s.mpc'))
-        self.song2 = MPCFile(os.path.join(DATA_DIR, 'silence-44-s.sv8.mpc'))
+        self.song = MPCFile(get_data_path('silence-44-s.mpc'))
+        self.song2 = MPCFile(get_data_path('silence-44-s.sv8.mpc'))
 
     def test_length(self):
         self.assertAlmostEqual(self.song("~#length"), 0.065306, 3)
         self.assertAlmostEqual(self.song2("~#length"), 3.684716, 3)
 
+    def test_channels(self):
+        assert self.song("~#channels") == 2
+        assert self.song2("~#channels") == 2
+
     def test_bitrate(self):
         self.failUnlessEqual(self.song("~#bitrate"), 239)
         self.failUnlessEqual(self.song2("~#bitrate"), 1)
 
     def test_invalid(self):
-        path = os.path.join(DATA_DIR, 'empty.xm')
+        path = get_data_path('empty.xm')
         self.failUnless(os.path.exists(path))
         self.failUnlessRaises(Exception, MPCFile, path)
 
