@@ -5,9 +5,10 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from senf import fsnative
+
 from tests import TestCase
 
-from quodlibet.util.path import is_fsnative
 from quodlibet.formats.remote import RemoteFile
 
 
@@ -15,12 +16,12 @@ class TRemoteFile(TestCase):
 
     def test_path_types(self):
         f = RemoteFile("http://example.com")
-        self.assertTrue(is_fsnative(f["~mountpoint"]))
-        self.assertTrue(is_fsnative(f["~filename"]))
+        self.assertTrue(isinstance(f["~mountpoint"], fsnative))
+        self.assertTrue(isinstance(f["~filename"], fsnative))
 
     def test_fix_old_types(self):
         f = RemoteFile("http://example.com")
         dict.__setitem__(f, "~filename", b"foo")
-        self.assertTrue(is_fsnative(f["~filename"]))
+        self.assertTrue(isinstance(f["~filename"], fsnative))
         dict.__setitem__(f, "~filename", u"foo")
-        self.assertTrue(is_fsnative(f["~filename"]))
+        self.assertTrue(isinstance(f["~filename"], fsnative))
