@@ -7,13 +7,13 @@
 
 import os
 
-from tests import TestCase, DATA_DIR
+from tests import TestCase, get_data_path
 from quodlibet.formats.spc import SPCFile
 
 
 class TSPCFile(TestCase):
     def setUp(self):
-        self.song = SPCFile(os.path.join(DATA_DIR, 'test.spc'))
+        self.song = SPCFile(get_data_path('test.spc'))
 
     def test_tags(self):
         tags = {
@@ -36,7 +36,7 @@ class TSPCFile(TestCase):
         self.failUnless(self.song.can_change("title"))
 
     def test_invalid(self):
-        path = os.path.join(DATA_DIR, 'empty.xm')
+        path = get_data_path('empty.xm')
         self.failUnless(os.path.exists(path))
         self.failUnlessRaises(Exception, SPCFile, path)
 
