@@ -5,20 +5,21 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import os
-
-from tests import TestCase, DATA_DIR
+from tests import TestCase, get_data_path
 from quodlibet.formats.wavpack import WavpackFile
 
 
 class TWavpackFile(TestCase):
 
     def setUp(self):
-        self.song = WavpackFile(os.path.join(DATA_DIR, 'silence-44-s.wv'))
+        self.song = WavpackFile(get_data_path('silence-44-s.wv'))
 
     def test_length(self):
         self.assertAlmostEqual(self.song("~#length"), 3.68471, 3)
 
+    def test_channels(self):
+        assert self.song("~#channels") == 2
+
     def test_bitrate(self):
         self.failUnlessEqual(self.song("~#bitrate"), 76)
 
