@@ -4,7 +4,7 @@
 # published by the Free Software Foundation
 
 from quodlibet.config import RATINGS
-from tests import DATA_DIR, skipUnless, mkstemp, TestCase
+from tests import get_data_path, skipUnless, mkstemp, TestCase
 
 import os
 import sys
@@ -66,6 +66,9 @@ class TVCFileMixin(object):
         song = type(self.song)(self.filename)
         self.failUnlessEqual(song["~#rating"], 0.2)
 
+    def test_channels(self):
+        assert self.song("~#channels") == 2
+
     def test_deletes_rating(self):
         config.set("editing", "save_email", "foo@bar.org")
         self.song["~#rating"] = 0.2
@@ -158,7 +161,7 @@ class TTotalTagsBase(TestCase):
 
     def setUp(self):
         config.init()
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.ogg'))
+        self.filename = get_temp_copy(get_data_path('empty.ogg'))
 
     def tearDown(self):
         os.unlink(self.filename)
@@ -286,7 +289,7 @@ class TFLACFile(TVCFile, TVCFileMixin):
     def setUp(self):
         TVCFile.setUp(self)
 
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.flac'))
+        self.filename = get_temp_copy(get_data_path('empty.flac'))
         self.song = FLACFile(self.filename)
 
     def test_format_codec(self):
@@ -487,7 +490,7 @@ class TVCCoverMixin(object):
 class TVCCoverOgg(TVCCover, TVCCoverMixin):
     def setUp(self):
         TVCCover.setUp(self)
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.ogg'))
+        self.filename = get_temp_copy(get_data_path('empty.ogg'))
         self.MutagenType = OggVorbis
         self.QLType = OggFile
 
@@ -495,7 +498,7 @@ class TVCCoverOgg(TVCCover, TVCCoverMixin):
 class TVCCoverFlac(TVCCover, TVCCoverMixin):
     def setUp(self):
         TVCCover.setUp(self)
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.flac'))
+        self.filename = get_temp_copy(get_data_path('empty.flac'))
         self.MutagenType = FLAC
         self.QLType = FLACFile
 
@@ -503,7 +506,7 @@ class TVCCoverFlac(TVCCover, TVCCoverMixin):
 class TFlacPicture(TestCase):
     def setUp(self):
         config.init()
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.flac'))
+        self.filename = get_temp_copy(get_data_path('empty.flac'))
 
     def test_get_images(self):
         pic = Picture()
@@ -578,7 +581,7 @@ class TOggFile(TVCFile, TVCFileMixin):
     def setUp(self):
         TVCFile.setUp(self)
 
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.ogg'))
+        self.filename = get_temp_copy(get_data_path('empty.ogg'))
         self.song = OggFile(self.filename)
 
     def tearDown(self):
@@ -596,13 +599,16 @@ class TOggOpusFile(TVCFile, TVCFileMixin):
     def setUp(self):
         TVCFile.setUp(self)
 
-        self.filename = get_temp_copy(os.path.join(DATA_DIR, 'empty.opus'))
+        self.filename = get_temp_copy(get_data_path('empty.opus'))
         self.song = OggOpusFile(self.filename)
 
     def test_length(self):
         self.assertAlmostEqual(self.song("~#length"), 3.6847, 3)
         self.failUnless("opusenc" in self.song("encoder"))
 
+    def test_channels(self):
+        assert self.song("~#channels") == 2
+
     def test_format_codec(self):
         self.assertEqual(self.song("~format"), "Ogg Opus")
         self.assertEqual(self.song("~codec"), "Ogg Opus")
