@@ -8,20 +8,23 @@
 import os
 import sys
 
-from tests import TestCase, DATA_DIR, mkstemp
+from senf import fsnative, path2fsn
+
+from tests import TestCase, get_data_path, mkstemp
 from .helper import capture_output, get_temp_copy
 
 from quodlibet import config
 from quodlibet import util
-from quodlibet.util.path import fsnative
 from quodlibet.formats import MusicFile
 from quodlibet.operon.main import main as operon_main
+from quodlibet.compat import listkeys
 
 
-def call(args=None):
+def call(args):
+    args = [path2fsn(a) for a in args]
     with capture_output() as (out, err):
         try:
-            return_code = operon_main(["operon.py"] + args)
+            return_code = operon_main([path2fsn("operon.py")] + args)
         except SystemExit as e:
             return_code = e.code
 
@@ -32,13 +35,13 @@ class TOperonBase(TestCase):
     def setUp(self):
         config.init()
 
-        self.f = get_temp_copy(os.path.join(DATA_DIR, 'silence-44-s.ogg'))
-        self.f2 = get_temp_copy(os.path.join(DATA_DIR, 'silence-44-s.mp3'))
+        self.f = get_temp_copy(get_data_path('silence-44-s.ogg'))
+        self.f2 = get_temp_copy(get_data_path('silence-44-s.mp3'))
         self.s = MusicFile(self.f)
         self.s2 = MusicFile(self.f2)
 
         fd, self.f3 = mkstemp(".mp3")
-        os.write(fd, "garbage")
+        os.write(fd, b"garbage")
         os.close(fd)
 
     def tearDown(self):
@@ -99,7 +102,7 @@ class TOperonAdd(TOperonBase):
         self.check_true(["add", "tag", "value", self.f, self.f], False, False)
 
     def test_add_check(self):
-        keys = self.s.keys()
+        keys = listkeys(self.s)
         self.check_true(["add", "foo", "bar", self.f], False, False)
         self.s.reload()
         self.failUnlessEqual(self.s["foo"], "bar")
@@ -465,6 +468,17 @@ class TOperonTags(TOperonBase):
         self.check_true(["tags", "-t", "-ctag, desc"], True, False)
         self.check_false(["tags", "-t", "-cfoo"], False, True)
 
+    def test_output(self):
+        o, e = self.check_true(["tags"], True, False)
+        assert not e
+        assert "tracknumber" in o
+        assert "replaygain_album_gain" not in o
+
+        o, e = self.check_true(["tags", "-a"], True, False)
+        assert not e
+        assert "tracknumber" in o
+        assert "replaygain_album_gain" in o
+
 
 class TOperonImageExtract(TOperonBase):
     # [--dry-run] [--primary] [-d <destination>] <file> [<files>]
@@ -472,7 +486,7 @@ class TOperonImageExtract(TOperonBase):
     def setUp(self):
         super(TOperonImageExtract, self).setUp()
 
-        self.fcover = get_temp_copy(os.path.join(DATA_DIR, 'test-2.wma'))
+        self.fcover = get_temp_copy(get_data_path('test-2.wma'))
         self.cover = MusicFile(self.fcover)
 
     def tearDown(self):
@@ -536,10 +550,10 @@ class TOperonImageSet(TOperonBase):
         wide = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 150, 10)
         wide.savev(self.filename, "png", [], [])
 
-        self.fcover = get_temp_copy(os.path.join(DATA_DIR, 'test-2.wma'))
+        self.fcover = get_temp_copy(get_data_path('test-2.wma'))
         self.cover = MusicFile(self.fcover)
 
-        self.fcover2 = get_temp_copy(os.path.join(DATA_DIR, 'test-2.wma'))
+        self.fcover2 = get_temp_copy(get_data_path('test-2.wma'))
         self.cover2 = MusicFile(self.fcover2)
 
     def tearDown(self):
@@ -554,7 +568,7 @@ class TOperonImageSet(TOperonBase):
         self.check_false(["image-set", self.filename], False, True)
 
     def test_not_supported(self):
-        path = os.path.join(DATA_DIR, 'test.mid')
+        path = get_data_path('test.mid')
         out, err = self.check_false(
             ["image-set", self.filename, path], False, True)
         self.assertTrue("supported" in err)
@@ -591,7 +605,7 @@ class TOperonImageClear(TOperonBase):
 
     def setUp(self):
         super(TOperonImageClear, self).setUp()
-        self.fcover = get_temp_copy(os.path.join(DATA_DIR, 'test-2.wma'))
+        self.fcover = get_temp_copy(get_data_path('test-2.wma'))
         self.cover = MusicFile(self.fcover)
 
     def tearDown(self):
@@ -604,7 +618,7 @@ class TOperonImageClear(TOperonBase):
         self.check_false(["image-clear"], False, True)
 
     def test_not_supported(self):
-        path = os.path.join(DATA_DIR, 'test.mid')
+        path = get_data_path('test.mid')
         out, err = self.check_false(["image-clear", path], False, True)
         self.assertTrue("supported" in err)
 
