@@ -3,12 +3,13 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from tests import TestCase
-
 import os
 
+from senf import fsnative
+
+from tests import TestCase
+
 from quodlibet.formats import AudioFile
-from quodlibet.util.path import is_fsnative
 from quodlibet.pattern import (FileFromPattern, XMLFromPattern, Pattern,
     XMLFromMarkupPattern, ArbitraryExtensionFileFromPattern)
 
@@ -16,35 +17,35 @@ from quodlibet.pattern import (FileFromPattern, XMLFromPattern, Pattern,
 class _TPattern(TestCase):
 
     def setUp(self):
-        s1 = {'tracknumber': '5/6', 'artist': 'Artist', 'title': 'Title5',
-              '~filename': '/path/to/a.mp3', 'xmltest': "<&>"}
-        s2 = {'tracknumber': '6', 'artist': 'Artist', 'title': 'Title6',
-              '~filename': '/path/to/b.ogg', 'discnumber': '2',
+        s1 = {'tracknumber': u'5/6', 'artist': u'Artist', 'title': u'Title5',
+              '~filename': '/path/to/a.mp3', 'xmltest': u"<&>"}
+        s2 = {'tracknumber': u'6', 'artist': u'Artist', 'title': u'Title6',
+              '~filename': '/path/to/b.ogg', 'discnumber': u'2',
               'unislash': u"foo\uff0fbar"}
-        s3 = {'title': 'test/subdir', 'genre': '/\n/',
-              '~filename': '/one/more/a.flac', 'version': 'Instrumental'}
-        s4 = {'performer': 'a\nb', 'artist': 'foo\nbar'}
-        s5 = {'tracknumber': '7/1234', 'artist': 'Artist', 'title': 'Title7',
-              '~filename': '/path/to/e.mp3'}
-        s6 = {'artist': 'Foo', 'albumartist': 'foo.bar', 'album': 'Best Of',
-              '~filename': '/path/to/f.mp3', 'title': 'The.Final.Word'}
+        s3 = {'title': u'test/subdir', 'genre': u'/\n/',
+              '~filename': '/one/more/a.flac', 'version': u'Instrumental'}
+        s4 = {'performer': u'a\nb', 'artist': u'foo\nbar'}
+        s5 = {'tracknumber': u'7/1234', 'artist': u'Artist',
+              'title': u'Title7', '~filename': '/path/to/e.mp3'}
+        s6 = {'artist': u'Foo', 'albumartist': u'foo.bar', 'album': u'Best Of',
+              '~filename': '/path/to/f.mp3', 'title': u'The.Final.Word'}
         s7 = {'artist': u'un élève français', '~filename': '/path/to/g.mp3',
               'albumartist': u'Lee "Scratch" Perry',
-              'album': "The 'only' way!", 'comment': 'Trouble|Strife'}
-        s8 = {'tracknumber': '7/8', 'artist': 'Artist1\n\nArtist3',
-              'artistsort': 'SortA1\nSortA2',
-              'album': 'Album5', 'albumsort': 'SortAlbum5',
-              '~filename': '/path/to/g.mp3', 'xmltest': "<&>"}
+              'album': u"The 'only' way!", 'comment': u'Trouble|Strife'}
+        s8 = {'tracknumber': u'7/8', 'artist': u'Artist1\n\nArtist3',
+              'artistsort': u'SortA1\nSortA2',
+              'album': u'Album5', 'albumsort': u'SortAlbum5',
+              '~filename': '/path/to/g.mp3', 'xmltest': u"<&>"}
 
         if os.name == "nt":
-            s1["filename"] = u"C:\\path\\to\\a.mp3"
-            s2["filename"] = u"C:\\path\\to\\b.ogg"
-            s3["filename"] = u"C:\\one\\more\\a.flac"
-            s4["filename"] = u"C:\\path\\to\\a.mp3"
-            s5["filename"] = u"C:\\path\\to\\a.mp3"
-            s6["filename"] = u"C:\\path\\to\\f.mp3"
-            s7["filename"] = u"C:\\path\\to\\g.mp3"
-            s8["filename"] = u"C:\\path\\to\\h.mp3"
+            s1["~filename"] = u"C:\\path\\to\\a.mp3"
+            s2["~filename"] = u"C:\\path\\to\\b.ogg"
+            s3["~filename"] = u"C:\\one\\more\\a.flac"
+            s4["~filename"] = u"C:\\path\\to\\a.mp3"
+            s5["~filename"] = u"C:\\path\\to\\a.mp3"
+            s6["~filename"] = u"C:\\path\\to\\f.mp3"
+            s7["~filename"] = u"C:\\path\\to\\g.mp3"
+            s8["~filename"] = u"C:\\path\\to\\h.mp3"
 
         self.a = AudioFile(s1)
         self.b = AudioFile(s2)
@@ -124,8 +125,8 @@ class TPattern(_TPattern):
 
     def test_duplicate_query(self):
         pat = Pattern('<u=yes|<u=yes|x|y>|<u=yes|q|z>>')
-        self.assertEqual(pat.format(AudioFile({"u": "yes"})), "x")
-        self.assertEqual(pat.format(AudioFile({"u": "no"})), "z")
+        self.assertEqual(pat.format(AudioFile({"u": u"yes"})), "x")
+        self.assertEqual(pat.format(AudioFile({"u": u"no"})), "z")
 
     def test_tag_query_escaping(s):
         pat = Pattern('<albumartist=Lee "Scratch" Perry|matched|not matched>')
@@ -154,10 +155,16 @@ class TPattern(_TPattern):
         s.assertEquals(pat.format(s.g), 'not matched')
 
     def test_tag_internal(self):
-        pat = Pattern("<~filename='/path/to/a.mp3'|matched|not matched>")
-        self.assertEquals(pat.format(self.a), 'matched')
-        pat = Pattern("<~filename=/\\/path\\/to\\/a.mp3/|matched|not matched>")
-        self.assertEquals(pat.format(self.a), 'matched')
+        if os.name != "nt":
+            pat = Pattern("<~filename='/path/to/a.mp3'|matched|not matched>")
+            self.assertEquals(pat.format(self.a), 'matched')
+            pat = Pattern(
+                "<~filename=/\\/path\\/to\\/a.mp3/|matched|not matched>")
+            self.assertEquals(pat.format(self.a), 'matched')
+        else:
+            pat = Pattern(
+                r"<~filename='C:\\\path\\\to\\\a.mp3'|matched|not matched>")
+            self.assertEquals(pat.format(self.a), 'matched')
 
     def test_tag_query_disallowed_free_text(s):
         pat = Pattern("<The only way|matched|not matched>")
@@ -220,10 +227,10 @@ class TPattern(_TPattern):
 
     def test_unicode_with_int(s):
         song = AudioFile({"tracknumber": "5/6",
-            "title": "\xe3\x81\x99\xe3\x81\xbf\xe3\x82\x8c".decode('utf-8')})
+            "title": b"\xe3\x81\x99\xe3\x81\xbf\xe3\x82\x8c".decode('utf-8')})
         pat = Pattern('<~#track>. <title>')
         s.assertEquals(pat.format(song),
-            "5. \xe3\x81\x99\xe3\x81\xbf\xe3\x82\x8c".decode('utf-8'))
+            b"5. \xe3\x81\x99\xe3\x81\xbf\xe3\x82\x8c".decode('utf-8'))
 
 
 class _TFileFromPattern(_TPattern):
@@ -232,10 +239,13 @@ class _TFileFromPattern(_TPattern):
 
     def test_escape_slash(s):
         fpat = s._create('<~filename>')
-        s.assertTrue(fpat.format(s.a).startswith("_path_to_a.mp3"))
+        s.assertTrue(fpat.format(s.a).endswith("_path_to_a.mp3"))
 
         pat = Pattern('<~filename>')
-        s.assertTrue(pat.format(s.a).startswith("/path/to/a"))
+        if os.name != "nt":
+            s.assertTrue(pat.format(s.a).startswith("/path/to/a"))
+        else:
+            s.assertTrue(pat.format(s.a).startswith("C:\\path\\to\\a"))
 
         if os.name != "nt":
             wpat = s._create(r'\\<artist>\\ "<title>')
@@ -278,28 +288,28 @@ class _TFileFromPattern(_TPattern):
         p1 = s._create('<~filename>')
         p2 = s._create('<~dirname>_<~basename>')
         s.assertEquals(p1.format(s.a), p2.format(s.a))
-        s.assertEquals(p1.format(s.a), '_path_to_a.mp3')
+        s.assertTrue(p1.format(s.a).endswith('_path_to_a.mp3'))
         s.assertEquals(p1.format(s.b), p2.format(s.b))
-        s.assertEquals(p1.format(s.b), '_path_to_b.ogg')
+        s.assertTrue(p1.format(s.b).endswith('_path_to_b.ogg'))
         s.assertEquals(p1.format(s.c), p2.format(s.c))
-        s.assertEquals(p1.format(s.c), '_one_more_a.flac')
+        s.assertTrue(p1.format(s.c).endswith('_one_more_a.flac'))
 
     def test_long_filename(s):
         if os.name == "nt":
             a = AudioFile({"title": "x" * 300, "~filename": u"C:\\f.mp3"})
             path = s._create(u'C:\\foobar\\ä<title>\\<title>').format(a)
-            assert is_fsnative(path)
+            assert isinstance(path, fsnative)
             s.failUnlessEqual(len(path), 3 + 6 + 1 + 255 + 1 + 255)
             path = s._create(u'äüö<title><title>').format(a)
-            assert is_fsnative(path)
+            assert isinstance(path, fsnative)
             s.failUnlessEqual(len(path), 255)
         else:
             a = AudioFile({"title": "x" * 300, "~filename": "/f.mp3"})
             path = s._create(u'/foobar/ä<title>/<title>').format(a)
-            assert is_fsnative(path)
+            assert isinstance(path, fsnative)
             s.failUnlessEqual(len(path), 1 + 6 + 1 + 255 + 1 + 255)
             path = s._create(u'äüö<title><title>').format(a)
-            assert is_fsnative(path)
+            assert isinstance(path, fsnative)
             s.failUnlessEqual(len(path), 255)
 
 
@@ -307,6 +317,12 @@ class TFileFromPattern(_TFileFromPattern):
     def _create(self, string):
         return FileFromPattern(string)
 
+    def test_type(self):
+        pat = self._create('')
+        self.assertTrue(isinstance(pat.format(self.a), fsnative))
+        pat = self._create('<title>')
+        self.assertTrue(isinstance(pat.format(self.a), fsnative))
+
     def test_number_dot_title_dot(s):
         pat = s._create('<tracknumber>. <title>.')
         s.assertEquals(pat.format(s.a), '05. Title5..mp3')
@@ -319,7 +335,7 @@ class TFileFromPattern(_TFileFromPattern):
         s.assertEquals(pat.format(s.e), '0007. Title7.mp3')
 
     def test_ext_case_preservation(s):
-        x = AudioFile({'~filename': '/tmp/Xx.Flac', 'title': 'Xx'})
+        x = AudioFile({'~filename': fsnative(u'/tmp/Xx.Flac'), 'title': 'Xx'})
         # If pattern has a particular ext, preserve case of ext
         p1 = s._create('<~basename>')
         s.assertEquals(p1.format(x), 'Xx.Flac')
