@@ -44,7 +44,7 @@ class TSongWrapper(TestCase):
                  for i in range(10)]
         songs.reverse()
         songs.sort()
-        self.failUnlessEqual([s("~#track") for s in songs], range(10))
+        self.failUnlessEqual([s("~#track") for s in songs], list(range(10)))
 
     def test_needs_write_yes(self):
         self.failIf(self.wrap._needs_write)
@@ -78,9 +78,6 @@ class TSongWrapper(TestCase):
     def test_realkeys(self):
         self.failUnlessEqual(self.pwrap.realkeys(), self.psong.realkeys())
 
-    def test_website(self):
-        self.failUnlessEqual(self.pwrap.website(), self.psong.website())
-
     def test_can_change(self):
         for key in ["~foo", "title", "whee", "a test", "foo=bar", ""]:
             self.failUnlessEqual(
@@ -96,7 +93,8 @@ class TSongWrapper(TestCase):
 
     def test_dicty(self):
         self.failUnlessEqual(self.pwrap.keys(), self.psong.keys())
-        self.failUnlessEqual(self.pwrap.values(), self.psong.values())
+        self.failUnlessEqual(
+            list(self.pwrap.values()), list(self.psong.values()))
         self.failUnlessEqual(self.pwrap.items(), self.psong.items())
 
     def test_mtime(self):
