@@ -3,6 +3,8 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+import shutil
+
 from quodlibet.library import SongLibrary
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.plugins.songshelpers import any_song, each_song
@@ -35,9 +37,7 @@ class TSongsMenuPlugins(TestCase):
     def tearDown(self):
         self.library.destroy()
         self.pm.quit()
-        for f in os.listdir(self.tempdir):
-            os.remove(os.path.join(self.tempdir, f))
-        os.rmdir(self.tempdir)
+        shutil.rmtree(self.tempdir)
 
     def create_plugin(self, id='', name='', desc='', icon='',
                       funcs=None, mod=False):
