@@ -6,15 +6,26 @@
 from tests import TestCase
 
 from gi.repository import Gtk, Gdk
+from senf import fsnative, fsn2bytes
 
+from quodlibet.formats import AudioFile
 from quodlibet import qltk
+from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet import util
+from quodlibet.plugins import PluginManager
+from tests.gtk_helpers import MockSelData
 
 
 class TQltk(TestCase):
     def test_none(self):
         self.failUnless(qltk.get_top_parent(None) is None)
 
+    def test_get_fg_highlight_color(self):
+        widget = Gtk.Button()
+        color = qltk.get_fg_highlight_color(widget)
+        assert color is not None
+        assert isinstance(color, Gdk.RGBA)
+
     def test_gtp(self):
         w = Gtk.Window()
         l = Gtk.Label()
@@ -48,6 +59,11 @@ class TQltk(TestCase):
         self.failUnless(qltk.is_accel(e, "<ctrl>Return", "b"))
         self.failIf(qltk.is_accel(e, "a", "b"))
 
+    def test_is_accel_invalid(self):
+        e = Gdk.Event.new(Gdk.EventType.KEY_PRESS)
+        with self.assertRaises(ValueError):
+            qltk.is_accel(e, "NOPE")
+
     def test_is_accel_primary(self):
         e = Gdk.Event.new(Gdk.EventType.KEY_PRESS)
         e.keyval = Gdk.KEY_Return
@@ -94,3 +110,22 @@ class TQltk(TestCase):
         menu = Gtk.Menu()
         menu.append(item)
         self.assertTrue(qltk.get_menu_item_top_parent(item) is None)
+
+    def test_show_uri_with_existing_window(self):
+        PluginManager.instance = PluginManager()
+        # Force an instance
+        PluginWindow()
+        qltk.show_uri("foo", "quodlibet:///prefs/plugins/Squeezebox Output")
+        # TODO: proper assertions, etc
+
+
+class Tselection_data(TestCase):
+
+    def test_selection_set_songs(self):
+        song = AudioFile()
+        song["~filename"] = fsnative(u"foo")
+        sel = MockSelData()
+        qltk.selection_set_songs(sel, [song])
+        assert sel.data == fsn2bytes(fsnative(u"foo"), "utf-8")
+
+        assert qltk.selection_get_filenames(sel) == [fsnative(u"foo")]
