@@ -1,4 +1,8 @@
 # -*- coding: utf-8 -*-
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation
+
 from tests import TestCase
 
 from quodlibet.qltk.browser import LibraryBrowser
@@ -12,17 +16,17 @@ class TLibraryBrowser(TestCase):
 
     def test_ctr(self):
         from quodlibet.library import SongLibrary
-        from quodlibet.browsers.empty import EmptyBar
-        win = LibraryBrowser(EmptyBar, SongLibrary(), NullPlayer())
+        from quodlibet.browsers.albums import AlbumList
+        win = LibraryBrowser(AlbumList, SongLibrary(), NullPlayer())
         win.browser.emit("songs-selected", [], False)
         win.songlist.get_selection().emit("changed")
         win.destroy()
 
     def test_open(self):
-        from quodlibet.browsers.empty import EmptyBar
+        from quodlibet.browsers.search import SearchBar
         from quodlibet.library import SongLibrary
 
-        widget = LibraryBrowser.open(EmptyBar, SongLibrary(), NullPlayer())
+        widget = LibraryBrowser.open(SearchBar, SongLibrary(), NullPlayer())
         self.assertTrue(widget)
         self.assertTrue(widget.get_visible())
         widget.destroy()
