@@ -7,8 +7,7 @@
 
 from tests import TestCase
 
-from quodlibet.qltk.controls import PlayControls, VolumeMenu, SeekBar, \
-    TimeLabel
+from quodlibet.qltk.controls import PlayControls, VolumeMenu
 from quodlibet.qltk.controls import Volume
 from quodlibet.library import SongLibrary
 from quodlibet.player.nullbe import NullPlayer
@@ -32,20 +31,6 @@ class TControls(TestCase):
         w = VolumeMenu(self.p)
         w.destroy()
 
-    def test_seekbar(self):
-        w = SeekBar(self.p, self.l)
-        w.destroy()
-
     def test_volume(self):
         w = Volume(self.p)
         w.destroy()
-
-    def test_time_label(self):
-        l = TimeLabel()
-        l.set_time(42)
-        time_text = l.get_text()
-        l.set_disabled(True)
-        disabled_text = l.get_text()
-        self.assertNotEqual(time_text, disabled_text)
-        l.set_disabled(False)
-        self.assertEqual(l.get_text(), time_text)
