@@ -4,12 +4,12 @@
 # published by the Free Software Foundation
 
 from gi.repository import Gtk
+from senf import fsnative
 
 from tests import TestCase
 
 from quodlibet import config
 from quodlibet.formats import AudioFile
-from quodlibet.util.path import fsnative
 from quodlibet.qltk.delete import DeleteDialog, TrashDialog, TrashMenuItem
 
 SONG = AudioFile({"~filename": fsnative(u"/dev/null")})
