@@ -11,6 +11,7 @@ from gi.repository import Gtk
 
 from quodlibet.qltk.models import ObjectStore, ObjectModelFilter
 from quodlibet.qltk.models import ObjectModelSort, ObjectTreeStore
+from quodlibet.compat import cmp, xrange
 
 
 class _TObjectStoreMixin(object):
@@ -21,7 +22,7 @@ class _TObjectStoreMixin(object):
         m = self.Store()
         for i in range(10):
             m.append(row=[i])
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
     def test_column_count(self):
         m = self.Store()
@@ -31,13 +32,13 @@ class _TObjectStoreMixin(object):
         m = self.Store()
         for i in reversed(range(10)):
             m.insert(0, row=[i])
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
     def test_prepend(self):
         m = self.Store()
         for i in reversed(range(10)):
             m.prepend(row=[i])
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
     def test_insert_before(self):
         m = self.Store()
@@ -109,7 +110,7 @@ class TObjectStore(TestCase, _TObjectStoreMixin):
     def test_append_many(self):
         m = ObjectStore()
         m.append_many(range(10))
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
     def test_append_many_set(self):
         m = ObjectStore()
@@ -120,15 +121,15 @@ class TObjectStore(TestCase, _TObjectStoreMixin):
     def test_iter_append_many(self):
         m = ObjectStore()
         iters = list(m.iter_append_many(range(10)))
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
         values = [m.get_value(i) for i in iters]
-        self.failUnlessEqual(values, range(10))
+        self.failUnlessEqual(values, list(range(10)))
 
     def test_iter_append_many_iterable_int(self):
         m = ObjectStore()
         for x in m.iter_append_many((i for i in xrange(10))):
             pass
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
     def test_iter_append_many_iterable_object(self):
         objects = [object() for i in xrange(10)]
@@ -150,21 +151,21 @@ class TObjectStore(TestCase, _TObjectStoreMixin):
         m.append(row=[42])
         m.append(row=[24])
         m.insert_many(1, range(10))
-        self.failUnlessEqual([r[0] for r in m], [42] + range(10) + [24])
+        self.failUnlessEqual([r[0] for r in m], [42] + list(range(10)) + [24])
 
     def test_insert_many_append(self):
         m = ObjectStore()
         m.insert_many(-1, range(10))
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
         m = ObjectStore()
         m.insert_many(99, range(10))
-        self.failUnlessEqual([r[0] for r in m], range(10))
+        self.failUnlessEqual([r[0] for r in m], list(range(10)))
 
     def test_itervalues(self):
         m = ObjectStore()
         m.insert_many(0, range(10))
-        self.failUnlessEqual(range(10), list(m.itervalues()))
+        self.failUnlessEqual(list(range(10)), list(m.itervalues()))
 
     def test_itervalues_empty(self):
         m = ObjectStore()
@@ -391,7 +392,7 @@ class TObjectModelFilter(TestCase):
         m = ObjectStore()
         f = ObjectModelFilter(child_model=m)
         m.insert_many(0, range(10))
-        self.failUnlessEqual(range(10), list(f.itervalues()))
+        self.failUnlessEqual(list(range(10)), list(f.itervalues()))
 
     def test_filter(self):
         m = ObjectStore()
@@ -405,7 +406,7 @@ class TObjectModelFilter(TestCase):
 
         f.set_visible_func(filter_func)
         f.refilter()
-        self.failUnlessEqual(range(0, 10, 2), list(f.itervalues()))
+        self.failUnlessEqual(list(range(0, 10, 2)), list(f.itervalues()))
 
 
 class TObjectModelSort(TestCase):
@@ -414,7 +415,7 @@ class TObjectModelSort(TestCase):
         m = ObjectStore()
         f = ObjectModelSort(model=m)
         m.insert_many(0, range(10))
-        self.failUnlessEqual(range(10), list(f.itervalues()))
+        self.failUnlessEqual(list(range(10)), list(f.itervalues()))
 
     def test_sort(self):
         m = ObjectStore()
