@@ -7,6 +7,7 @@ from tests import TestCase
 from .helper import visible
 
 from gi.repository import Gtk
+from senf import devnull
 
 from quodlibet.qltk.songlistcolumns import create_songlist_column
 from quodlibet.qltk.songmodel import PlaylistModel
@@ -25,7 +26,7 @@ class TSongListColumns(TestCase):
         view = Gtk.TreeView()
         model = PlaylistModel()
         view.set_model(model)
-        song = AudioFile({"~filename": "/dev/null", "~#rating": 0.6666})
+        song = AudioFile({"~filename": devnull, "~#rating": 0.6666})
         song.update(kwargs)
         model.append(row=[song])
         view.append_column(column)
