@@ -8,9 +8,9 @@ from tests import TestCase
 from gi.repository import Gtk
 
 from quodlibet.player.nullbe import NullPlayer
-from quodlibet.formats import AudioFile
 from quodlibet.qltk.songmodel import PlaylistModel, PlaylistMux
-from quodlibet.qltk.playorder import ORDERS, Order
+from quodlibet.qltk.playorder import Order, OrderShuffle, OrderInOrder, \
+    RepeatSongForever, RepeatListForever
 
 
 def do_events():
@@ -50,10 +50,10 @@ class TPlaylistModel(TestCase):
         self.failUnless(self.pl.is_empty())
 
     def test_get(self):
-        self.assertEqual(self.pl.get(), range(10))
+        self.assertEqual(self.pl.get(), list(range(10)))
         self.pl.set(range(12))
         Gtk.main_iteration_do(False)
-        self.assertEqual(self.pl.get(), range(12))
+        self.assertEqual(self.pl.get(), list(range(12)))
 
     def test_next(self):
         self.pl.next()
@@ -109,70 +109,60 @@ class TPlaylistModel(TestCase):
         self.pl.next()
         self.failUnlessEqual(self.pl.current, 9)
 
-    def test_next_repeat(self):
-        self.pl.repeat = True
-        self.pl.go_to(3)
-        for i in range(9):
-            self.pl.next()
-        self.assertEqual(self.pl.current, 2)
-        for i in range(12):
-            self.pl.next()
-        self.assertEqual(self.pl.current, 4)
+    def test_next_at_end_finishes(self):
+        self.pl.go_to(9)
+        self.pl.next()
+        self.assertEqual(self.pl.current, None)
 
     def test_shuffle(self):
-        self.pl.order = ORDERS[1](self.pl)
+        self.pl.order = OrderShuffle()
         for i in range(5):
-            numbers = [self.pl.current for i in range(10)
+            history = [self.pl.current for _ in range(10)
                        if self.pl.next() or True]
-            self.assertNotEqual(numbers, range(10))
-            numbers.sort()
-            self.assertEqual(numbers, range(10))
+            self.assertNotEqual(history, list(range(10)))
+            self.assertEqual(sorted(history), list(range(10)))
             self.pl.next()
             self.assertEqual(self.pl.current, None)
+            self.pl.order.reset(self.pl)
 
-    def test_weighted(self):
-        self.pl.order = ORDERS[2](self.pl)
-        r0 = AudioFile({'~#rating': 0})
-        r1 = AudioFile({'~#rating': 1})
-        r2 = AudioFile({'~#rating': 2})
-        r3 = AudioFile({'~#rating': 3})
-        self.pl.set([r0, r1, r2, r3])
-        Gtk.main_iteration_do(False)
-        songs = [self.pl.current for i in range(1000)
-                 if self.pl.next() or True]
-        self.assert_(songs.count(r1) > songs.count(r0))
-        self.assert_(songs.count(r2) > songs.count(r1))
-        self.assert_(songs.count(r3) > songs.count(r2))
+    def test_shuffle_repeat_forever(self):
+        self.pl.order = RepeatSongForever(OrderShuffle())
+        old = self.pl.current
+        for i in range(5):
+            self.pl.next_ended()
+            self.assertEqual(self.pl.current, old)
 
     def test_shuffle_repeat(self):
-        self.pl.order = ORDERS[1](self.pl)
-        self.pl.repeat = True
-        numbers = [self.pl.current for i in range(30)
-                   if self.pl.next() or True]
-        allnums = range(10) * 3
-        allnums.sort()
+        self.pl.order = RepeatListForever(OrderShuffle())
+        numbers = [self.pl.current for _ in range(30)
+                   if self.pl.next_ended() or True]
+        allnums = sorted(list(range(10)) * 3)
         self.assertNotEqual(numbers, allnums)
         numbers.sort()
         self.assertEqual(numbers, allnums)
 
-    def test_onesong(self):
+    def test_repeat_song_repeats_on_end(self):
+        self.pl.order = RepeatSongForever(OrderInOrder())
         self.pl.go_to(3)
-        self.pl.order = ORDERS[3](self.pl)
         self.failUnlessEqual(self.pl.current, 3)
-        self.pl.next()
-        self.failUnlessEqual(self.pl.current, 4)
         self.pl.next_ended()
-        self.failUnlessEqual(self.pl.current, None)
+        self.failUnlessEqual(self.pl.current, 3)
 
-    def test_onesong_repeat(self):
+    def test_repeat_song_uses_underlying_on_explicit(self):
+        self.pl.order = RepeatSongForever(OrderInOrder())
         self.pl.go_to(3)
-        self.pl.order = ORDERS[3](self.pl)
-        self.pl.repeat = True
+        self.pl.next()
+        self.failUnlessEqual(self.pl.current, 4)
+
+    def test_repeat_all_cycles_playlist(self):
+        self.pl.go_to(3)
+        self.pl.order = RepeatListForever(OrderInOrder())
         self.failUnlessEqual(self.pl.current, 3)
         self.pl.next()
         self.failUnlessEqual(self.pl.current, 4)
-        self.pl.next_ended()
-        self.failUnlessEqual(self.pl.current, 4)
+        for i in range(9):
+            self.pl.next_ended()
+        self.failUnlessEqual(self.pl.current, 3)
 
     def test_previous(self):
         self.pl.go_to(2)
@@ -193,7 +183,7 @@ class TPlaylistModel(TestCase):
         self.failUnlessEqual(self.pl.current, 10)
 
     def test_go_to_order(self):
-        self.pl.order = ORDERS[1](self.pl)
+        self.pl.order = OrderShuffle()
         for i in range(5):
             self.pl.go_to(5)
             self.failUnlessEqual(self.pl.current, 5)
@@ -211,7 +201,7 @@ class TPlaylistModel(TestCase):
             def set_implicit(self, playlist, iter):
                 return playlist.iter_next(playlist.iter_next(iter))
 
-        self.pl.order = SetOrder(self.pl)
+        self.pl.order = SetOrder()
         self.failUnlessEqual(self.pl[self.pl.go_to(5, True)][0], 6)
         self.failUnlessEqual(self.pl[self.pl.go_to(5, False)][0], 7)
 
@@ -229,7 +219,7 @@ class TPlaylistModel(TestCase):
         self.failUnlessEqual(self.pl.current, 0)
 
     def test_reset_order(self):
-        self.pl.order = ORDERS[0](self.pl)
+        self.pl.order = OrderInOrder()
         self.pl.go_to(5)
         self.failUnlessEqual(self.pl.current, 5)
         self.pl.reset()
@@ -244,8 +234,7 @@ class TPlaylistModel(TestCase):
 
     def test_next_nosong_536(self):
         self.pl.go_to(1)
-        self.pl.repeat = True
-        self.pl.order = ORDERS[1](self.pl)
+        self.pl.order = OrderShuffle()
         self.pl.set([])
         Gtk.main_iteration_do(False)
         self.pl.next()
@@ -276,7 +265,7 @@ class TPlaylistMux(TestCase):
         do_events()
         self.failUnless(self.mux.current is None)
         songs = [self.next() for i in range(10)]
-        self.failUnlessEqual(songs, range(10))
+        self.failUnlessEqual(songs, list(range(10)))
         self.next()
         self.failUnless(self.mux.current is None)
 
@@ -285,7 +274,7 @@ class TPlaylistMux(TestCase):
         do_events()
         self.failUnless(self.mux.current is None)
         songs = [self.next() for i in range(10)]
-        self.failUnlessEqual(songs, range(10))
+        self.failUnlessEqual(songs, list(range(10)))
         self.next()
         self.failUnless(self.mux.current is None)
 
@@ -295,7 +284,7 @@ class TPlaylistMux(TestCase):
         do_events()
         self.failUnless(self.mux.current is None)
         songs = [self.next() for i in range(10)]
-        self.failUnlessEqual(songs, range(10))
+        self.failUnlessEqual(songs, list(range(10)))
         self.next()
         self.failUnless(self.mux.current is None)
 
@@ -366,7 +355,7 @@ class TPlaylistMux(TestCase):
             self.pl.set([1])
             do_events()
             self.failUnless(self.mux.current is None)
-            self.q.order = ORDERS[1](self.pl)
+            self.q.order = OrderShuffle()
             self.failUnless(self.next() == 1)
             self.q.set([10, 11])
             do_events()
@@ -395,7 +384,7 @@ class TPlaylistMux(TestCase):
 
     def test_queue(self):
         self.mux.enqueue(range(40))
-        self.failUnlessEqual(list(self.q.itervalues()), range(40))
+        self.failUnlessEqual(list(self.q.itervalues()), list(range(40)))
 
     def test_queue_move_entry(self):
         self.q.set(range(10))
