@@ -3,13 +3,15 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from senf import fsnative
+
 from tests import TestCase
 
 from quodlibet.formats import AudioFile
 from quodlibet.library import SongLibrary
-from quodlibet.util.path import fsnative
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet import config
+from quodlibet.compat import text_type
 import quodlibet.player
 
 
@@ -23,7 +25,7 @@ class TSongsMenu(TestCase):
         self.songs = [AudioFile({"title": x}) for x in
                       ["song1", "song2", "song3"]]
         for song in self.songs:
-            song.sanitize(fsnative(unicode(song["title"])))
+            song.sanitize(fsnative(text_type(song["title"])))
 
     def test_empty(self):
         self.menu = SongsMenu(self.library, self.songs, plugins=False,
@@ -73,7 +75,8 @@ class TSongsMenu(TestCase):
             queue=False, devices=True, remove=False, delete=False, edit=False,
             ratings=False)
         from quodlibet import browsers
-        if browsers.media.MediaDevices in browsers.browsers:
+        from quodlibet.browsers.media import MediaDevices
+        if MediaDevices in browsers.browsers and len(MediaDevices.devices()):
             self.failUnlessEqual(1, len(self.menu))
         else:
             self.failUnlessEqual(0, len(self.menu))
