@@ -101,8 +101,8 @@ class TFSInterface(TestCase):
     def test_start(self):
         self.p.emit('song_started', AudioFile({"woo": "bar", "~#length": 10}))
         self.do()
-        with open(self.filename) as h:
-            self.failUnless("woo=bar\n" in h.read())
+        with open(self.filename, "rb") as h:
+            self.failUnless(b"woo=bar\n" in h.read())
 
     def test_song_ended(self):
         self.p.emit('song-started', AudioFile({"woo": "bar", "~#length": 10}))
