@@ -2,13 +2,16 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
+import time
 
-from tests import TestCase
+from senf import fsnative
 
 from quodlibet import config
+from quodlibet.compat import xrange
+from quodlibet.formats import AudioFile
 from quodlibet.query import Query, QueryType
 from quodlibet.query import _match as match
-from quodlibet.formats import AudioFile
+from tests import TestCase, skip
 
 
 class TQuery_is_valid(TestCase):
@@ -156,36 +159,74 @@ class TQuery(TestCase):
     def setUp(self):
         config.init()
         self.s1 = AudioFile(
-            {"album": "I Hate: Tests", "artist": "piman", "title": "Quuxly",
-             "version": "cake mix", "~filename": "/dir1/foobar.ogg",
+            {"album": u"I Hate: Tests", "artist": u"piman", "title": u"Quuxly",
+             "version": u"cake mix",
+             "~filename": fsnative(u"/dir1/foobar.ogg"),
              "~#length": 224, "~#skipcount": 13, "~#playcount": 24,
-             "date": "2007-05-24"})
+             "date": u"2007-05-24"})
         self.s2 = AudioFile(
-            {"album": "Foo the Bar", "artist": "mu", "title": "Rockin' Out",
-             "~filename": "/dir2/something.mp3", "tracknumber": "12/15"})
+            {"album": u"Foo the Bar", "artist": u"mu", "title": u"Rockin' Out",
+             "~filename": fsnative(u"/dir2/something.mp3"),
+             "tracknumber": u"12/15"})
 
-        self.s3 = AudioFile(
-            {"artist": "piman\nmu",
-             "~filename": "/test/\xc3\xb6\xc3\xa4\xc3\xbc/fo\xc3\xbc.ogg"})
-        self.s4 = AudioFile({"title": u"Ångström", "utf8": "Ångström"})
-        self.s5 = AudioFile({"title": "oh&blahhh", "artist": "!ohno"})
+        self.s3 = AudioFile({
+            "artist": u"piman\nmu",
+            "~filename": fsnative(u"/test/\xf6\xe4\xfc/fo\xfc.ogg"),
+            "~mountpoint": fsnative(u"/bla/\xf6\xe4\xfc/fo\xfc"),
+        })
+        self.s4 = AudioFile({"title": u"Ångström", "utf8": u"Ångström"})
+        self.s5 = AudioFile({"title": u"oh&blahhh", "artist": u"!ohno"})
 
     def tearDown(self):
         config.quit()
 
+    def test_basic_tag(self):
+        assert Query("album=foo").search(self.s2)
+        assert not Query("album=.").search(self.s2)
+        assert Query("album=/./").search(self.s2)
+
+    def test_inequality(self):
+        self.failUnless(Query("album!=foo").search(self.s1))
+        self.failIf(Query("album!=foo").search(self.s2))
+
+    @skip("Enable for basic benchmarking of Query")
+    def test_inequality_performance(self):
+        t = time.time()
+        for i in xrange(500):
+            # Native assert is a bit lighter...
+            assert Query("album!=foo the bar").search(self.s1)
+            assert Query("album=foo the bar").search(self.s2)
+            assert Query("foo the bar").search(self.s2)
+            assert not Query("foo the bar").search(self.s1)
+        us = (time.time() - t) * 1000000 / ((i + 1) * 4)
+        print("Blended Query searches average %.0f μs" % us)
+
+    @skip("Enable for basic benchmarking of Query")
+    def test_inequality_equalish_performance(self):
+        t0 = time.time()
+        repeats = 2000
+        for i in xrange(repeats):
+            assert Query("album!=foo the bar").search(self.s1)
+        ineq_time = (time.time() - t0)
+        t1 = time.time()
+        for i in xrange(repeats):
+            assert Query("album=!foo the bar").search(self.s1)
+        not_val_time = (time.time() - t1)
+        self.assertAlmostEqual(ineq_time, not_val_time, places=1)
+
     def test_repr(self):
         query = Query("foo = bar", [])
         self.assertEqual(
-            repr(query),
-            "<Query string=u'foo = bar' type=QueryType.VALID star=[]>")
+            repr(query).replace("u'", "'"),
+            "<Query string='foo = bar' type=QueryType.VALID star=[]>")
 
         query = Query("bar", ["foo"])
         self.assertEqual(
-            repr(query),
-            "<Query string=u'&(/bar/d)' type=QueryType.TEXT star=['foo']>")
+            repr(query).replace("u'", "'"),
+            "<Query string='&(/bar/d)' type=QueryType.TEXT star=['foo']>")
 
     def test_2007_07_27_synth_search(self):
-        song = AudioFile({"~filename": "foo/64K/bar.ogg"})
+        song = AudioFile({"~filename": fsnative(u"foo/64K/bar.ogg")})
         query = Query("~dirname = !64K")
         self.failIf(query.search(song), "%r, %r" % (query, song))
 
@@ -203,6 +244,12 @@ class TQuery(TestCase):
         self.failUnless(f(self.s1))
         self.failUnless(f(self.s2))
 
+    def test_re_escape(self):
+        af = AudioFile({"foo": "\""})
+        assert Query('foo="\\""').search(af)
+        af = AudioFile({"foo": "/"})
+        assert Query('foo=/\\//').search(af)
+
     def test_not(self):
         for s in ["album = !hate", "artist = !pi"]:
             self.failIf(Query(s).search(self.s1))
@@ -338,12 +385,14 @@ class TQuery(TestCase):
         tests = [inter | tag, tag | tag, neg | neg, tag | inter, neg | union,
             union | union, inter | inter, numcmp | numcmp, numcmp | union]
 
-        self.failIf(filter(lambda x: not isinstance(x, match.Union), tests))
+        self.failIf(
+            list(filter(lambda x: not isinstance(x, match.Union), tests)))
 
         tests = [inter & tag, tag & tag, neg & neg, tag & inter, neg & union,
             union & union, inter & inter, numcmp & numcmp, numcmp & inter]
 
-        self.failIf(filter(lambda x: not isinstance(x, match.Inter), tests))
+        self.failIf(
+            list(filter(lambda x: not isinstance(x, match.Inter), tests)))
 
         self.assertTrue(isinstance(-neg, match.Tag))
 
@@ -385,11 +434,19 @@ class TQuery(TestCase):
         self.failUnless(Query(u"filename=foü.ogg").search(self.s3))
         self.failUnless(Query(u"filename=öä").search(self.s3))
 
+    def test_mountpoint_utf8_fallback(self):
+        self.failUnless(Query(u"mountpoint=foü").search(self.s3))
+        self.failUnless(Query(u"mountpoint=öä").search(self.s3))
+
+    def test_mountpoint_no_value(self):
+        af = AudioFile({"~filename": fsnative(u"foo")})
+        assert not Query(u"~mountpoint=bla").search(af)
+
     def test_star_numeric(self):
         self.assertRaises(ValueError, Query, u"foobar", star=["~#mtime"])
 
     def test_match_diacriticals_explcit(self):
-        self.failIf(Query(u'title=angstrom').search(self.s4))
+        assert Query(u'title=angstrom').search(self.s4)
         self.failIf(Query(u'title="Ångstrom"').search(self.s4))
         self.failUnless(Query(u'title="Ångstrom"d').search(self.s4))
         self.failUnless(Query(u'title=Ångström').search(self.s4))
@@ -424,6 +481,7 @@ class TQuery(TestCase):
         self.failUnless(Query("#(40 seconds < length/5 < 1 minute)")
                         .search(self.s1))
         self.failUnless(Query("#(2+3 * 5 = 17)").search(self.s1))
+        self.failUnless(Query("#(playcount / 0 > 0)").search(self.s1))
 
         self.failIf(Query("#(track + 1 != 13)").search(self.s2))
 
