@@ -3,10 +3,13 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from . import TestCase
+from senf import fsn2bytes, bytes2fsn
+
+from . import TestCase, skipIf
 from .helper import temp_filename
 
 from quodlibet.remote import QuodLibetUnixRemote
+from quodlibet.util import is_windows
 
 
 class Mock(object):
@@ -19,6 +22,7 @@ class Mock(object):
         return self.resp
 
 
+@skipIf(is_windows(), "unix only")
 class TUnixRemote(TestCase):
 
     def test_fifo(self):
@@ -26,13 +30,14 @@ class TUnixRemote(TestCase):
         remote = QuodLibetUnixRemote(None, mock)
         remote._callback(b"foo\n")
         remote._callback(b"bar\nbaz")
-        self.assertEqual(mock.lines, [b"foo", b"bar", b"baz"])
+        self.assertEqual(
+            mock.lines, [bytes2fsn(b, None) for b in [b"foo", b"bar", b"baz"]])
 
     def test_response(self):
         with temp_filename() as fn:
-            mock = Mock(resp=b"resp")
+            mock = Mock(resp=bytes2fsn(b"resp", None))
             remote = QuodLibetUnixRemote(None, mock)
-            remote._callback(b"\x00foo\x00%s\x00" % fn)
-            self.assertEqual(mock.lines, [b"foo"])
+            remote._callback(b"\x00foo\x00" + fsn2bytes(fn, None) + b"\x00")
+            self.assertEqual(mock.lines, [bytes2fsn(b"foo", None)])
             with open(fn, "rb") as h:
                 self.assertEqual(h.read(), b"resp")
