@@ -8,6 +8,7 @@
 from quodlibet import config
 from quodlibet.browsers.soundcloud.api import SoundcloudApiClient
 from quodlibet.browsers.soundcloud.library import SoundcloudLibrary
+from quodlibet.compat import listvalues
 from tests import TestCase
 
 PERMALINK = "https://soundcloud.com/"
@@ -86,7 +87,7 @@ class TSoundcloudLibrary(TestCase):
     def test_parse(self):
         lib = self.lib
         lib.query_with_refresh("dummy search")
-        songs = lib._contents.values()
+        songs = listvalues(lib._contents)
         self.failUnlessEqual(len(songs), 1)
         s = songs[0]
         self.failUnlessEqual(s("artist"), "Kerstin Eden")
@@ -96,12 +97,12 @@ class TSoundcloudLibrary(TestCase):
         self.failUnlessEqual(s("~#favoritings_count"), 882)
         self.failUnlessEqual(s("~#rating"), 1.0)
         self.failUnlessEqual(s("~#playcount"), 4)
-        self.failUnlessAlmostEqual(s("~#bitrate"), 319)
+        assert int(s("~#bitrate")) == 319
 
     def test_artwork_url(self):
         lib = SoundcloudLibrary(self.FakeClient())
         lib.query_with_refresh("")
-        s = lib._contents.values()[0]
+        s = listvalues(lib._contents)[0]
         self.failUnlessEqual(
             s("artwork_url"),
             "https://i1.sndcdn.com/artworks-000108682375-q4j7y6-t500x500.jpg")
