@@ -10,27 +10,32 @@ import sys
 import threading
 import traceback
 import time
+import logging
 
+from senf import getcwd, fsnative, fsn2bytes, bytes2fsn, mkdtemp
+
+from quodlibet import _
 from quodlibet.compat import text_type, PY2
-from quodlibet.config import HardCodedRatingsPrefs
+from quodlibet.config import HardCodedRatingsPrefs, DurationFormat
 from quodlibet import config
 from quodlibet import util
-from quodlibet.util.dprint import print_exc, format_exception, extract_tb
-from quodlibet.util import format_time_long as f_t_l
+from quodlibet.util.dprint import print_exc, format_exception, extract_tb, \
+    PrintHandler
+from quodlibet.util import format_time_long as f_t_l, format_time_preferred, \
+    format_time_display, format_time_seconds
 from quodlibet.util import re_escape
 from quodlibet.util.library import set_scan_dirs, get_scan_dirs
-from quodlibet.util.path import is_fsnative, fsnative2glib, glib2fsnative, \
+from quodlibet.util.path import fsn2glib, glib2fsn, \
     parse_xdg_user_dirs, xdg_get_system_data_dirs, escape_filename, \
     strip_win32_incompat_from_path, xdg_get_cache_home, environ, \
     xdg_get_data_home, unexpand, expanduser, xdg_get_user_dirs, \
-    xdg_get_config_home, fsnative, bytes2fsnative, fsnative2bytes, \
-    get_temp_cover_file, mkdir, getcwd, mtime
+    xdg_get_config_home, get_temp_cover_file, mkdir, mtime
 from quodlibet.util.string import decode, encode, split_escape, join_escape
 from quodlibet.util.string.splitters import split_people, split_title, \
     split_album
 
 from . import TestCase, skipIf
-from .helper import capture_output
+from .helper import capture_output, locale_numeric_conv
 
 
 is_win = os.name == "nt"
@@ -45,20 +50,26 @@ class Tmkdir(TestCase):
 
     def test_manydeep(self):
         self.failUnless(not os.path.isdir("nonext"))
-        mkdir("nonext/test/test2/test3")
+        t = mkdtemp()
+        path = os.path.join(t, "nonext", "test", "test2", "test3")
+        mkdir(path)
         try:
-            self.failUnless(os.path.isdir("nonext/test/test2/test3"))
+            self.failUnless(os.path.isdir(path))
         finally:
-            os.rmdir("nonext/test/test2/test3")
-            os.rmdir("nonext/test/test2")
-            os.rmdir("nonext/test")
-            os.rmdir("nonext")
+            os.rmdir(path)
+            path = os.path.dirname(path)
+            os.rmdir(path)
+            path = os.path.dirname(path)
+            os.rmdir(path)
+            path = os.path.dirname(path)
+            os.rmdir(path)
+            os.rmdir(t)
 
 
 class Tgetcwd(TestCase):
 
     def test_Tgetcwd(self):
-        self.assertTrue(is_fsnative(getcwd()))
+        self.assertTrue(isinstance(getcwd(), fsnative))
 
 
 class Tmtime(TestCase):
@@ -70,6 +81,28 @@ class Tmtime(TestCase):
         self.failUnlessEqual(mtime("/dev/doesnotexist"), 0)
 
 
+class Tget_locale_encoding(TestCase):
+
+    def test_main(self):
+        assert isinstance(util.get_locale_encoding(), str)
+
+
+class Tformat_locale(TestCase):
+
+    def test_format_int_locale(self):
+        assert isinstance(util.format_int_locale(1024), text_type)
+
+    def test_format_float_locale(self):
+        assert isinstance(util.format_float_locale(1024.1024), text_type)
+
+    def test_format_time_seconds(self):
+        assert isinstance(util.format_time_seconds(1024), text_type)
+
+        with locale_numeric_conv():
+            assert format_time_seconds(1024) == "1,024 seconds"
+            assert format_time_seconds(1) == "1 second"
+
+
 class Tunexpand(TestCase):
     d = expanduser("~")
     u = unexpand(d)
@@ -138,11 +171,11 @@ class Tformat_rating(TestCase):
         self.failUnlessEqual(util.format_rating(0.8), "11110")
         self.failUnlessEqual(util.format_rating(1.0), "11111")
         # A bit arbitrary, but standard behaviour
-        self.failUnlessEqual(util.format_rating(0.5), "11100")
+        self.failUnlessEqual(util.format_rating(0.51), "11100")
         # Test rounding down...
         self.failUnlessEqual(util.format_rating(0.6), "11100")
         # Test rounding up...
-        self.failUnlessEqual(util.format_rating(0.9), "11111")
+        self.failUnlessEqual(util.format_rating(0.91), "11111")
         # You never know...
         self.failUnlessEqual(util.format_rating(3.0), "11111")
         self.failUnlessEqual(util.format_rating(-0.5), "00000")
@@ -169,12 +202,12 @@ class Tpango(TestCase):
 
 class Tre_esc(TestCase):
     def test_empty(self):
-        self.failUnlessEqual(re_escape(""), "")
-        self.assertTrue(isinstance(re_escape(""), bytes))
+        self.failUnlessEqual(re_escape(b""), b"")
+        self.assertTrue(isinstance(re_escape(b""), bytes))
 
     def test_empty_unicode(self):
         self.failUnlessEqual(re_escape(u""), u"")
-        self.assertTrue(isinstance(re_escape(u""), unicode))
+        self.assertTrue(isinstance(re_escape(u""), text_type))
 
     def test_safe(self):
         self.failUnlessEqual(re_escape("fo o"), "fo o")
@@ -189,22 +222,22 @@ class Tre_esc(TestCase):
 
 class Tdecode(TestCase):
     def test_empty(self):
-        self.failUnlessEqual(decode(""), "")
+        self.failUnlessEqual(decode(b""), "")
 
     def test_safe(self):
-        self.failUnlessEqual(decode("foo!"), "foo!")
+        self.failUnlessEqual(decode(b"foo!"), "foo!")
 
     def test_invalid(self):
         self.failUnlessEqual(
-            decode("fo\xde"), u'fo\ufffd [Invalid Encoding]')
+            decode(b"fo\xde"), u'fo\ufffd [Invalid Encoding]')
 
 
 class Tencode(TestCase):
     def test_empty(self):
-        self.failUnlessEqual(encode(""), "")
+        self.failUnlessEqual(encode(""), b"")
 
     def test_unicode(self):
-        self.failUnlessEqual(encode(u"abcde"), "abcde")
+        self.failUnlessEqual(encode(u"abcde"), b"abcde")
 
 
 class Tcapitalize(TestCase):
@@ -340,7 +373,9 @@ class Tdate_key(TestCase):
 class Tformat_size(TestCase):
     def t_dict(self, d):
         for key, value in d.items():
-            self.failUnlessEqual(util.format_size(key), value)
+            formatted = util.format_size(key)
+            self.failUnlessEqual(formatted, value)
+            assert isinstance(formatted, text_type)
 
     def test_bytes(self):
         self.t_dict({0: "0 B", 1: "1 B", 1023: "1023 B"})
@@ -613,6 +648,35 @@ class Tformat_time_long(TestCase):
         s.assertEquals(len(f_t_l(2 ** 31).split(", ")), 2)
 
 
+class TFormatTimePreferred(TestCase):
+
+    def test_default_setting_is_standard(s):
+        s.assertEquals(config.DURATION.format, DurationFormat.STANDARD)
+
+    def test_raw_config_is_standard(s):
+        s.assertEquals(config.get('display', 'duration_format'),
+                       DurationFormat.STANDARD)
+
+    def test_acts_like_long(s):
+        s._fuzz_loop(format_time_preferred, f_t_l)
+
+    def _fuzz_loop(s, f, f2):
+        x = 1
+        while x < 100000000:
+            s.assertEquals(f(x), f2(x))
+            x = x * 3 / 2 + 1
+
+    def test_acts_like_display(s):
+        def fmt_numeric(x):
+            return format_time_preferred(x, DurationFormat.NUMERIC)
+        s._fuzz_loop(fmt_numeric, format_time_display)
+
+    def test_seconds(s):
+        def fmt_seconds(x):
+            return format_time_preferred(x, DurationFormat.SECONDS)
+        s._fuzz_loop(fmt_seconds, format_time_seconds)
+
+
 class Tspawn(TestCase):
 
     def test_simple(self):
@@ -624,11 +688,6 @@ class Tspawn(TestCase):
         from gi.repository import GLib
         self.failUnlessRaises(GLib.GError, util.spawn, ["not a command"])
 
-    def test_types(self):
-        if is_win:
-            return
-        self.failUnlessRaises(TypeError, util.spawn, [u"ls"])
-
     def test_get_output(self):
         if is_win:
             return
@@ -663,20 +722,20 @@ class Txdg_dirs(TestCase):
         xdg_get_user_dirs()
 
     def test_parse_xdg_user_dirs(self):
-        data = '# foo\nBLA="$HOME/blah"\n'
+        data = b'# foo\nBLA="$HOME/blah"\n'
         vars_ = parse_xdg_user_dirs(data)
-        self.assertTrue("BLA" in vars_)
+        self.assertTrue(b"BLA" in vars_)
         expected = os.path.join(environ.get("HOME", ""), "blah")
-        self.assertEqual(vars_["BLA"], expected)
+        self.assertEqual(vars_[b"BLA"], expected)
 
-        vars_ = parse_xdg_user_dirs('BLA="$HOME/"')
-        self.assertTrue("BLA" in vars_)
-        self.assertEqual(vars_["BLA"], environ.get("HOME", ""))
+        vars_ = parse_xdg_user_dirs(b'BLA="$HOME/"')
+        self.assertTrue(b"BLA" in vars_)
+        self.assertEqual(vars_[b"BLA"], environ.get("HOME", ""))
 
         # some invalid
-        self.assertFalse(parse_xdg_user_dirs("foo"))
-        self.assertFalse(parse_xdg_user_dirs("foo=foo bar"))
-        self.assertFalse(parse_xdg_user_dirs("foo='foo"))
+        self.assertFalse(parse_xdg_user_dirs(b"foo"))
+        self.assertFalse(parse_xdg_user_dirs(b"foo=foo bar"))
+        self.assertFalse(parse_xdg_user_dirs(b"foo='foo"))
 
     def test_on_windows(self):
         self.assertTrue(xdg_get_system_data_dirs())
@@ -695,7 +754,7 @@ class Tlibrary(TestCase):
     def test_basic(self):
         self.failIf(get_scan_dirs())
         if os.name == "nt":
-            set_scan_dirs([u"C:\\foo", u"D:\\bar", ""])
+            set_scan_dirs([u"C:\\foo", u"D:\\bar", u""])
             self.failUnlessEqual(get_scan_dirs(), [u"C:\\foo", u"D:\\bar"])
         else:
             set_scan_dirs(["foo", "bar", ""])
@@ -722,10 +781,12 @@ class TNormalizePath(TestCase):
         os.close(f)
         path = norm(path)
 
+        link_dir = mkdtemp()
         link = None
         if not is_win:
-            link = str(uuid.uuid4())
+            link = os.path.join(link_dir, str(uuid.uuid4()))
             os.symlink(path, link)
+
         try:
             self.failUnlessEqual(norm(path, canonicalise=True), path)
             self.failUnlessEqual(norm(os.path.join(path, "foo", ".."), True),
@@ -741,6 +802,7 @@ class TNormalizePath(TestCase):
             if link:
                 os.remove(link)
             os.remove(path)
+            os.rmdir(link_dir)
 
 
 class Tescape_filename(TestCase):
@@ -748,12 +810,12 @@ class Tescape_filename(TestCase):
     def test_str(self):
         result = escape_filename("\x00\x01")
         self.assertEqual(result, "%00%01")
-        self.assertTrue(is_fsnative(result))
+        self.assertTrue(isinstance(result, fsnative))
 
     def test_unicode(self):
         result = escape_filename(u'abc\xe4')
         self.assertEqual(result, "abc%C3%A4")
-        self.assertTrue(is_fsnative(result))
+        self.assertTrue(isinstance(result, fsnative))
 
 
 @skipIf(is_win, "not on Windows")
@@ -782,15 +844,15 @@ class Tload_library(TestCase):
 class Tstrip_win32_incompat_from_path(TestCase):
 
     def test_types(self):
-        v = strip_win32_incompat_from_path("")
-        self.assertTrue(isinstance(v, bytes))
-        v = strip_win32_incompat_from_path("foo")
-        self.assertTrue(isinstance(v, bytes))
+        v = strip_win32_incompat_from_path(fsnative(u""))
+        self.assertTrue(isinstance(v, fsnative))
+        v = strip_win32_incompat_from_path(fsnative(u"foo"))
+        self.assertTrue(isinstance(v, fsnative))
 
         v = strip_win32_incompat_from_path(u"")
-        self.assertTrue(isinstance(v, unicode))
+        self.assertTrue(isinstance(v, text_type))
         v = strip_win32_incompat_from_path(u"foo")
-        self.assertTrue(isinstance(v, unicode))
+        self.assertTrue(isinstance(v, text_type))
 
     def test_basic(self):
         if is_win:
@@ -805,14 +867,14 @@ class TPathHandling(TestCase):
 
     def test_main(self):
         v = fsnative(u"foo")
-        self.assertTrue(is_fsnative(v))
+        self.assertTrue(isinstance(v, fsnative))
 
-        v2 = glib2fsnative(fsnative2glib(v))
-        self.assertTrue(is_fsnative(v2))
+        v2 = glib2fsn(fsn2glib(v))
+        self.assertTrue(isinstance(v2, fsnative))
         self.assertEqual(v, v2)
 
-        v3 = bytes2fsnative(fsnative2bytes(v))
-        self.assertTrue(is_fsnative(v3))
+        v3 = bytes2fsn(fsn2bytes(v, "utf-8"), "utf-8")
+        self.assertTrue(isinstance(v3, fsnative))
         self.assertEqual(v, v3)
 
 
@@ -821,7 +883,7 @@ class Tget_temp_cover_file(TestCase):
     def test_main(self):
         fobj = get_temp_cover_file(b"foobar")
         try:
-            self.assertTrue(is_fsnative(fobj.name))
+            self.assertTrue(isinstance(fobj.name, fsnative))
         finally:
             fobj.close()
 
@@ -851,37 +913,37 @@ class Tsplit_escape(TestCase):
             self.assertEqual(split_escape(*inargs), out)
 
     def test_types(self):
-        parts = split_escape("\xff:\xff", ":")
-        self.assertEqual(parts, ["\xff", "\xff"])
+        parts = split_escape(b"\xff:\xff", b":")
+        self.assertEqual(parts, [b"\xff", b"\xff"])
         self.assertTrue(isinstance(parts[0], bytes))
 
         parts = split_escape(u"a:b", u":")
         self.assertEqual(parts, [u"a", u"b"])
-        self.assertTrue(all(isinstance(p, unicode) for p in parts))
+        self.assertTrue(all(isinstance(p, text_type) for p in parts))
 
         parts = split_escape(u"", u":")
         self.assertEqual(parts, [u""])
-        self.assertTrue(all(isinstance(p, unicode) for p in parts))
+        self.assertTrue(all(isinstance(p, text_type) for p in parts))
 
         parts = split_escape(u":", u":")
         self.assertEqual(parts, [u"", u""])
-        self.assertTrue(all(isinstance(p, unicode) for p in parts))
+        self.assertTrue(all(isinstance(p, text_type) for p in parts))
 
     def test_join_escape_types(self):
-        self.assertEqual(join_escape([], ":"), "")
-        self.assertTrue(isinstance(join_escape([], ":"), bytes))
-        self.assertTrue(isinstance(join_escape([], u":"), unicode))
-        self.assertEqual(join_escape(["\xff", "\xff"], ":"), "\xff:\xff")
+        self.assertEqual(join_escape([], b":"), b"")
+        self.assertTrue(isinstance(join_escape([], b":"), bytes))
+        self.assertTrue(isinstance(join_escape([], u":"), text_type))
+        self.assertEqual(join_escape([b"\xff", b"\xff"], b":"), b"\xff:\xff")
         self.assertEqual(join_escape([u'\xe4', u'\xe4'], ":"), u'\xe4:\xe4')
 
     def test_join_escape(self):
-        self.assertEqual(join_escape([":"], ":"), "\\:")
-        self.assertEqual(join_escape(["\\:", ":"], ":"), "\\\\\\::\\:")
+        self.assertEqual(join_escape([b":"], b":"), b"\\:")
+        self.assertEqual(join_escape([b"\\:", b":"], b":"), b"\\\\\\::\\:")
 
     def test_roundtrip(self):
-        values = ["\\:", ":"]
-        joined = join_escape(values, ":")
-        self.assertEqual(split_escape(joined, ":"), values)
+        values = [b"\\:", b":"]
+        joined = join_escape(values, b":")
+        self.assertEqual(split_escape(joined, b":"), values)
 
 
 class TMainRunner(TestCase):
@@ -1035,32 +1097,47 @@ class Tcached_property(TestCase):
         self.assertRaises(AssertionError, define_class)
 
 
+@util.enum
+class Foo(str):
+    FOO = "blah"
+    BAR = "not foo"
+    BAZ = "baz!"
+
+
 class Tenum(TestCase):
 
     def test_main(self):
 
         @util.enum
-        class Foo(int):
+        class IntFoo(int):
             FOO = 0
             BAR = 1
 
-        self.assertTrue(issubclass(Foo, int))
-        self.assertTrue(isinstance(Foo.BAR, Foo))
-        self.assertTrue(isinstance(Foo.FOO, Foo))
-        self.assertEqual(Foo.FOO, 0)
-        self.assertEqual(Foo.BAR, 1)
+        self.assertTrue(issubclass(IntFoo, int))
+        self.assertTrue(isinstance(IntFoo.BAR, IntFoo))
+        self.assertTrue(isinstance(IntFoo.FOO, IntFoo))
+        self.assertEqual(IntFoo.FOO, 0)
+        self.assertEqual(IntFoo.BAR, 1)
 
     def test_str(self):
-        @util.enum
-        class Foo(str):
-            FOO = "blah"
-            BAR = "foo"
-
         self.assertTrue(issubclass(Foo, str))
         self.assertTrue(isinstance(Foo.BAR, Foo))
         self.assertEqual(Foo.FOO, "blah")
         self.assertEqual(repr(Foo.BAR), "Foo.BAR")
 
+    def test_values(self):
+        self.assertEqual(Foo.values, {Foo.FOO, Foo.BAR, Foo.BAZ})
+
+    def test_value_of(self):
+        self.assertEqual(Foo.value_of("blah"), Foo.FOO)
+        self.assertEqual(Foo.value_of("baz!"), Foo.BAZ)
+
+    def test_value_of_raises_for_unknown(self):
+        self.assertRaises(ValueError, Foo.value_of, "??")
+
+    def test_value_of_uses_default(self):
+        self.assertEquals(Foo.value_of("??", "default"), "default")
+
 
 class Tlist_unique(TestCase):
 
@@ -1092,7 +1169,7 @@ class Tenviron(TestCase):
     def test_main(self):
         for v in util.environ.values():
             if os.name == "nt":
-                self.assertTrue(isinstance(v, unicode))
+                self.assertTrue(isinstance(v, text_type))
             else:
                 self.assertTrue(isinstance(v, str))
 
@@ -1110,12 +1187,12 @@ class Tget_module_dir(TestCase):
 
     def test_self(self):
         path = util.get_module_dir()
-        self.assertTrue(is_fsnative(path))
+        self.assertTrue(isinstance(path, fsnative))
         self.assertTrue(os.path.exists(path))
 
     def test_other(self):
         path = util.get_module_dir(util)
-        self.assertTrue(is_fsnative(path))
+        self.assertTrue(isinstance(path, fsnative))
         self.assertTrue(os.path.exists(path))
 
 
@@ -1124,7 +1201,7 @@ class Tget_ca_file(TestCase):
     def test_main(self):
         path = util.get_ca_file()
         if path is not None:
-            self.assertTrue(is_fsnative(path))
+            self.assertTrue(isinstance(path, fsnative))
             self.assertTrue(os.path.exists(path))
 
 
@@ -1137,6 +1214,24 @@ class Tprint_exc(TestCase):
             with capture_output():
                 print_exc()
 
+    def test_pass_exc_info(self):
+        try:
+            1 / 0
+        except:
+            with capture_output():
+                print_exc(exc_info=sys.exc_info(), context="foo")
+
+
+class TPrintHandler(TestCase):
+
+    def test_main(self):
+        handler = PrintHandler()
+        for level in range(0, 70, 10):
+            record = logging.LogRecord(
+                "foo", level, "a.py", 45, "bar", None, None)
+            with capture_output():
+                handler.handle(record)
+
 
 class Tformat_exception(TestCase):
 
@@ -1158,7 +1253,7 @@ class Textract_tb(TestCase):
             result = extract_tb(sys.exc_info()[2])
             self.assertTrue(isinstance(result, list))
             for fn, l, fu, text in result:
-                self.assertTrue(is_fsnative(fn))
+                self.assertTrue(isinstance(fn, fsnative))
                 self.assertTrue(isinstance(l, int))
                 self.assertTrue(isinstance(fu, text_type))
                 self.assertTrue(isinstance(text, text_type))
