@@ -5,6 +5,10 @@
 
 import shutil
 import os
+from collections import defaultdict
+
+from senf import fsnative
+
 from quodlibet import config
 
 from tests import TestCase, mkdtemp
@@ -14,7 +18,7 @@ from quodlibet.util.collection import Album, Playlist, avg, bayesian_average, \
     FileBackedPlaylist
 from quodlibet.library.libraries import FileLibrary
 from quodlibet.util import format_rating
-from quodlibet.util.path import fsnative
+from quodlibet.compat import long
 
 config.RATINGS = config.HardCodedRatingsPrefs()
 
@@ -70,7 +74,7 @@ class TAlbum(TestCase):
         album = Album(songs[0])
         album.songs = set(songs)
 
-        s.failUnlessEqual(album.comma("~artist~dummy"), "a - e, d")
+        s.failUnlessEqual(album.comma("~artist~dummy"), "a - d, e")
 
     def test_tied_num_tags(s):
         songs = [
@@ -83,9 +87,9 @@ class TAlbum(TestCase):
         album.songs = set(songs)
 
         s.failUnlessEqual(album.comma("~foo~~s~~~"), "")
-        s.failUnlessEqual(album.comma("~#length~dummy"), "12 - e, d")
-        s.failUnlessEqual(album.comma("~#rating~dummy"), "0.50 - e, d")
-        s.failUnlessEqual(album.comma("~#length:sum~dummy"), "12 - e, d")
+        s.failUnlessEqual(album.comma("~#length~dummy"), "12 - d, e")
+        s.failUnlessEqual(album.comma("~#rating~dummy"), "0.50 - d, e")
+        s.failUnlessEqual(album.comma("~#length:sum~dummy"), "12 - d, e")
         s.failUnlessEqual(album.comma("~#dummy2"), 5)
         s.failUnlessEqual(album.comma("~#dummy3"), "")
 
@@ -230,7 +234,6 @@ class TAlbum(TestCase):
         failUnlessEq(album("~performer", "x"), song("~performer", "x"))
         failUnlessEq(album("~performersort", "x"), song("~performersort", "x"))
 
-        failUnlessEq(album("~cover", "x"), song("~cover", "x"))
         failUnlessEq(album("~rating", "x"), song("~rating", "x"))
 
         for p in PEOPLE:
@@ -258,7 +261,7 @@ class TAlbum(TestCase):
         config.quit()
 
 
-class TestPlaylistResource(object):
+class MockPlaylistResource(object):
     def __init__(self, pl):
         self.pl = pl
 
@@ -275,11 +278,34 @@ class TPlaylist(TestCase):
         Fakesong({"~#length": 7, "dummy": "d\ne", "discnumber": "2"})
     ]
 
+    class FakeLib(object):
+
+        def __init__(self):
+            self.reset()
+
+        def emit(self, name, songs):
+            self.emitted[name].extend(songs)
+
+        def masked(self, songs):
+            return False
+
+        def reset(self):
+            self.emitted = defaultdict(list)
+
+        @property
+        def changed(self):
+            return self.emitted.get('changed', [])
+
+    FAKE_LIB = FakeLib()
+
+    def setUp(self):
+        self.FAKE_LIB.reset()
+
     def pl(self, name, lib=None):
         return Playlist(name, lib)
 
-    def wrap(self, name, lib=None):
-        return TestPlaylistResource(self.pl(name, lib))
+    def wrap(self, name, lib=FAKE_LIB):
+        return MockPlaylistResource(self.pl(name, lib))
 
     def test_equality(s):
         pl = s.pl("playlist")
@@ -402,6 +428,23 @@ class TPlaylist(TestCase):
             s.failUnlessEqual(NUMERIC_SONGS[1:2], pl[1:2])
             s.failUnless(NUMERIC_SONGS[1] in pl)
 
+    def test_extend_signals(s):
+        with s.wrap("playlist") as pl:
+            pl.extend(NUMERIC_SONGS)
+            s.failUnlessEqual(s.FAKE_LIB.changed, NUMERIC_SONGS)
+
+    def test_append_signals(s):
+        with s.wrap("playlist") as pl:
+            song = NUMERIC_SONGS[0]
+            pl.append(song)
+            s.failUnlessEqual(s.FAKE_LIB.changed, [song])
+
+    def test_clear_signals(s):
+        with s.wrap("playlist") as pl:
+            pl.extend(NUMERIC_SONGS)
+            pl.clear()
+            s.failUnlessEqual(s.FAKE_LIB.changed, NUMERIC_SONGS * 2)
+
     def test_make(self):
         with self.wrap("Does not exist") as pl:
             self.failUnlessEqual(0, len(pl))
@@ -409,8 +452,11 @@ class TPlaylist(TestCase):
 
     def test_rename_working(self):
         with self.wrap("Foobar") as pl:
+            assert pl.name == "Foobar"
             pl.rename("Foo Quuxly")
-            self.failUnlessEqual(pl.name, "Foo Quuxly")
+            assert pl.name == "Foo Quuxly"
+            # Rename should not fire signals
+            self.failIf(self.FAKE_LIB.changed)
 
     def test_rename_nothing(self):
         with self.wrap("Foobar") as pl:
@@ -457,10 +503,31 @@ class TPlaylist(TestCase):
                             ("Playlist has un-detected duplicates: %s "
                              % "\n".join([str(s) for s in pl._list])))
 
+    def test_remove_leaving_duplicates(self):
+        with self.wrap("playlist") as pl:
+            pl.extend(self.TWO_SONGS)
+            [first, second] = self.TWO_SONGS
+            pl.extend(NUMERIC_SONGS + self.TWO_SONGS)
+            self.failUnlessEqual(len(self.FAKE_LIB.changed), 7)
+            self.FAKE_LIB.reset()
+            pl.remove_songs(self.TWO_SONGS, leave_dupes=True)
+            self.failUnless(first in pl)
+            self.failUnless(second in pl)
+            self.failIf(len(self.FAKE_LIB.changed))
+
+    def test_remove_fully(self):
+        with self.wrap("playlist") as pl:
+            pl.extend(self.TWO_SONGS * 2)
+            self.FAKE_LIB.reset()
+            pl.remove_songs(self.TWO_SONGS, leave_dupes=False)
+            self.failIf(len(pl))
+            self.failUnlessEqual(self.FAKE_LIB.changed, self.TWO_SONGS)
+
 
 class TFileBackedPlaylist(TPlaylist):
 
     def setUp(self):
+        super(TFileBackedPlaylist, self).setUp()
         self.temp = mkdtemp()
         self.temp2 = mkdtemp()
 
