@@ -5,14 +5,18 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import sys
+import os
+
+from senf import fsnative, expanduser
+
 from quodlibet import config
-from quodlibet.util.library import split_scan_dirs, set_scan_dirs
-from quodlibet.util.path import fsnative
+from quodlibet.util.library import split_scan_dirs, set_scan_dirs, \
+    get_exclude_dirs, get_scan_dirs
+from quodlibet.util import is_windows
+from quodlibet.util.path import get_home_dir, unexpand
 
 from tests import TestCase
 
-ON_WINDOWS = sys.platform == "win32"
 
 STANDARD_PATH = fsnative(u"/home/user/Music")
 OTHER_PATH = fsnative(u"/opt/party")
@@ -22,30 +26,39 @@ GVFS_PATH_ESCAPED = fsnative(u"/run/user/12345/gvfs/smb-share"
                               "\:server=12.23.34.45,share=/foo/bar/baz/path")
 
 
-class Tsplit_scan_dirs(TestCase):
+class Tlibrary_utils(TestCase):
+
     def test_basic(self):
-        if ON_WINDOWS:
-            res = split_scan_dirs(r":Z:\foo:C:/windows:")
-            self.assertEquals(res, [r"Z:\foo", "C:/windows"])
+        if is_windows():
+            res = split_scan_dirs(u":Z:\\foo:C:/windows:")
+            self.assertEquals(res, [u"Z:\\foo", u"C:/windows"])
         else:
             res = split_scan_dirs(":%s:%s:" % (STANDARD_PATH, OTHER_PATH))
             self.assertEquals(res, [STANDARD_PATH, OTHER_PATH])
 
     def test_colon_paths(self):
-        if not ON_WINDOWS:
+        if not is_windows():
             res = split_scan_dirs(
                 ":%s:%s" % (STANDARD_PATH, GVFS_PATH_ESCAPED))
             self.assertEquals(res, [STANDARD_PATH, GVFS_PATH])
 
+    def test_get_exclude_dirs(self):
+        some_path = os.path.join(unexpand(get_home_dir()), "foo")
+        config.set('library', 'exclude', some_path)
+        assert expanduser(some_path) in get_exclude_dirs()
+
+        assert all([isinstance(p, fsnative) for p in get_exclude_dirs()])
+
+    def test_get_scan_dirs(self):
+        some_path = os.path.join(unexpand(get_home_dir()), "foo")
+        config.set('settings', 'scan', some_path)
+        assert expanduser(some_path) in get_scan_dirs()
+
+        assert all([isinstance(p, fsnative) for p in get_scan_dirs()])
+
 
 class Tset_scan_dirs(TestCase):
 
-    def setUp(self):
-        config.init()
-
-    def tearDown(self):
-        config.quit()
-
     @property
     def scan_dirs(self):
         return config.get('settings', 'scan')
@@ -65,5 +78,5 @@ class Tset_scan_dirs(TestCase):
 
     def test_set_scan_dirs_colons(self):
         set_scan_dirs([STANDARD_PATH, GVFS_PATH])
-        expected = GVFS_PATH if ON_WINDOWS else GVFS_PATH_ESCAPED
+        expected = GVFS_PATH if is_windows() else GVFS_PATH_ESCAPED
         self.assertEqual(self.scan_dirs, "%s:%s" % (STANDARD_PATH, expected))
