@@ -7,6 +7,7 @@ from tests import TestCase
 
 from quodlibet.util.massagers import Massager, validate, is_valid, \
     error_message, get_options, ValidationError
+from quodlibet.compat import text_type
 
 
 class TMassagers(TestCase):
@@ -15,7 +16,7 @@ class TMassagers(TestCase):
         for val in values:
             self.assertTrue(massager.is_valid(val))
             self.assertTrue(
-                isinstance(massager.validate(unicode(val)), unicode))
+                isinstance(massager.validate(text_type(val)), text_type))
 
     def invalidate(self, key, values):
         for val in values:
@@ -26,7 +27,7 @@ class TMassagers(TestCase):
         for value, normed in equivs.items():
             self.assertEqual(normed, massager.validate(value))
             self.assertTrue(
-                isinstance(massager.validate(unicode(value)), unicode))
+                isinstance(massager.validate(text_type(value)), text_type))
 
     def test_validate_helper(self):
         self.assertEqual(validate("foo", "bar"), "bar")
