@@ -7,8 +7,10 @@ from tests import TestCase
 
 import os
 
+from senf import fsnative
+
 from quodlibet.util.tagsfrompath import TagsFromPattern
-from quodlibet.util.path import fsnative
+from quodlibet.compat import iteritems
 
 
 class TTagsFromPattern(TestCase):
@@ -33,7 +35,7 @@ class TTagsFromPattern(TestCase):
         from quodlibet import formats
         pat = TagsFromPattern('<tracknumber>. <title>')
         tracktitle = {'tracknumber': '01', 'title': 'Title'}
-        for ext, kind in formats.loaders.iteritems():
+        for ext, kind in iteritems(formats.loaders):
             f = formats._audio.AudioFile()
             if not isinstance(kind, type):
                 continue
