@@ -4,9 +4,10 @@
 # published by the Free Software Foundation
 
 from quodlibet.util.path import mtime
-from tests import TestCase, NamedTemporaryFile
+from tests import TestCase, NamedTemporaryFile, get_data_path
 
 from gi.repository import GdkPixbuf
+from senf import fsn2uri, fsnative
 
 import os
 
@@ -16,10 +17,10 @@ except ImportError:
     import md5 as hash
 
 from quodlibet.util import thumbnails
-from quodlibet.util.path import pathname2url, is_fsnative, getcwd
 
 
 class TThumb(TestCase):
+
     def setUp(s):
         s.wide = GdkPixbuf.Pixbuf.new(
             GdkPixbuf.Colorspace.RGB, True, 8, 150, 10)
@@ -27,13 +28,12 @@ class TThumb(TestCase):
             GdkPixbuf.Colorspace.RGB, True, 8, 10, 100)
         s.small = GdkPixbuf.Pixbuf.new(
             GdkPixbuf.Colorspace.RGB, True, 8, 10, 20)
-        s.filename = os.path.join(getcwd(), "test_thumbnail.png")
-        s.wide.savev(s.filename, "png", [], [])
+        s.filename = get_data_path("test.png")
 
     def tearDown(self):
         p1 = thumbnails.get_cache_info(self.filename, (10, 10))[0]
         p2 = thumbnails.get_cache_info(self.filename, (1000, 1000))[0]
-        for path in [p1, p2, self.filename]:
+        for path in [p1, p2]:
             try:
                 os.remove(path)
             except OSError:
@@ -41,7 +41,7 @@ class TThumb(TestCase):
 
     def test_get_thumbnail_folder(self):
         path = thumbnails.get_thumbnail_folder()
-        self.assertTrue(is_fsnative(path))
+        self.assertTrue(isinstance(path, fsnative))
 
     def test_thumb_from_file(self):
         with open(self.filename, "rb") as h:
@@ -92,11 +92,11 @@ class TThumb(TestCase):
 
         #check for right scaling
         s.failUnless(thumb)
-        s.failUnlessEqual((thumb.get_width(), thumb.get_height()), (50, 3))
+        s.failUnlessEqual((thumb.get_width(), thumb.get_height()), (50, 25))
 
         #test the thumbnail filename
-        uri = "file://" + pathname2url(s.filename)
-        name = hash.md5(uri).hexdigest() + ".png"
+        uri = fsn2uri(s.filename)
+        name = hash.md5(uri.encode("ascii")).hexdigest() + ".png"
 
         path = thumbnails.get_thumbnail_folder()
         path = os.path.join(path, "normal", name)
