@@ -8,10 +8,10 @@
 import os
 import sys
 
-from tests import TestCase
-
 from quodlibet import config
-from quodlibet.util.trash import use_trash
+from quodlibet.util import is_linux
+from quodlibet.util.trash import use_trash, trash
+from tests import TestCase, mkstemp, skipUnless
 
 
 class Ttrash(TestCase):
@@ -63,3 +63,12 @@ class Ttrash(TestCase):
         finally:
             os.name = old_os_name
             sys.platform = old_sys_platform
+
+    @skipUnless(is_linux(), reason="Trash only supported for Linux")
+    def test_trash_removes_file(self):
+        filename = mkstemp()[1]
+        with open(filename, "w") as f:
+            f.write("\n")
+        self.failUnless(os.path.exists(filename))
+        trash(filename)
+        self.failIf(os.path.exists(filename))
