@@ -6,13 +6,13 @@
 # published by the Free Software Foundation
 
 import os
-import sys
 
-from tests import TestCase, DATA_DIR, skipUnless
+from tests import TestCase, get_data_path, skipUnless
 
 from quodlibet.util.path import normalize_path
 from quodlibet.util import is_wine
-from quodlibet import windows
+from quodlibet.util import windows
+from quodlibet.compat import text_type
 
 
 @skipUnless(os.name == "nt", "Wrong platform")
@@ -20,34 +20,34 @@ class TWindows(TestCase):
 
     def test_dir_funcs(self):
         d = windows.get_personal_dir()
-        self.assertTrue(d is None or isinstance(d, unicode))
+        self.assertTrue(d is None or isinstance(d, text_type))
 
         d = windows.get_appdate_dir()
-        self.assertTrue(d is None or isinstance(d, unicode))
+        self.assertTrue(d is None or isinstance(d, text_type))
 
         d = windows.get_desktop_dir()
-        self.assertTrue(d is None or isinstance(d, unicode))
+        self.assertTrue(d is None or isinstance(d, text_type))
 
         d = windows.get_music_dir()
-        self.assertTrue(d is None or isinstance(d, unicode))
+        self.assertTrue(d is None or isinstance(d, text_type))
 
         d = windows.get_profile_dir()
-        self.assertTrue(d is None or isinstance(d, unicode))
+        self.assertTrue(d is None or isinstance(d, text_type))
 
         d = windows.get_links_dir()
-        self.assertTrue(d is None or isinstance(d, unicode))
+        self.assertTrue(d is None or isinstance(d, text_type))
 
     def test_get_link_target(self):
-        path = os.path.join(DATA_DIR, "test.lnk")
+        path = get_data_path("test.lnk")
         d = windows.get_link_target(path)
-        self.assertTrue(isinstance(d, unicode))
+        self.assertTrue(isinstance(d, text_type))
         self.assertEqual(
             normalize_path(d), normalize_path(u"C:\Windows\explorer.exe"))
 
     def test_get_link_target_unicode(self):
-        path = os.path.join(DATA_DIR, "test2.lnk")
+        path = get_data_path("test2.lnk")
         d = windows.get_link_target(path)
-        self.assertTrue(isinstance(d, unicode))
+        self.assertTrue(isinstance(d, text_type))
         if is_wine():
             # wine doesn't support unicode paths here..
             self.assertEqual(os.path.basename(d), u"\xe1??.txt")
@@ -57,29 +57,3 @@ class TWindows(TestCase):
     def test_get_link_target_non_exist(self):
         with self.assertRaises(WindowsError):
             windows.get_link_target(u"nopenope.lnk")
-
-    def test_environ(self):
-        env = windows.WindowsEnviron()
-        len_ = len(env)
-        env[u"FOO"] = u"bar"
-        self.assertEqual(len(env), len_ + 1)
-        self.assertEqual(env.get(u"FOO"), u"bar")
-        self.assertTrue("FOO" in repr(env))
-        self.assertEqual(len(list(env)), len(env))
-        del env[u"FOO"]
-
-    def test_environ_ascii(self):
-        env = windows.WindowsEnviron()
-        env["FOO"] = u"bar"
-        env["FOO"]
-        del env["FOO"]
-
-
-@skipUnless(os.name == "nt", "Wrong platform")
-class Tget_win32_unicode_argv(TestCase):
-
-    def test_main(self):
-        newargv = windows.get_win32_unicode_argv()
-        self.assertEqual(len(sys.argv), len(newargv))
-        if newargv:
-            self.assertTrue(isinstance(newargv[0], unicode))
